@@ -79,6 +79,8 @@ private:
     friend class REF_CHECKVNNode;
     friend class REF_EXPRVNNode;
     friend class ParameterVNNode;
+    friend class WRAPPED_FUNNode;
+    friend class WRAPPED_PRONode;
 
 public: 
 
@@ -446,13 +448,28 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         if( it == objHeap.end()) throw HeapException();
         return it->second.get();
     }
-    static DStructGDL*& GetObjHeap( DObj ID, ObjHeapT::iterator& it)
+
+    // for overload functions
+    static DSubUD* GetObjHeapOperator( DObj ID, int opIx)
     {
-//         ObjHeapT::iterator it=objHeap.find( ID);
-        it=objHeap.find( ID);
-        if( it == objHeap.end()) throw HeapException();
-        return it->second.get();
+        if( ID == 0) return NULL;
+        ObjHeapT::iterator it=objHeap.find( ID);
+        if( it == objHeap.end()) return NULL;
+        return it->second.get()->Desc()->GetOperator( opIx);
     }
+    // static DStructGDL* GetObjHeapNoThrow( DObj ID)
+    // {
+    //     ObjHeapT::iterator it=objHeap.find( ID);
+    //     if( it == objHeap.end()) return NULL;
+    //     return it->second.get();
+    // }
+//     static DStructGDL*& GetObjHeap( DObj ID, ObjHeapT::iterator& it)
+//     {
+// //         ObjHeapT::iterator it=objHeap.find( ID);
+//         it=objHeap.find( ID);
+//         if( it == objHeap.end()) throw HeapException();
+//         return it->second.get();
+//     }
 
     static bool PtrValid( DPtr ID)
     {
@@ -696,7 +713,8 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
     }
 
     static EnvStackT& CallStack() { return callStack;} // the callstack
-    static EnvBaseT*  CallStackBack() { return callStack.back();} 
+//    static EnvBaseT*  CallStackBack() { return callStack.back();} 
+    static EnvUDT*  CallStackBack() { return callStack.back();} 
     
     std::string GetClearActualLine()
     {
@@ -806,6 +824,9 @@ public:
 		EnvBaseT* actEnv
 	);
 	public: ArrayIndexListT*  arrayindex_list_noassoc(ProgNodeP _t);
+	public: void arrayindex_list_overload(ProgNodeP _t,
+		IxExprListT& indexList
+	);
 public:
 	antlr::RefAST getAST()
 	{
@@ -818,10 +839,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 232;
+	static const int NUM_TOKENS = 235;
 #else
 	enum {
-		NUM_TOKENS = 232
+		NUM_TOKENS = 235
 	};
 #endif
 	
