@@ -42,6 +42,7 @@ class CUSTOM_API GDLTreeParser : public antlr::TreeParser, public GDLTreeParserT
             lT == FCALL_LIB ||
 //            lT == FCALL_LIB_N_ELEMENTS ||
             lT == FCALL_LIB_RETNEW || 
+            lT == FCALL_LIB_DIRECT || 
             lT == MFCALL_LIB || 
             lT == MFCALL_LIB_RETNEW || 
             lT == MFCALL_PARENT_LIB ||
@@ -104,6 +105,7 @@ class CUSTOM_API GDLTreeParser : public antlr::TreeParser, public GDLTreeParserT
     }
 
   bool ActiveProCompiled() const { return comp.ActiveProCompiled();} 
+  int NCompileErrors() const { return comp.NCompileErrors();} 
 public:
 	GDLTreeParser();
 	static void initializeASTFactory( antlr::ASTFactory& factory );
@@ -193,10 +195,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 232;
+	static const int NUM_TOKENS = 235;
 #else
 	enum {
-		NUM_TOKENS = 232
+		NUM_TOKENS = 235
 	};
 #endif
 	
