@@ -42,7 +42,7 @@ void push_back( BaseGDL* p) { assert( sz<3*MAXRANK); eArr[ sz++] = p;}
 BaseGDL* operator[]( SizeT i) const { assert( i<3*MAXRANK && i<sz); return eArr[i];}
 SizeT size() const { return sz;}
 void Cleanup() { for( int i=0; i<sz; ++i) delete eArr[i]; sz=0;}
-IxExprListT& operator=( const IxExprListT&r) { sz = r.sz; for( int i=0; i<sz; ++i) eArr[i] = r.eArr[i];}
+IxExprListT& operator=(IxExprListT&r) { sz = r.sz; for(int i=0; i<sz; ++i) eArr[i] = r.eArr[i]; return r;}
 };
 
 enum IndexType
@@ -77,6 +77,12 @@ public:
   virtual void Init( BaseGDL*, BaseGDL*) { assert( false);}
   virtual void Init( BaseGDL*, BaseGDL*, BaseGDL*) { assert( false);}
 
+  virtual bool IsRange() { return false;} // default for non-ranges
+  virtual BaseGDL* OverloadIndexNew() { assert( false);}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*) { assert( false);}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*) { assert( false);}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*, BaseGDL*) { assert( false);}
+
   virtual void Clear() {}
   virtual ~ArrayIndexT() {}
 
@@ -165,6 +171,7 @@ public:
 // 	assert( ix != NULL);
 //     return (*ix)[ i];
 //   }
+  BaseGDL* OverloadIndexNew(); 
 
   SizeT GetVarIx() const { return varIx;}
 
@@ -225,6 +232,13 @@ protected:
 public:
  IndexType Type() { return ArrayIndexScalarVPID;}
 
+ BaseGDL* OverloadIndexNew() 
+  { 
+    BaseGDL* v = varPtr->Data();
+    if( v == NULL) return NULL;
+    return v->Dup();
+  }
+
   DVar* GetVarPtr() const { return varPtr;}
 
   SizeT NParam() { return 0;} // number of parameter to Init(...)
@@ -241,6 +255,7 @@ public:
   void Init() 
   {
     sInit = varPtr->Data()->LoopIndex();
+    s = sInit;
   }
 
   void Clear()
@@ -283,13 +298,29 @@ public:
 class CArrayIndexScalar: public ArrayIndexT
 {
 private:
-  SizeT s;
+  RangeT sInit; // can be < 0 as well
+  RangeT s;
+  BaseGDL* rawData; // for overloaded object indexing
+
+  CArrayIndexScalar( const CArrayIndexScalar& c): s( c.s), sInit( c.sInit) 
+  {
+    assert( c.rawData != NULL);
+    rawData = c.rawData->Dup();
+  }
 
 public:
- IndexType Type() { return CArrayIndexScalarID;}
+  IndexType Type() { return CArrayIndexScalarID;}
+
+  BaseGDL* OverloadIndexNew() 
+  { 
+    assert( rawData != NULL);
+    return rawData->Dup();
+  }
 
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
+  BaseGDL* StealRawData() { BaseGDL* r = rawData; rawData = NULL; return r;}
+  
   bool Scalar() const { return true;}
   bool Scalar( RangeT& s_) const
   { 
@@ -305,23 +336,55 @@ public:
   RangeT GetS() { return s;}
 
   ~CArrayIndexScalar() 
-  {}
-
-  CArrayIndexScalar( BaseGDL* c)
   {
-    s = c->LoopIndex();
+    delete rawData;
   }
 
-  CArrayIndexScalar( SizeT s_): s( s_) 
+  // grabs c
+  CArrayIndexScalar( BaseGDL* c)
+  : rawData(c)
+  {
+//     if( c->Type() == GDL_STRING)
+//     {
+//       DStringGDL* cString = static_cast<DStringGDL*>(c);
+//       if( (*cString)[0] == "")
+//       {
+// 	s = 0;
+//       }
+//       else
+//       {
+// 	const char* cStart=(*cString)[0].c_str();
+// 	char* cEnd;
+// 	RangeT ix=strtol(cStart,&cEnd,10);
+// 	if( cEnd == cStart)
+// 	  {
+// 	    // in LoopIndex() just a warning is printed
+// 	    // An exception is necessary as then ArrayIndexScalar
+// 	    // is used instead (see: gdlc.tree.g: arrayindex)
+// 	    throw GDLException("Type conversion error: "
+// 	       "Unable to convert given STRING: '"+
+// 	       (*cString)[0]+"' to index.");
+// 	  }
+// 	s = ix;
+//       }
+//     }
+//     else
+      sInit = c->LoopIndex(); // non STRING throw if not allowed
+      s = sInit;
+  }
+
+  CArrayIndexScalar( RangeT s_): sInit( s_), s( s_), rawData(NULL)  
   {}
 
   ArrayIndexT* Dup() const
   {
-    return new CArrayIndexScalar( s);
+    return new CArrayIndexScalar( *this);
   }
  
   void Clear()
   {}
+  void Init()
+  {}
 
 //   // if this is used, Init was NOT called before
 //   BaseGDL* Index( BaseGDL* var, IxExprListT& ixL)
@@ -331,21 +394,26 @@ public:
 //     return var->NewIx( s);
 //   }
 
-  void Init( BaseGDL* ix_) 
-  {
-    assert( 0);
-  } 
-
   // number of iterations
   // also checks/adjusts range 
   SizeT NIter( SizeT varDim) 
   {
-//     if( s > 0 && s >= varDim)
+    if( sInit < 0)
+      s = sInit + varDim;
+    else
+      s = sInit;
+
+    if( s < 0)
+	throw GDLException(NULL,"Constant scalar subscript out of range [-i].",true,false);
     if( s >= varDim && s > 0) // varDim == 0 && s == 0 ok
-      throw GDLException(NULL,"Scalar subscript out of range [>].h2",true,false);
+	throw GDLException(NULL,"Constant scalar out of range [i].",true,false);
     return 1;
+
+//     if( s >= varDim && s > 0) // varDim == 0 && s == 0 ok
+//       throw GDLException(NULL,"Scalar subscript out of range [>].h2",true,false);
+//     return 1;
   }
-};
+}; //class CArrayIndexScalar: public ArrayIndexT
 
 
 
@@ -372,6 +440,12 @@ protected:
 public:
   IndexType Type() { return ArrayIndexIndexedID;}
 
+  BaseGDL* OverloadIndexNew( BaseGDL* p1) 
+  { 
+    if( p1 == NULL) return NULL;
+    return p1->Dup();
+  }
+
   SizeT NParam() { return 1;} // number of parameter to Init(...)
 
   RangeT GetS() { return s;}
@@ -390,7 +464,7 @@ public:
 
   bool Indexed() { return (ix != NULL);}
 
-  const dimension& GetDim() { return *ixDim;}
+  const dimension& GetDim() { assert(ixDim != NULL); return *ixDim;}
 
   RangeT GetIx0()
   {
@@ -443,61 +517,12 @@ public:
     ix = NULL; // marker ONE or INDEXED
   }
 
-
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ixL)
-//   {
-//     int ret = ixL[0]->Scalar2RangeT(s);
-// 
-//     if( ret == 0) // more than one element
-//       return var->NewIx( ixL[0], strictArrSubs);
-// 
-//     // scalar (-1,1) or one-element array (-2,2)
-//     //if( ret >= 1) // type ONE
-//     if( ret == 1) // type ONE
-//     {
-// 	    if( s < 0) //&& s >= var->Size())
-// 	    {
-// 		    RangeT ix = var->Size() + s;
-// 		    if( ix < 0)
-// 			    throw GDLException(NULL,"Subscript out of range [-i].",true,false);
-// 		    //if( strictArrSubs || ret == 1)
-// 		    //return var->NewIx( var->Size()-1);
-// 		    return var->NewIx( ix);
-// 	    }
-// 	    if( s >= var->Size())
-// 	    {
-// 		    throw GDLException(NULL,"Subscript out of range [i].",true,false);
-// 	    }
-// 	    return var->NewIx( s);
-//     }
-// 	    // ret == 2 (one dim array)
-//     if( s < 0) //&& s >= var->Size())
-//     {
-// 	    if( strictArrSubs)
-// 		    throw GDLException(NULL,"Subscript out of range [-i].",true,false);
-// 	    BaseGDL* res = var->NewIx( 0);
-// 	    res->SetDim( dimension( 1));
-// 	    return res;
-//     }
-//     if( s >= var->Size())
-//     {
-// 	    if( strictArrSubs)
-// 		    throw GDLException(NULL,"Subscript out of range [i].",true,false);
-// 	    BaseGDL* res = var->NewIx( var->Size()-1);
-// 	    res->SetDim( dimension( 1));
-// 	    return res;
-//     }
-// 
-//     BaseGDL* res = var->NewIx( s);
-//     res->SetDim( dimension( 1));
-//     return res;
-//   }
-
-  void Init( BaseGDL* ix_)
+  void Init( BaseGDL* ix_)   
   {
     if( ix_->Rank() == 0) // type ONE
     {
       ix_->Scalar2RangeT(sInit);
+      s = sInit; // in case of assoc NIter is not called
       // int ret = ix_->Scalar2RangeT(s);
       // from GDL 0.9 on negative indices are fine
       // 	if( ret == -1) // index < 0
@@ -513,7 +538,7 @@ public:
     // type INDEXED
     DType dType = ix_->Type();
 
-    assert( dType != UNDEF);
+    assert( dType != GDL_UNDEF);
     //     assert( maxVal == 0);
 
     int typeCheck = DTypeOrder[ dType];
@@ -523,6 +548,7 @@ public:
     //SizeT nElem = ix_->N_Elements();
     //    ix = new SizeT[ nElem]; // allocate array
 
+  //DEBUG if( ix != NULL)    
     assert( ix == NULL);
 
     //     ix = new AllIxMultiT( nElem);
@@ -547,7 +573,7 @@ public:
 
       if( s < 0)
 	throw GDLException(NULL,"Subscript out of range [-i].",true,false);
-      if( s >= varDim)
+      if( s >= varDim && s > 0)
 	throw GDLException(NULL,"Subscript out of range [i].",true,false);
       return 1;
     }
@@ -555,166 +581,150 @@ public:
     ix->SetUpper( varDim-1);
     return ix->size();
   }
-};
+}; // class ArrayIndexIndexed: public ArrayIndexT
 
-
-
-// constant version
-class CArrayIndexIndexed: public ArrayIndexIndexed
+// INDEXED or ONE [v] (must handle both)
+class CArrayIndexIndexed: public ArrayIndexT
 {
-protected:
-  AllIxIndicesT*    ixOri;
-  char ixOriBuf[ AllIxMaxSize];
+private:
+  bool   strictArrSubs; // for compile_opt STRICTARRSUBS
+  
+  RangeT sInit; // can be < 0 as well
+  RangeT s;
 
-//  SizeT      maxIx;
+  AllIxIndicesT*    ix;
+  char ixBuf[ AllIxMaxSize];
+  
+  const dimension* ixDim; // keep dimension of ix
 
+  // forbid c-i
+  CArrayIndexIndexed( const ArrayIndexT& r) { assert(false);}
+
+  BaseGDL* rawData;
+  bool     isScalar;
+  
 public:
- IndexType Type() { return CArrayIndexIndexedID;}
+  IndexType Type() { return CArrayIndexIndexedID;}
 
-  ~CArrayIndexIndexed() { /*delete ixOri;*/}
-
-  CArrayIndexIndexed( BaseGDL* c, bool strictArrSubs_ = false): 
-    ArrayIndexIndexed( strictArrSubs_)//, ixOri( NULL) //, maxIx( 0)
-  {
-    ArrayIndexIndexed::Init( c);
-    ixOri = ix;
-    ix = NULL;
-//     if( ixOri != NULL) maxIx = ixOri->max();
+  BaseGDL* OverloadIndexNew() 
+  { 
+    assert( rawData != NULL);
+    return rawData->Dup();
   }
 
-  CArrayIndexIndexed( const CArrayIndexIndexed& cp):
-    ArrayIndexIndexed( cp.strictArrSubs), ixOri( NULL) //, maxIx( cp.maxIx)
+  SizeT NParam() { return 0;} // number of parameter to Init(...)
+
+  RangeT GetS() 
+  {     
+    return s;
+  }
+
+  bool Scalar() const { return isScalar;}
+  bool Scalar( RangeT& s_) const // changed from RangeT for proper overloading
   {
-    assert( cp.ix == NULL);
+    if( isScalar)
+    {
+      s_ = s;
+      return true;
+    }
+    s_ = (*ix)[0];
+    return (ix->size() == 1);
+}
 
-    s = cp.s;
-//    maxVal = cp.maxVal;
-//     maxIx  = cp.maxIx;
+  bool Indexed() { return !isScalar;}
 
-    if( cp.ixOri != NULL)
-      ixOri = cp.ixOri->CloneAt( ixOriBuf);
-//       ixOri = new AllIxT( *cp.ixOri);
+  const dimension& GetDim() { assert(ixDim != NULL); return *ixDim;}
+
+  RangeT GetIx0()
+  {
+    if( !isScalar) return (*ix)[0]; // from array
+    return s;
+  }
+
+  SizeT GetIx( SizeT i)
+  {
+    assert( ix != NULL);
+    return (*ix)[ i];
+  }
+
+  //  SizeT* StealIx() { SizeT* ret = ix; ix = NULL; return ret;}
+  //AllIxIndicesT* StealIx() { AllIxIndicesT* ret = ix; ix = NULL; return ret;}
+  AllIxIndicesT* GetAllIx() const 
+  { 
+    return ix;
+  }
+
+  ~CArrayIndexIndexed()
+  {
+    delete rawData;
+  }
+
+  // grabs c
+  CArrayIndexIndexed( BaseGDL* c, bool strictArrSubs_ = false)
+  : strictArrSubs( strictArrSubs_)
+  , ix( NULL), ixDim( NULL)
+  , rawData( c)
+  {
+    assert( rawData != NULL);
+    
+    if( rawData->Rank() == 0) // type ONE
+    {
+      rawData->Scalar2RangeT(sInit);
+      s = sInit; // in case of assoc NIter is not called
+      isScalar = true;
+      return;
+    }
+
+    // type INDEXED
+    isScalar = false;;
+    
+    ixDim = &rawData->Dim();
+
+    assert( rawData->Type() != GDL_UNDEF);
+    DType dType = rawData->Type();
+    int typeCheck = DTypeOrder[ dType];
+    if( typeCheck >= 100)
+      throw GDLException(NULL,"Type not allowed as subscript.",true,false);
+
+    if( strictArrSubs)
+      ix = new (ixBuf) AllIxIndicesStrictT( rawData);
+    else
+      ix = new (ixBuf) AllIxIndicesT( rawData);
   }
 
   ArrayIndexT* Dup() const
   {
-    return new CArrayIndexIndexed( *this);
+    return new CArrayIndexIndexed( rawData->Dup(), strictArrSubs);
   }
 
-  SizeT NParam() { return 0;} // number of parameter to Init(...)
   void Clear()
-  {
-//   delete ix;
-  ix=NULL;
-  } // note that ixDim is untouched
+  {} // nothing to clear
 
-  // make the following work even before call to NIter(...)
-  bool Scalar() const { return (ixOri == NULL);}
-  bool Scalar( RangeT& s_) const
-  { 
-    if( ixOri == NULL)
-      {
-	s_ = s;
-	return true;
-      }
-    s_ = (*ixOri)[0];
-    return (ixOri->size() == 1);
-  }
-  bool Indexed() { return (ixOri != NULL);}
-
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix_)
-//   {
-//     if( ixOri == NULL) // ONE
-//       {
-// 		if( s >= var->Size())
-// 			throw GDLException(NULL,"Subscript out of range [i].",true,false);
-// 		return var->NewIx( s);
-//       }
-// 
-// 	ixOri->SetUpper(var->Size()-1);
-// 
-// //     if( maxIx >= var->Size())
-// //       {
-// // 		if( strictArrSubs)
-// // 		{ // strictArrSubs -> exception if out of bounds
-// // 			throw GDLException(NULL,"Array used to subscript array "
-// // 					"contains out of range subscript.",true,false);
-// // 		}
-// // 
-// // 		SizeT ix_size = ix->size();
-// // 		ix = new AllIxMultiT( ixOri->size()); // make copy as changed (see below)
-// // 		SizeT upper = var->Size()-1;
-// // 		for( SizeT i=0; i < ix_size; ++i)
-// // 		{
-// // 			if( (*ixOri)[i] > upper)
-// // 					static_cast<AllIxMultiT*>(ix)->SetIx( i, upper);
-// // 		      (*ix)[i] = upper;
-// // 			else
-// // 					static_cast<AllIxMultiT*>(ix)->SetIx( i, (*ixOri)[i]);
-// // 		      (*ix)[i] = (*ixOri)[i];
-// // 		}
-// // 		return var->NewIx( ix, ixDim);
-// //       }
-// //     else
-//       return var->NewIx( ixOri, ixDim);
-//   }
-
-  // old (before ixOri): special here no stealing is allowed
-  //  AllIxT* StealIx() { return new AllIxT( *ix);} 
+  void Init()   
+  {} // already initialized in constructor
 
   // number of iterations
-  // also checks/adjusts range 
-  SizeT NIter( SizeT varDim) 
+  // also checks/adjusts range
+  SizeT NIter( SizeT varDim)
   {
-    if( ixOri == NULL) // ONE
-      {
-	if( sInit < 0)
-	  s = sInit + varDim;
-	else
-	  s = sInit;
-	if( s < 0)
-	  throw GDLException(NULL,"Subscript out of range [-i].",true,false);
-	if( s >= varDim)
-	  throw GDLException(NULL,"Subscript out of range [i].",true,false);
-	return 1;
-      }
+    if( isScalar) // ONE
+    {
+      if( sInit < 0)
+	s = sInit + varDim;
+      else
+	s = sInit;
 
+      if( s < 0)
+	throw GDLException(NULL,"Subscript out of range [-i].",true,false);
+      if( s >= varDim && s > 0)
+	throw GDLException(NULL,"Subscript out of range [i].",true,false);
+      return 1;
+    }
     // INDEXED
-    assert( ix == NULL);
-    ix = ixOri->CloneAt(ixBuf); //new AllIxMultiT( ixOri->size()); // make copy as changed (see below)
-
     ix->SetUpper( varDim-1);
-    
-//     if( maxIx >= varDim)
-//       {
-// 		if( strictArrSubs)
-// 		{ // strictArrSubs -> exception if out of bounds
-// 			throw GDLException(NULL,"Array used to subscript array "
-// 					"contains out of range subscript.",true,false);
-// 		}
-// 
-// 		SizeT upper = varDim-1;
-// 		for( SizeT i=0; i < ix_size; ++i)
-// 		{
-// 			if( (*ixOri)[i] > upper)
-// 				static_cast<AllIxMultiT*>(ix)->SetIx( i, upper);
-// 		      (*ix)[i] = upper;
-// 			else
-// 				static_cast<AllIxMultiT*>(ix)->SetIx( i, (*ixOri)[i]);
-// 		      (*ix)[i] = (*ixOri)[i];
-// 		}
-//       }
-//     else
-//       for( SizeT i=0; i < ix_size; ++i)
-// 	   		static_cast<AllIxMultiT*>(ix)->SetIx( i, (*ixOri)[i]);
-// 	(*ix)[i] = (*ixOri)[i];
-    
-    return ix->size(); 
+    return ix->size();
   }
-};
-
-
-
+}; //class CArrayIndexIndexed: public ArrayIndexIndexed
 
 
 
@@ -724,6 +734,14 @@ class ArrayIndexAll: public ArrayIndexT
 public:
  IndexType Type() { return ArrayIndexAllID;}
 
+ bool IsRange() { return true;}
+ 
+ BaseGDL* OverloadIndexNew()
+  {
+    const DLong arr[3] = {0,-1,1};
+    return new DLongGDL( arr, 3);
+  }
+ 
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
   void Init() {};
@@ -735,14 +753,6 @@ public:
     return new ArrayIndexAll();
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-//     // make index 1-dim
-//     BaseGDL* clone =  var->Dup();
-//     clone->SetDim( dimension( clone->N_Elements()));
-//     return clone;
-//   }
-
   // number of iterations
   // also checks/adjusts range 
   SizeT NIter( SizeT varDim)
@@ -763,6 +773,15 @@ protected:
 public:
  IndexType Type() { return ArrayIndexORangeID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew( BaseGDL* s_) 
+  {
+    Init( s_);
+    DLong arr[3] = {sInit,-1,1};
+    return new DLongGDL( arr, 3);
+  }
+  
   SizeT NParam() { return 1;} // number of parameter to Init(...)
 
   RangeT GetS() { return s;}
@@ -801,30 +820,9 @@ public:
 //       }
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-//     Init( ix[0]);
-// 
-// 	SizeT varSize = var->Size();
-//     
-// 	if( s < 0)
-// 		{
-// 			RangeT i = s + varSize;
-// 			if( i < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-s:*].",true,false);
-//     
-// 			return var->NewIxFrom( i);
-// 		}
-// 	
-//     if( s >= varSize)
-//       throw GDLException(NULL,"Subscript out of range [s:*].",true,false);
-// 
-//     return var->NewIxFrom( s);
-//   }
-
   SizeT NIter( SizeT varDim)
   {
-    if( sInit >= varDim)
+    if( sInit >= varDim) // && s > 0)
       throw GDLException(NULL,"Subscript out of range [s:*].",true,false);
     if( sInit < 0)
     {
@@ -846,6 +844,14 @@ class CArrayIndexORange: public ArrayIndexORange
 public:
  IndexType Type() { return CArrayIndexORangeID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew()
+  {
+    DLong arr[3] = {sInit,-1,1};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
   CArrayIndexORange( BaseGDL* c): ArrayIndexORange()
@@ -863,24 +869,6 @@ public:
     return d;
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-// 	SizeT varSize = var->Size();
-//     
-// 	if( s < 0)
-// 		{
-// 			RangeT i = s + varSize;
-// 			if( i < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-s:*].",true,false);
-//     
-// 			return var->NewIxFrom( i);
-// 		}
-//     
-//     if( s >= varSize)
-//       throw GDLException(NULL,"Subscript out of range [s:*].",true,false);
-// 
-//     return var->NewIxFrom( s);
-//   }
 };
 
 
@@ -895,45 +883,20 @@ protected:
 public:
  IndexType Type() { return ArrayIndexRangeID;}
 
+  bool IsRange() { return true;}
+  
+  BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* e_) 
+  {
+    Init( s_, e_);
+    DLong arr[3] = {sInit,eInit,1};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 2;} // number of parameter to Init(...)
 
   RangeT GetS() { return s;}
   RangeT GetIx0() { return s;}
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-//     Init( ix[0], ix[1]);
-//     
-// 	SizeT varSize = var->Size();
-// 	
-//     RangeT sl,el;
-// 	if( s < 0)
-// 		{
-// 			sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:e].",true,false);
-// 		}
-// 	else
-// 		sl = s;
-// 	if( e < 0)
-// 		{
-// 			el = e + varSize;
-// 			if( el < 0)
-// 				throw GDLException(NULL,"Subscript out of range [s:-E].",true,false);
-// 		}
-// 	else
-// 		el = e;
-//     
-//     if( sl > el)
-// 		throw 
-// 			GDLException(NULL,"Subscript range values of the form low:high "
-// 				"must be < size, with low <= high",true,false);
-//     if( el >= var->Size())
-//       throw GDLException(NULL,"Subscript out of range [s:e].",true,false);
-// 
-//     return var->NewIxFrom( sl, el);
-//   }
-
   ArrayIndexT* Dup() const
   {
     ArrayIndexRange* d = new ArrayIndexRange();
@@ -975,26 +938,6 @@ public:
 	  throw 
 	    GDLException(NULL,"Expression must be a scalar in this context.",true,false); 
       }
-//     if( retMsg == -1) // index < 0
-//       {
-// 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form low:high " 
-// 			"must be >= 0, < size, with low <= high.",true,false);
-//       }
-
-// checked later at NIter()
-//     if( eInit>=0 && sInit>=0 && eInit < sInit)
-//       {
-// 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form low:high "
-// 		  "must be < size, with low <= high",true,false);
-//       }
-//     if( eInit<0 && sInit<0 && eInit < sInit)
-//       {
-// 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form -low:-high "
-// 		  "must be < size, with low <= high",true,false);
-//       }
   }
 
   // number of iterations
@@ -1023,7 +966,7 @@ public:
       throw 
 	GDLException(NULL,"Subscript range values of the form low:high "
 		"must be < size, with low <= high",true,false);
-    if( e >= varDim)
+    if( e >= varDim) // && e > 0)
 		throw GDLException(NULL,"Subscript out of range [s:e].",true,false);
     return (e - s + 1);
   }
@@ -1036,6 +979,14 @@ class CArrayIndexRange: public ArrayIndexRange
 public:
  IndexType Type() { return CArrayIndexRangeID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew()
+  {
+    DLong arr[3] = {sInit,eInit,1};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
   CArrayIndexRange( BaseGDL* c1, BaseGDL* c2): ArrayIndexRange()
@@ -1055,37 +1006,6 @@ public:
     return d;
   }
 
-//    BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//    {
-// 	SizeT varSize = var->Size();
-// 	
-//     RangeT sl,el;
-// 	if( s < 0)
-// 		{
-// 			sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:e].",true,false);
-// 		}
-// 	else
-// 		sl = s;
-// 	if( e < 0)
-// 		{
-// 			el = e + varSize;
-// 			if( el < 0)
-// 				throw GDLException(NULL,"Subscript out of range [s:-E].",true,false);
-// 		}
-// 	else
-// 		el = e;
-//     
-//     if( sl > el)
-// 		throw 
-// 			GDLException(NULL,"Subscript range values of the form low:high "
-// 				"must be < size, with low <= high",true,false);
-//     if( el >= var->Size())
-//       throw GDLException(NULL,"Subscript out of range [s:e].",true,false);
-// 
-//     return var->NewIxFrom( sl, el);
-//    }
 };
 
 
@@ -1101,6 +1021,15 @@ protected:
 public:
  IndexType Type() { return ArrayIndexORangeSID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* stride_) 
+  {
+    Init( s_, stride_);
+    DLong arr[3] = {sInit,-1,stride};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 2;} // number of parameter to Init(...)
 
   RangeT GetS() { return s;}
@@ -1156,27 +1085,6 @@ public:
       }
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-//     Init( ix[0], ix[1]);
-// 	
-// 	SizeT varSize = var->Size();
-// 
-// 	if( s < 0)
-// 		{
-// 			RangeT sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:*:stride].",true,false);
-// 
-// 			return var->NewIxFromStride( sl, stride);
-// 		}
-// 
-//     if( s >= var->Size())
-//       throw GDLException(NULL,"Subscript out of range [s:*:stride].",true,false);
-// 
-//     return var->NewIxFromStride( s, stride);
-//   }
-
   // number of iterations
   // also checks/adjusts range 
   SizeT NIter( SizeT varDim)
@@ -1191,7 +1099,7 @@ public:
     else
       s= sInit;
 
-    if( s >= varDim)
+    if( s >= varDim) // && s > 0)
       throw GDLException(NULL,"Subscript out of range [s:*:stride].",true,false);
     return (varDim - s + stride - 1)/stride;
   }
@@ -1202,6 +1110,14 @@ class CArrayIndexORangeS: public ArrayIndexORangeS
 public:
  IndexType Type() { return CArrayIndexORangeSID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew()
+  {
+    DLong arr[3] = {sInit,-1,stride};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
   CArrayIndexORangeS( BaseGDL* c1, BaseGDL* c2): ArrayIndexORangeS()
@@ -1209,7 +1125,7 @@ public:
     ArrayIndexORangeS::Init( c1, c2);
   }
 
-CArrayIndexORangeS(){}
+  CArrayIndexORangeS(){}
 
   ArrayIndexT* Dup() const
   {
@@ -1220,24 +1136,6 @@ CArrayIndexORangeS(){}
     return d;
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-// 	SizeT varSize = var->Size();
-// 
-// 	if( s < 0)
-// 		{
-// 			RangeT sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:*:stride].",true,false);
-// 
-// 			return var->NewIxFromStride( sl, stride);
-// 		}
-// 
-//     if( s >= var->Size())
-//       throw GDLException(NULL,"Subscript out of range [s:*:stride].",true,false);
-// 
-//     return var->NewIxFromStride( s, stride);
-//   }
 };
 
 // [s:e:st]
@@ -1248,9 +1146,30 @@ protected:
   RangeT s,e;
   SizeT stride;
 
+  void Init( DLong s_, DLong e_, DLong stride_)
+  {
+    sInit = s_;
+    eInit = e_;
+    if( stride_ <= 0)
+      {
+	throw 
+	  GDLException(  "Range subscript stride must be >= 1.",true,false);
+      }
+    stride = stride_;
+  }
+
 public:
  IndexType Type() { return ArrayIndexRangeSID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* e_, BaseGDL* stride_)
+  {
+    Init( s_, e_, stride_);
+    DLong arr[3] = {sInit,eInit,stride};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 3;} // number of parameter to Init(...)
 
   RangeT GetS() { return s;}
@@ -1297,26 +1216,6 @@ public:
 	  throw 
 	    GDLException(  "Expression must be a scalar in this context.",true,false); 
       }
-//     if( retMsg == -1) // index < 0
-//       {
-// 	throw 
-// 	  GDLException(  "Subscript range values of the form low:high " 
-// 			"must be >= 0, < size, with low <= high.",true,false);
-//       }
-
-// checked later at NIter()
-//     if( eInit>=0 && sInit>=0 && eInit < sInit)
-//       {
-// 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form low:high "
-// 		  "must be < size, with low <= high",true,false);
-//       }
-//     if( eInit<0 && sInit<0 && eInit < sInit)
-//       {
-// 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form -low:-high "
-// 		  "must be < size, with low <= high",true,false);
-//       }
                             
     // stride
     retMsg=stride_->Scalar2index(stride);
@@ -1338,43 +1237,6 @@ public:
       }
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-//     Init( ix[0], ix[1], ix[2]);
-// 
-// 	SizeT varSize = var->Size();
-// 	
-//     RangeT sl,el;
-// 	if( s < 0)
-// 		{
-// 			sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:e:stride].",true,false);
-// 		}
-// 	else
-// 		sl = s;
-// 	if( e < 0)
-// 		{
-// 			el = e + varSize;
-// 			if( el < 0)
-// 				throw GDLException(NULL,"Subscript out of range [s:-E:stride].",true,false);
-// 		}
-// 	else
-// 		el = e;
-//     
-//     if( sl > el)
-// 		throw 
-// 			GDLException(NULL,"Subscript range values of the form low:high "
-// 				"must be < size, with low <= high",true,false);
-//     
-//     if( el >= var->Size())
-//       {
-// 	throw GDLException(NULL,"Subscript out of range [s:e:st].",true,false);
-//       }
-// 
-//     return var->NewIxFromStride( sl, el, stride);
-//   }
-
   // number of iterations
   // also checks/adjusts range 
   SizeT NIter( SizeT varDim)
@@ -1402,7 +1264,7 @@ public:
 			GDLException(NULL,"Subscript range values of the form low:high "
 				"must be < size, with low <= high",true,false);
     
-    if( e >= varDim)
+    if( e >= varDim) // && e > 0)
       {
 		throw GDLException(NULL,"Subscript out of range [s:E:st].",true,false);
       }
@@ -1415,6 +1277,14 @@ class CArrayIndexRangeS: public ArrayIndexRangeS
 public:
   IndexType Type() { return CArrayIndexRangeSID;}
 
+  bool IsRange() { return true;}
+
+  BaseGDL* OverloadIndexNew()
+  {
+    DLong arr[3] = {sInit,eInit,stride};
+    return new DLongGDL( arr, 3);
+  }
+
   SizeT NParam() { return 0;} // number of parameter to Init(...)
 
   CArrayIndexRangeS( BaseGDL* c1, BaseGDL* c2, BaseGDL* c3): 
@@ -1423,6 +1293,13 @@ public:
     ArrayIndexRangeS::Init( c1, c2, c3);
   }
 
+  // for internal routines to ease definition
+  CArrayIndexRangeS( DLong s_, DLong e_, DLong stride_): 
+    ArrayIndexRangeS()
+  {
+    ArrayIndexRangeS::Init( s_, e_, stride_);
+  }
+
   CArrayIndexRangeS(){}
 
   ArrayIndexT* Dup() const
@@ -1436,40 +1313,6 @@ public:
     return d;
   }
 
-//   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
-//   {
-// 	SizeT varSize = var->Size();
-// 	
-//     RangeT sl,el;
-// 	if( s < 0)
-// 		{
-// 			sl = s + varSize;
-// 			if( sl < 0)
-// 				throw GDLException(NULL,"Subscript out of range [-S:e:stride].",true,false);
-// 		}
-// 	else
-// 		sl = s;
-// 	if( e < 0)
-// 		{
-// 			el = e + varSize;
-// 			if( el < 0)
-// 				throw GDLException(NULL,"Subscript out of range [s:-E:stride].",true,false);
-// 		}
-// 	else
-// 		el = e;
-//     
-//     if( sl > el)
-// 		throw 
-// 			GDLException(NULL,"Subscript range values of the form low:high "
-// 				"must be < size, with low <= high",true,false);
-//     
-//     if( el >= var->Size())
-//       {
-// 		throw GDLException(NULL,"Subscript out of range [s:e:st].",true,false);
-//       }
-// 
-//     return var->NewIxFromStride( sl, el, stride);
-//   }
 };
 
 #endif
