@@ -43,10 +43,22 @@ public:
 
   virtual ArrayIndexListT* Clone() { assert( 0); return NULL;}
 
-  virtual void Init( IxExprListT& ix, IxExprListT* cleanupIx) { assert( 0);}
+  virtual void InitAsOverloadIndex( IxExprListT& ixIn, IxExprListT* cleanupIx, IxExprListT& ixOut) 
+  { 
+    throw GDLException( NULL,"Internal error: ArrayIndexListT::InitAsOverladIndex( IxExprListT& ixInOut) called.",true,false);    
+  }
+  virtual void Init( IxExprListT& ix, IxExprListT* cleanupIx) 
+  { 
+    assert( 0);
+    throw GDLException( NULL,"Internal error: ArrayIndexListT::Init( IxExprListT& ix, IxExprListT* cleanupIx) called.",true,false);    
+  }
   virtual void Init() {}
   
-   virtual bool ToAssocIndex( SizeT& lastIx) = 0;
+   virtual bool ToAssocIndex( SizeT& lastIx) 
+   { 
+    assert( 0);       
+    throw GDLException( NULL,"Internal error: ArrayIndexListT::ToAssocIndex( SizeT& lastIx) called.",true,false);
+   }
 //  virtual bool ToAssocIndex( RangeT& lastIx) = 0;
 
   // set the root variable which is indexed by this ArrayIndexListT
@@ -191,7 +203,8 @@ public:
     assert( allIx == NULL);
 
     // for assoc variables last index is the record
-    if( var->IsAssoc()) return;
+    // we cannot return here as sInit is not yet copied to s
+    //if( var->IsAssoc()) return;
 
     // ArrayIndexScalar[VP] are not initialized
     // they need the NIter call, but
@@ -225,6 +238,8 @@ public:
   // returns 1-dim index for all elements
   AllIxBaseT* BuildIx()
   {
+    if( allIx != NULL) // can happen if called from DotAccessDescT::DoAssign()
+      return allIx;
     assert( allIx == NULL);
 //     if( allIx != NULL)
 // 		return allIx;
@@ -289,7 +304,7 @@ public:
   {
     // scalar case
     if( right->N_Elements() == 1 && !var->IsAssoc() &&
-	ix->NIter( var->Size()) == 1)// && var->Type() != STRUCT) 
+	ix->NIter( var->Size()) == 1)// && var->Type() != GDL_STRUCT) 
       {
 	var->AssignAtIx( ix->GetIx0(), right);
 	return;
@@ -314,8 +329,11 @@ public:
   BaseGDL* Index( BaseGDL* var, IxExprListT& ix_)
   {
     Init( ix_, NULL);
-    if( !var->IsAssoc() && ix->NIter( var->Size()) == 1)// && var->Type() != STRUCT) 
+    if( !var->IsAssoc() && ix->Scalar()) //ix->NIter( var->Size()) == 1)// && var->Type() != GDL_STRUCT) 
       {
+	SizeT assertValue = ix->NIter( var->Size());
+	assert( assertValue == 1);
+
 	return var->NewIx( ix->GetIx0());
       }
     // normal case
@@ -578,7 +596,7 @@ public:
   {
     // Init() was already called
     // scalar case
-    if( right->N_Elements() == 1 && !var->IsAssoc()) // && var->Type() != STRUCT) 
+    if( right->N_Elements() == 1 && !var->IsAssoc()) // && var->Type() != GDL_STRUCT) 
       {
 	s = varPtr->Data()->LoopIndex();
 	if( s >= var->Size())
@@ -729,7 +747,7 @@ public:
 
     // Init() was already called
     // scalar case
-    if( right->N_Elements() == 1 && !var->IsAssoc())// && var->Type() != STRUCT) 
+    if( right->N_Elements() == 1 && !var->IsAssoc())// && var->Type() != GDL_STRUCT) 
       {
 	if( sInit < 0)
 	  s = sInit + var->Size();
@@ -759,7 +777,7 @@ public:
   BaseGDL* Index( BaseGDL* var, IxExprListT& ix_)
   {
     // Init() not called
-    if( !var->IsAssoc())// && var->Type() != STRUCT)
+    if( !var->IsAssoc())// && var->Type() != GDL_STRUCT)
       {
 	if( sInit < 0)
 	  s = sInit + var->Size();
@@ -1117,7 +1135,7 @@ protected:
 
   enum AccessType
   {
-	  UNDEF=0,      // for init access type
+	  GDL_UNDEF=0,      // for init access type
 	  INDEXED_ONE,  // all indexed OR one
 	  NORMAL,       // mixed
 	  ALLINDEXED,
@@ -1223,10 +1241,6 @@ public:
 	    CArrayIndexScalarID == ixList[ixList.size()-1]->Type()) nScalar++;
     else if( ArrayIndexIndexedID == ixList[ixList.size()-1]->Type() ||
 	    CArrayIndexIndexedID == ixList[ixList.size()-1]->Type()) nIndexed++;
-/*    if( dynamic_cast< ArrayIndexScalar*>(ixList[ixList.size()-1]) ||
-    dynamic_cast< CArrayIndexScalar*>(ixList[ixList.size()-1])) nScalar++;
-if( dynamic_cast<ArrayIndexIndexed*>(ixList[ixList.size()-1]) ||
-    dynamic_cast<CArrayIndexIndexed*>(ixList[ixList.size()-1]) ) nIndexed++;*/
     
     if( nScalar == ixList.size())
       accessTypeInit = ALLONE;
@@ -1555,6 +1569,9 @@ if( dynamic_cast<ArrayIndexIndexed*>(ixList[ixList.size()-1]) ||
   // returns 1-dim index for all elements
   AllIxBaseT* BuildIx() // ArrayIndexListMultiT
   {
+    if( allIx != NULL) // can happen if called from DotAccessDescT::DoAssign()
+      return allIx;
+
     assert( allIx == NULL);
 
     if( accessType == ALLINDEXED) //  nIterLimitGt1 is not properly set
@@ -1613,38 +1630,6 @@ if( dynamic_cast<ArrayIndexIndexed*>(ixList[ixList.size()-1]) ||
     // higher indices of variable are implicitely zero,
     // therefore they are not checked in 'SetRoot'
 
-// 	SizeT nIterLimitGt1 = 0;
-// 	SizeT baseIx = 0;
-// 	RankT gt1Rank;
-// 	bool indexed;
-// 	for( SizeT l=0; l<acRank; ++l)
-// 	{
-// 		if( nIterLimit[l] > 1)
-// 		{
-// 			++nIterLimitGt1;
-// 			gt1Rank = l;
-// 			if( !ixList[l]->Indexed())
-// 			{
-// 				baseIx += ixList[l]->GetS() * varStride[l];
-// 				indexed = false;
-// 			}
-// 			else
-// 			{
-// 				indexed = true;
-// 			}
-// 		}
-// 		else
-// 		{
-// 			if( ixList[l]->Indexed())
-// 			{
-// 				baseIx += static_cast< ArrayIndexIndexed*>( ixList[l])->GetIx( 0) * varStride[l];
-// 			}
-// 			else
-// 			{
-// 				baseIx += ixList[l]->GetS()  * varStride[l];
-// 			}
-// 		}
-// 	}
     if( nIterLimitGt1 == 1) // only one variable dimension
     {
       if( indexed)
@@ -1661,103 +1646,6 @@ if( dynamic_cast<ArrayIndexIndexed*>(ixList[ixList.size()-1]) ||
     allIx = new (allIxInstance) AllIxNewMultiT( &ixList, acRank, nIx, varStride, nIterLimit, stride);
     return allIx;
 }
-//     // init allIx from first index
-//     if( ixList[0]->Indexed())
-//       {
-// 		AllIxMultiT* tmpIx = static_cast< ArrayIndexIndexed*>(ixList[0])->StealIx();
-// 
-// 		for( SizeT i=0; i<nIx; ++i)
-// 		{
-// 		static_cast<AllIxMultiT*>(allIx)->SetIx( i, tmpIx->GetIx( i %  nIterLimit[0]));
-// 	// 	  	    (*allIx)[ i] = (*tmpIx)[ i %  nIterLimit[0]];
-// 		}
-// 
-// 		delete tmpIx;
-//       }
-//     else
-//       {
-// 		SizeT s = ixList.FrontGetS(); //ixList[0]->GetS();
-// 		SizeT ixStride = ixList[0]->GetStride();
-// 		
-// 		if( ixStride <= 1)
-// 			if( s != 0)
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->SetIx( i,  (i %  nIterLimit[0]) + s);
-// 		// 		(*allIx)[i] = (i %  nIterLimit[0]) + s; // stride[0], varStride[0] == 1
-// 				}
-// 			else
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->SetIx( i, (i %  nIterLimit[0]));
-// 		// 		(*allIx)[i] = (i %  nIterLimit[0]); // stride[0], varStride[0] == 1
-// 				}
-// 		else
-// 			if( s != 0)
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->SetIx( i, (i %  nIterLimit[0]) * ixStride + s);
-// 		// 		(*allIx)[i] = (i %  nIterLimit[0]) * ixStride + s; // stride[0], varStride[0] == 1
-// 				}
-// 			else
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->SetIx( i, (i %  nIterLimit[0]) * ixStride);
-// 		// 		(*allIx)[i] = (i %  nIterLimit[0]) * ixStride; // stride[0], varStride[0] == 1
-// 				}
-//       }
-// 
-//     for( SizeT l=1; l < acRank; ++l)
-//     {
-// 		if( ixList[l]->Indexed())
-// 		{
-// 				AllIxMultiT* tmpIx = static_cast< ArrayIndexIndexed*>(ixList[l])->StealIx();
-// 				//	    SizeT* tmpIx = ixList[l]->StealIx();
-// 			
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->AddToIx( i,  tmpIx->GetIx( (i / stride[l]) %  nIterLimit[l]) * varStride[l]);
-// 			// 		(*allIx)[ i] += (*tmpIx)[ (i / stride[l]) %  nIterLimit[l]] * varStride[l];
-// 				}
-// 			
-// 				delete tmpIx;
-// 		}
-// 		else
-// 		{
-// 				SizeT s = ixList[l]->GetS();
-// 				SizeT ixStride = ixList[l]->GetStride();
-// 			
-// 				if( ixStride <= 1)
-// 				if( s != 0)
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->AddToIx( i, ((i / stride[l]) %  nIterLimit[l] + s) * varStride[l]);
-// 		// 		  (*allIx)[i] += ((i / stride[l]) %  nIterLimit[l] + s) * varStride[l];
-// 				}
-// 				else
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->AddToIx( i, ((i / stride[l]) %  nIterLimit[l]) * varStride[l]);
-// 		// 		  (*allIx)[i] += ((i / stride[l]) %  nIterLimit[l]) * varStride[l];
-// 				}
-// 				else // ixStride > 1
-// 				if( s != 0)
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->AddToIx( i, (((i / stride[l]) %  nIterLimit[l]) * ixStride + s) * varStride[l]);
-// 		// 		  (*allIx)[i] += (((i / stride[l]) %  nIterLimit[l]) * ixStride + s) * varStride[l];
-// 				}
-// 				else
-// 				for( SizeT i=0; i<nIx; ++i)
-// 				{
-// 					static_cast<AllIxMultiT*>(allIx)->AddToIx( i, ((i * ixStride / stride[l]) %  nIterLimit[l]) * ixStride * varStride[l]);
-// 		// 		  (*allIx)[i] += ((i * ixStride / stride[l]) %  nIterLimit[l]) * ixStride * varStride[l];
-// 				}
-// 		}
-// 	}
-//     
-//     return allIx;
-//   }
 
   // returns one dim long ix in case of one element array index
   // used by AssignAt and Index functions
@@ -1794,7 +1682,9 @@ if( dynamic_cast<ArrayIndexIndexed*>(ixList[ixList.size()-1]) ||
     SetVariable( var);
     if( nIx == 1 && !var->IsAssoc())
     {
-      return var->NewIx( baseIx);
+      BaseGDL* res = var->NewIx( baseIx);
+      res->MakeArrayFromScalar();
+      return res;
     }
     return var->Index( this);
   }
@@ -1981,6 +1871,9 @@ class ArrayIndexListMultiNoneIndexedT: public ArrayIndexListMultiT
   // returns 1-dim index for all elements
   AllIxBaseT* BuildIx()
   {
+    if( allIx != NULL) // can happen if called from DotAccessDescT::DoAssign()
+      return allIx;
+
 	assert( allIx == NULL);  
 // 	if( allIx != NULL)
 // 		return allIx;
@@ -2269,6 +2162,7 @@ private:
   ArrayIndexListT* aL;
 public:
   ArrayIndexListGuard(): aL( NULL) {}
+  ArrayIndexListGuard(ArrayIndexListT* aLToGuard): aL( aLToGuard) {}
   ~ArrayIndexListGuard() 
   {
     if( aL != NULL)
