@@ -83,14 +83,14 @@ namespace lib {
 
   BaseGDL* gdl_logical_and( EnvT* e);
   BaseGDL* gdl_logical_or( EnvT* e);
-  BaseGDL* logical_true( EnvT* e);
+  BaseGDL* logical_true( BaseGDL* p0, bool isReference);//( EnvT* e);
 
   BaseGDL* replicate( EnvT* e);
 
   BaseGDL* strcompress( EnvT* e);
-  BaseGDL* strlowcase( EnvT* e);
-  BaseGDL* strupcase( EnvT* e);
-  BaseGDL* strlen( EnvT* e);
+  BaseGDL* strlowcase( BaseGDL* p0, bool isReference);//( EnvT* e);
+  BaseGDL* strupcase( BaseGDL* p0, bool isReference);//( EnvT* e);
+  BaseGDL* strlen( BaseGDL* p0, bool isReference);//( EnvT* e);
   BaseGDL* strmid( EnvT* e);
   BaseGDL* strpos( EnvT* e);
   BaseGDL* strtrim( EnvT* e);
@@ -161,6 +161,10 @@ namespace lib {
   BaseGDL* idl_base64(EnvT* e);
   BaseGDL* get_drive_list(EnvT* e);
 
+  BaseGDL* scope_varfetch_value( EnvT* e); // regular library function
+
+  BaseGDL** scope_varfetch_reference( EnvT* e); // special version for LEval()
+
 } // namespace
 
 #endif
