@@ -41,14 +41,14 @@ namespace lib {
 
   BaseGDL* size( EnvT* e) 
   {
-    e->NParam( 1); // might be UNDEF, but must be given
+    e->NParam( 1); // might be GDL_UNDEF, but must be given
 
     // BaseGDL* p0 = e->GetParDefined( 0); //, "SIZE");
     BaseGDL* p0 = e->GetPar( 0); //, "SIZE");
 
     SizeT nEl = 0;
     SizeT Rank = 0;
-    SizeT vType = UNDEF;
+    SizeT vType = GDL_UNDEF;
 
     if (p0 != NULL) {
       nEl = p0->N_Elements();
@@ -107,7 +107,7 @@ namespace lib {
 
       DString tname;
       DString sname;
-      if (vType == STRUCT) {
+      if (vType == GDL_STRUCT) {
 	tname = "STRUCT";
 	DStructGDL* s = static_cast<DStructGDL*>( p0);
 	if (s->Desc()->IsUnnamed())
@@ -120,10 +120,10 @@ namespace lib {
       }
 
 
-      DLongGDL *dims_res = new DLongGDL(dimension(8), BaseGDL::ZERO);      
+      DLongGDL *dims_res = new DLongGDL(dimension(MAXRANK), BaseGDL::ZERO);      
 
       // Initialize dimension values to 0
-      for( SizeT i=Rank; i<8; ++i) (*dims_res)[ i] = 0;
+      for( SizeT i=Rank; i<MAXRANK; ++i) (*dims_res)[ i] = 0;
       for( SizeT i=0; i<Rank; ++i) {
 	(*dims_res)[ i] = p0->Dim(i);
       }
@@ -369,7 +369,7 @@ namespace lib {
 	    
       }
 
-    if( type < 0 || type > ULONG64 || type == STRUCT)
+    if( type < 0 || type > GDL_ULONG64 || type == GDL_STRUCT)
       {
 	e->Throw("Invalid type specified for result.");
       }
@@ -379,9 +379,9 @@ namespace lib {
 
     if (e->KeywordSet(indexix))
     {
-      if (type == PTR || e->KeywordSet(18)) 
+      if (type == GDL_PTR || e->KeywordSet(18)) 
         e->Throw("Index initialization of pointer array is invalid.");
-      if (type == OBJECT || e->KeywordSet(19)) 
+      if (type == GDL_OBJ || e->KeywordSet(19)) 
         e->Throw("Index initialization of object reference array is invalid..");
     }
 
@@ -403,57 +403,57 @@ namespace lib {
       }
 
     // BYTEDComplexDblGDL
-    if (e->KeywordSet(6) || type == BYTE) {
+    if (e->KeywordSet(6) || type == GDL_BYTE) {
 
       return make_array_template< DByteGDL>( e, dimKey, value);
 
-      //INT
-    } else if (e->KeywordSet(7) || type == INT) {
+      //GDL_INT
+    } else if (e->KeywordSet(7) || type == GDL_INT) {
 
       return make_array_template< DIntGDL>( e, dimKey, value);
 
-      // UINT
-    } else if (e->KeywordSet(8) || type == UINT) {
+      // GDL_UINT
+    } else if (e->KeywordSet(8) || type == GDL_UINT) {
 
       return make_array_template< DUIntGDL>( e, dimKey, value);
 
-      // LONG
-    } else if (e->KeywordSet(9) || type == LONG) {
+      // GDL_LONG
+    } else if (e->KeywordSet(9) || type == GDL_LONG) {
 
       return make_array_template< DLongGDL>( e, dimKey, value);
 
-      // ULONG
-    } else if (e->KeywordSet(10) || type == ULONG) {
+      // GDL_ULONG
+    } else if (e->KeywordSet(10) || type == GDL_ULONG) {
 
       return make_array_template< DULongGDL>( e, dimKey, value);
 
-      // LONG64
-    } else if (e->KeywordSet(11) || type == LONG64) {
+      // GDL_LONG64
+    } else if (e->KeywordSet(11) || type == GDL_LONG64) {
 
       return make_array_template< DLong64GDL>( e, dimKey, value);
 
-      // ULONG64
-    } else if (e->KeywordSet(12) || type == ULONG64) {
+      // GDL_ULONG64
+    } else if (e->KeywordSet(12) || type == GDL_ULONG64) {
 
       return make_array_template< DULong64GDL>( e, dimKey, value);
       
-      // DOUBLE
-    } else if (e->KeywordSet(14) || type == DOUBLE) {
+      // GDL_DOUBLE
+    } else if (e->KeywordSet(14) || type == GDL_DOUBLE) {
 
       return make_array_template< DDoubleGDL>( e, dimKey, value);
 
-      // COMPLEX
-    } else if (e->KeywordSet(15) || type == COMPLEX) {
+      // GDL_COMPLEX
+    } else if (e->KeywordSet(15) || type == GDL_COMPLEX) {
 
       return make_array_template< DComplexGDL>( e, dimKey, value);
 
       // DCOMPLEX
-    } else if (e->KeywordSet(16) || type == COMPLEXDBL) {
+    } else if (e->KeywordSet(16) || type == GDL_COMPLEXDBL) {
 
       return make_array_template< DComplexDblGDL>( e, dimKey, value);
 
-      // STRING (added by MS 29.10.2005)
-    } else if (e->KeywordSet(17) || type == STRING) {
+      // GDL_STRING (added by MS 29.10.2005)
+    } else if (e->KeywordSet(17) || type == GDL_STRING) {
 
       if (!e->KeywordSet(indexix))
 		return make_array_template< DStringGDL>( e, dimKey, value);
@@ -464,7 +464,7 @@ namespace lib {
 //       {
 
 	  BaseGDL* ret = make_array_template< DULongGDL>( e, dimKey, value); //, true);
-      return ret->Convert2( STRING);
+      return ret->Convert2( GDL_STRING);
 
 //         for (DLong i = 0; i < ret->N_Elements(); ++i)
 //         {
@@ -475,18 +475,18 @@ namespace lib {
 //       }
 //       return ret;
 
-      // PTR (added by SA 15.08.2009)
-    } else if (e->KeywordSet(18) || type == PTR) {
+      // GDL_PTR (added by SA 15.08.2009)
+    } else if (e->KeywordSet(18) || type == GDL_PTR) {
 
       return make_array_template< DPtrGDL>( e, dimKey, value);
 
       // OBJ (added by SA 15.08.2009)
-    } else if (e->KeywordSet(19) || type == OBJECT) {
+    } else if (e->KeywordSet(19) || type == GDL_OBJ) {
 
       return make_array_template< DObjGDL>( e, dimKey, value);
 
-      // STRUCT
-    } else if ( type == STRUCT) {
+      // GDL_STRUCT
+    } else if ( type == GDL_STRUCT) {
       
       assert( value != NULL);
 
@@ -587,48 +587,48 @@ namespace lib {
     res->SetDim(dim);
     return res;
 
-//     // BYTE
-//     if (Type == BYTE) {
+//     // GDL_BYTE
+//     if (Type == GDL_BYTE) {
     
 //       return reform_template< DByteGDL>( e, dim);
     
-//       //INT
-//     } else if (Type || Type == INT) {
+//       //GDL_INT
+//     } else if (Type || Type == GDL_INT) {
     
 //       return reform_template< DIntGDL>( e, dim);
     
-//       // UINT
-//     } else if (Type == UINT) {
+//       // GDL_UINT
+//     } else if (Type == GDL_UINT) {
     
 //       return reform_template< DUIntGDL>( e, dim);
     
-//       // LONG
-//     } else if (Type == LONG) {
+//       // GDL_LONG
+//     } else if (Type == GDL_LONG) {
     
 //       return reform_template< DLongGDL>( e, dim);
     
-//       // ULONG
-//     } else if (Type == ULONG) {
+//       // GDL_ULONG
+//     } else if (Type == GDL_ULONG) {
     
 //       return reform_template< DULongGDL>( e, dim);
     
-//       // LONG64
-//     } else if (Type == LONG64) {
+//       // GDL_LONG64
+//     } else if (Type == GDL_LONG64) {
     
 //       return reform_template< DLong64GDL>( e, dim);
     
-//       // ULONG64
-//     } else if (Type == ULONG64) {
+//       // GDL_ULONG64
+//     } else if (Type == GDL_ULONG64) {
     
 //       return reform_template< DULong64GDL>( e, dim);
       
-//       // FLOAT
-//     } else if (Type == FLOAT) {
+//       // GDL_FLOAT
+//     } else if (Type == GDL_FLOAT) {
     
 //       return reform_template< DFloatGDL>( e, dim);
     
-//       // DOUBLE
-//     } else if (Type == DOUBLE) {
+//       // GDL_DOUBLE
+//     } else if (Type == GDL_DOUBLE) {
     
 //       return reform_template< DDoubleGDL>( e, dim);
     
@@ -636,13 +636,18 @@ namespace lib {
     
   }
 
-
-  BaseGDL* routine_names( EnvT* e) 
+  
+  // note: changes here MUST be reflected in routine_names_reference() as well
+  // because DLibFun of this function is used for scope_varfetch_reference() the keyword
+  // indices must match
+  BaseGDL* routine_names_value( EnvT* e) 
   {
     SizeT nParam=e->NParam();
 
     EnvStackT& callStack = e->Interpreter()->CallStack();
-    DLong curlevnum = callStack.size()-1;
+//     DLong curlevnum = callStack.size()-1;
+	// 'e' is not on the stack
+	DLong curlevnum = callStack.size();
 
     if (e->KeywordSet( "S_FUNCTIONS")) {
       deque<DString> subList;
@@ -748,13 +753,13 @@ namespace lib {
 	int xI = pro->FindVar( varName);
 	//	cout << xI << endl;
 	if (xI != -1) {
-	  BaseGDL*& par = ((EnvT*)(callStack[desiredlevnum-1]))->GetPar( xI);
+	  BaseGDL* par = ((EnvT*)(callStack[desiredlevnum-1]))->GetPar( xI);
 
 	  if( par == NULL)
  		e->Throw( "Variable is undefined: " + varName);
 // 		return NULL;
 	  //	  char* addr = static_cast<char*>(par->DataAddr());
-	  return par->Dup();
+	  return par->Dup(); // no retnew function BUT: ret value is not from current environment
 	}
 	
  	e->Throw( "Variable not found: " + varName);
@@ -766,15 +771,21 @@ namespace lib {
 	DStringGDL* res = new DStringGDL( dimension( nParam), BaseGDL::NOZERO);
 
 	//	cout << "nVar:" << nVar << endl;
-
-	SizeT nCall = callStack[desiredlevnum]->NParam();
-
+	EnvBaseT* desiredCallStack;
+	if( desiredlevnum >= callStack.size())
+		desiredCallStack = e;
+	else
+		desiredCallStack = callStack[ desiredlevnum];
+	
+// 	SizeT nCall = callStack[desiredlevnum]->NParam();
+	SizeT nCall = desiredCallStack->NParam();
+	
 	//	cout << "nCall:" << nCall << "curlevnum:" << curlevnum << endl;
 	for( SizeT i = 0; i<nParam; ++i) {
 	  for( SizeT j = 0; j<nCall; ++j) {
 
 	    if (e->GetParString(i) == 
-		callStack[desiredlevnum]->GetParString(j)) {
+		desiredCallStack->GetParString(j)) {
 	      //	      cout << "Calling param: " << j+1 << endl;
 	      BaseGDL*& p = e->GetPar( i);
 	      if (p == NULL) {
@@ -865,6 +876,143 @@ namespace lib {
       return res;
     }
   }
-    
+
+  
+  
+    BaseGDL** routine_names_reference( EnvT* e) 
+  {
+    SizeT nParam=e->NParam();
+
+    EnvStackT& callStack = e->Interpreter()->CallStack();
+//     DLong curlevnum = callStack.size()-1;
+	// 'e' is not on the stack
+	DLong curlevnum = callStack.size();
+
+    if (e->KeywordSet( "S_FUNCTIONS")) {
+return NULL;
+    }
+
+    if (e->KeywordSet( "S_PROCEDURES")) {
+return NULL;
+    }
+
+    if (e->KeywordSet( "LEVEL")) {
+return NULL;
+    }
+
+    static int variablesIx = e->KeywordIx( "VARIABLES" );
+    static int fetchIx = e->KeywordIx( "FETCH" );
+    static int arg_namesIx = e->KeywordIx( "ARG_NAME" );
+    static int storeIx = e->KeywordIx( "STORE" );
+    bool var=false, fetch=false, arg=false, store=false;
+
+    DLongGDL* level;
+    level = e->IfDefGetKWAs<DLongGDL>( variablesIx);
+    if (level != NULL) {
+      var = true;
+    } else {
+      level = e->IfDefGetKWAs<DLongGDL>( fetchIx);
+      if (level != NULL) {
+	fetch = true;
+      } else {
+	level = e->IfDefGetKWAs<DLongGDL>( arg_namesIx);
+	if (level != NULL) {
+	  arg = true;
+	} else {
+	  level = e->IfDefGetKWAs<DLongGDL>( storeIx);
+	  if (level != NULL) {
+	    store = true;
+	  }
+	}
+      }
+    }
+
+    DString varName;
+
+    if (level != NULL) {
+      DLong desiredlevnum = (*level)[0];
+      if (desiredlevnum <= 0) desiredlevnum += curlevnum;
+      if (desiredlevnum < 1) return NULL;
+      if (desiredlevnum > curlevnum) desiredlevnum = curlevnum;
+
+      DSubUD* pro = static_cast<DSubUD*>(callStack[desiredlevnum-1]->GetPro());
+
+      SizeT nVar = pro->Size(); // # var in GDL for desired level 
+      int nKey = pro->NKey();
+      //cout << "nKey:" << nKey << endl;
+      //cout << "nVar:" << nVar << endl;
+      //cout << pro->Name() << endl;
+
+      if (fetch) { // FETCH
+
+	e->AssureScalarPar<DStringGDL>( 0, varName);
+	varName = StrUpCase( varName);
+	int xI = pro->FindVar( varName);
+	//	cout << xI << endl;
+	if (xI != -1) {
+	  BaseGDL*& par = ((EnvT*)(callStack[desiredlevnum-1]))->GetPar( xI);
+	  return &par; // <-  HERE IS THE DIFFERENCE
+	}
+	
+ 	e->Throw( "Variable not found: " + varName);
+ 	return NULL;
+	
+      } else if (var) { // ARG_NAME
+
+	return NULL;
+
+      } else if (arg) { // ARG_NAME
+
+	return NULL;
+	
+      } else { // STORE
+
+// 	if( nParam != 2)
+// 	  throw GDLException( e->CallingNode(),
+// 			      "ROUTINE_NAMES: Incorrect number of arguments.");
+// 
+// 	// "res" points to variables to be restored
+// 	BaseGDL* res = e->GetParDefined( 1);
+// 
+// 	SizeT s;
+// 	e->AssureScalarPar<DStringGDL>( 0, varName); 
+// 	int xI = pro->FindVar(StrUpCase( varName));
+// 	// cout << "varName: " << StrUpCase( varName) << " xI: " << xI << endl;
+// 	if (xI == -1) {
+// 
+// 	  SizeT u = pro->AddVar(StrUpCase(varName));
+//  	  s = callStack[desiredlevnum-1]->AddEnv();
+// 	  //cout << "AddVar u: " << u << endl;
+// 	  //cout << "AddEnv s: " << s << endl;
+// 
+// 	} else {
+// 	  s = xI;
+// 	  //cout << "FindVar s: " << s << endl;
+// 	}
+// 
+// // 	BaseGDL*& par = ((EnvT*)(callStack[desiredlevnum-1]))->GetPar( s-nKey);
+// 
+//  	((EnvT*)(callStack[desiredlevnum-1]))->GetPar( s-nKey) = res->Dup();
+// 
+// 	//	cout << "par: " << &par << endl << endl;
+// // 	memcpy(&par, &res, sizeof(par)); 
+
+	return NULL;
+      }
+    } else {
+	// Get Compiled Procedures & Functions 
+	return NULL;
+    }
+  }
+
+  
+  
+  
+  
+  
+  
+  
+  
+  
 } // namespace
 
