@@ -27,17 +27,21 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifndef _MSC_VER
 #include <sys/wait.h>
+#endif
 
 #ifdef __APPLE__
 # include <crt_externs.h>
 # define environ (*_NSGetEnviron())
 #else
+#ifdef _MSC_VER
+#define R_OK    4       /* Test for read permission.  */
+#define W_OK    2       /* Test for write permission.  */
+#define F_OK    0       /* Test for existence.  */
+#else
 #include <unistd.h>
 #endif
-
-#ifdef HAVE_LIBWXWIDGETS
-#include <wx/wx.h>
 #endif
 
 #ifdef _OPENMP
@@ -57,6 +61,10 @@
 #endif
 #include <signal.h>
 
+#ifdef HAVE_LIBWXWIDGETS
+#include <wx/wx.h>
+#endif
+
 namespace lib {
  
   using namespace std;
@@ -158,7 +166,7 @@ namespace lib {
     // Type display
     if( !par)
       {
-        os << "UNDEFINED = <Undefined>" << endl;
+        os << "UNDEFINED = !NULL" << endl;
         return;
       }
     os.width(10);
@@ -167,7 +175,7 @@ namespace lib {
     if( !doIndentation) os << "= ";
 
     // Data display
-    if( par->Type() == STRUCT)
+    if( par->Type() == GDL_STRUCT)
       {
         DStructGDL* s = static_cast<DStructGDL*>( par);
         os << "-> ";
@@ -176,7 +184,7 @@ namespace lib {
       }
     else if( par->Dim( 0) == 0)
       {
-        if (par->Type() == STRING)
+        if (par->Type() == GDL_STRING)
 	  {
             // trim string larger than 45 characters
             DString dataString = (*static_cast<DStringGDL*>(par))[0];
@@ -281,31 +289,76 @@ namespace lib {
 	cout << endl;
       }
 
-    if( e->KeywordSet( "LIB"))
+    bool kwLib = e->KeywordSet( "LIB"); 
+    if( kwLib)
       {
 	kw = true;
 
 	deque<DString> subList;
 	SizeT nPro = libProList.size();
-	cout << "Library procedures (" << nPro <<"):" << endl;
 	for( SizeT i = 0; i<nPro; ++i)
-	  subList.push_back(libProList[ i]->ToString());
-
+	{
+	  if( !libProList[ i]->GetHideHelp())
+	    subList.push_back(libProList[ i]->ToString());
+	}
 	sort( subList.begin(), subList.end());
 
-	for( SizeT i = 0; i<nPro; ++i)
+	SizeT nProList = subList.size();
+	cout << "Library procedures (" << nProList <<"):" << endl;
+	for( SizeT i = 0; i<nProList; ++i)
 	  cout << subList[ i] << endl;
 
 	subList.clear();
 
 	SizeT nFun = libFunList.size();
-	cout << "Library functions (" << nFun <<"):" << endl;
 	for( SizeT i = 0; i<nFun; ++i)
-	  subList.push_back(libFunList[ i]->ToString());
-
+	{
+	  if( !libFunList[ i]->GetHideHelp())
+	    subList.push_back(libFunList[ i]->ToString());
+	}
 	sort( subList.begin(), subList.end());
 
+	SizeT nFunList = subList.size();
+	cout << "Library functions (" << nFunList <<"):" << endl;
+	for( SizeT i = 0; i<nFunList; ++i)
+	  cout << subList[ i] << endl;
+      }
+
+    // internal library functions
+    bool kwLibInternal = e->KeywordSet( "LIB_GDL_INTERNAL"); 
+    if( kwLibInternal)
+      {
+	kw = true;
+
+	deque<DString> subList;
+	SizeT nPro = libProList.size();
+	for( SizeT i = 0; i<nPro; ++i)
+	{
+	  if( libProList[ i]->GetHideHelp()) // difference here
+	    subList.push_back(libProList[ i]->ToString());
+	}
+	sort( subList.begin(), subList.end());
+
+	SizeT nProList = subList.size();
+	cout << "NOTE: Internal subroutines are subject to change without notice." << endl;
+	cout << "They should never be called directly from a GDL program." << endl;
+	cout << "Internal library procedures (" << nProList <<"):" << endl;
+	for( SizeT i = 0; i<nProList; ++i)
+	  cout << subList[ i] << endl;
+
+	subList.clear();
+
+	SizeT nFun = libFunList.size();
 	for( SizeT i = 0; i<nFun; ++i)
+	{
+	  if( libFunList[ i]->GetHideHelp()) // difference here
+	    subList.push_back(libFunList[ i]->ToString());
+	}
+	sort( subList.begin(), subList.end());
+
+	SizeT nFunList = subList.size();
+	cout << "Internal library functions (" << nFunList <<"):" << endl;
+	for( SizeT i = 0; i<nFunList; ++i)
 	  cout << subList[ i] << endl;
       }
 
@@ -326,6 +379,10 @@ namespace lib {
     if (isKWSetMemory && (isKWSetProcedures || isKWSetFunctions))
       e->Throw( "Conflicting keywords.");
 
+    bool isKWSetPreferences  = e->KeywordSet( "PREFERENCES");
+    if (isKWSetPreferences && (isKWSetProcedures || isKWSetFunctions))
+      e->Throw( "Conflicting keywords.");
+
     // using this way, we don't need to manage HAVE_READLINE at this level ...
     if (isKWSetRecall) {
       DStringGDL *previous_commands;
@@ -361,7 +418,7 @@ namespace lib {
 	  {
 	    BaseGDL*& par=e->GetPar( i);
 	    DString parString = e->Caller()->GetString( par, true);
-	    if( !par || !isKWSetStructures || par->Type() != STRUCT) {
+	    if( !par || !isKWSetStructures || par->Type() != GDL_STRUCT) {
 	      nlines++;
 	    } else {
 	      DStructGDL* s = static_cast<DStructGDL*>( par);
@@ -382,7 +439,7 @@ namespace lib {
 
       // Setup output return variable
       outputKW = &e->GetKW( outputIx);
-      delete (*outputKW);
+      GDLDelete((*outputKW));
       dimension dim(&nlines, (size_t) 1);
       *outputKW = new DStringGDL(dim, BaseGDL::NOZERO);
     }
@@ -527,14 +584,33 @@ namespace lib {
         ostr.str("");
       }
       return;
-    }
+    } 
+
+    // Excluding keywords which are exclusive is not finished ...
+    if (isKWSetPreferences)
+      {	
+	//cout << "ici 1 " << isKWSetPreferences << endl;
+	std::ostream* ostrp = outputKW == NULL ? &cout : &ostr;
+	*ostrp << "Preferences";
+	// this is not ready ...
+	//*ostrp << GDL_GR_X_QSCREEN::GetValue();
+	if (outputKW == NULL) cout << endl;
+	else
+	  {
+	    (*(DStringGDL *) *outputKW)[nOut++] = ostr.rdbuf()->str();
+	    ostr.str("");
+	  }
+	return;	
+      }
+    
+    //    cout << "ici 2" << isKWSetPreferences << endl;
 
     for( SizeT i=0; i<nParam; i++)
       {
 	BaseGDL*& par=e->GetPar( i);
 	DString parString = e->Caller()->GetString( par, true);
 	// NON-STRUCTURES
-	if( !par || !isKWSetStructures || par->Type() != STRUCT)
+	if( !par || !isKWSetStructures || par->Type() != GDL_STRUCT)
           {
 	    // If no OUTPUT keyword send to stdout
 	    if (outputKW == NULL) {
@@ -713,7 +789,11 @@ namespace lib {
 			AppendIfNeeded(pathToGDL_history, "/");
 			pathToGDL_history += ".gdl";
 			// Create eventially the ".gdl" path in Home
+#ifdef _MSC_VER
+			result = mkdir(pathToGDL_history.c_str());
+#else
 			result = mkdir(pathToGDL_history.c_str(), 0700);
+#endif
 			if (debug)
 			{
 				if (result == 0) cout << "Creation of ~/.gdl PATH "<< endl;
@@ -754,7 +834,7 @@ namespace lib {
       e->Throw( "Expression must be a scalar in this context: "+
 		e->GetString( status));
 
-    DLongGDL* statusL=static_cast<DLongGDL*>(status->Convert2( LONG, 
+    DLongGDL* statusL=static_cast<DLongGDL*>(status->Convert2( GDL_LONG, 
 							       BaseGDL::COPY));
     
     DLong exit_status;
@@ -775,20 +855,137 @@ namespace lib {
 
     e->HeapGC( doPtr, doObj, verbose);
   }
+  
+  void HeapFreeObj( EnvT* env, BaseGDL* var, bool verbose)
+  {
+    if( var == NULL)
+      return;
+    if( var->Type() == GDL_STRUCT)
+    {
+      DStructGDL* varStruct = static_cast<DStructGDL*>( var);
+      DStructDesc* desc = varStruct->Desc();
+      for( SizeT e=0; e<varStruct->N_Elements(); ++e)
+	for( SizeT t=0; t<desc->NTags(); ++t)
+	{
+	    BaseGDL* actElementTag = varStruct->GetTag( t, e);
+	    HeapFreeObj( env, actElementTag, verbose);
+	}
+    }
+    else if( var->Type() == GDL_PTR)
+    {
+      // descent into pointer
+      DPtrGDL* varPtr = static_cast<DPtrGDL*>( var);
+      for( SizeT e=0; e<varPtr->N_Elements(); ++e)
+      {
+	DPtr actPtrID = (*varPtr)[e];
+	if( actPtrID == 0)
+	  continue;
+	
+	BaseGDL* derefPtr = DInterpreter::GetHeap( actPtrID);
+	HeapFreeObj( env, derefPtr, verbose);
+      }
+    }
+    else if( var->Type() == GDL_OBJ)
+    {
+      DObjGDL* varObj = static_cast<DObjGDL*>( var);
+      for( SizeT e=0; e<varObj->N_Elements(); ++e)
+      {
+	DObj actID = (*varObj)[e];
+	if( actID == 0)
+	  continue;
+	
+	if( verbose)
+	{
+	  BaseGDL* derefObj = DInterpreter::GetObjHeap( actID);
+	  help_item( cout, 
+		      derefObj, DString( "<ObjHeapVar")+
+		      i2s(actID)+">",
+		      false);
+	}
+	// 2. free object
+	env->ObjCleanup( actID);
+      }
+    }
+  }
+  
+  void HeapFreePtr( BaseGDL* var, bool verbose)
+  {
+    if( var == NULL)
+      return;
+    if( var->Type() == GDL_STRUCT)
+    {
+      DStructGDL* varStruct = static_cast<DStructGDL*>( var);
+      DStructDesc* desc = varStruct->Desc();
+      for( SizeT e=0; e<varStruct->N_Elements(); ++e)
+	for( SizeT t=0; t<desc->NTags(); ++t)
+	{
+	    BaseGDL* actElementTag = varStruct->GetTag( t, e);
+	    HeapFreePtr( actElementTag, verbose);  // recursive call
+	}
+    }
+    else if( var->Type() == GDL_PTR)
+    {
+      // 1. descent into pointer
+      DPtrGDL* varPtr = static_cast<DPtrGDL*>( var);
+      for( SizeT e=0; e<varPtr->N_Elements(); ++e)
+      {
+	DPtr actPtrID = (*varPtr)[e];
+	if( actPtrID == 0)
+	  continue;
+	
+	BaseGDL* derefPtr = DInterpreter::GetHeap( actPtrID);
+	if( verbose)
+	{
+	  help_item( cout, 
+		      derefPtr, DString( "<PtrHeapVar")+
+		      i2s(actPtrID)+">",
+		      false);
+	}
+	HeapFreePtr( derefPtr, verbose); // recursive call
+      }
+      // 2. free pointer
+      DInterpreter::FreeHeap( varPtr);
+    }
+  }
+  
+  void heap_free( EnvT* e)
+  {
+    static SizeT objIx = e->KeywordIx( "OBJ");
+    static SizeT ptrIx = e->KeywordIx( "PTR");
+    static SizeT verboseIx = e->KeywordIx( "VERBOSE");
+    bool doObj = e->KeywordSet( objIx);
+    bool doPtr = e->KeywordSet( ptrIx);
+    bool verbose =  e->KeywordSet( verboseIx);
+    if( !doObj && !doPtr)
+      doObj = doPtr = true;
+
+    e->NParam( 1);
+    BaseGDL* p0 = e->GetParDefined(0);
+   
+    if( doObj) // do first objects as they may in turn free some pointers
+      HeapFreeObj( e, p0, verbose);
+    if( doPtr)
+      HeapFreePtr( p0, verbose);
+  }
 
   void ptr_free( EnvT* e)
   {
     SizeT nParam=e->NParam();
     for( SizeT i=0; i<nParam; i++)
       {
-	DPtrGDL* par=dynamic_cast<DPtrGDL*>(e->GetPar( i));
-	if( par != NULL) 
-	  {
-	    e->FreeHeap( par);
-	  }
-	else
+	BaseGDL* p = e->GetPar( i);
+	if( p == NULL)
+	{
 	  e->Throw( "Pointer type required"
-		    " in this context: "+e->GetParString(i));
+		    " in this context: "+e->GetParString(i));	  
+	}
+	if( p->Type() != GDL_PTR)
+	{
+	  e->Throw( "Pointer type required"
+		    " in this context: "+e->GetParString(i));	  
+	}
+	DPtrGDL* par=static_cast<DPtrGDL*>(e->GetPar( i));
+	e->FreeHeap( par);
       }
   }
 
@@ -817,8 +1014,6 @@ namespace lib {
   
   void call_procedure( EnvT* e)
   {
-    StackGuard<EnvStackT> guard( e->Interpreter()->CallStack());
-
     int nParam=e->NParam();
     if( nParam == 0)
       e->Throw( "No procedure specified.");
@@ -833,14 +1028,17 @@ namespace lib {
     int proIx=LibProIx( callP);
     if( proIx != -1)
       {
-	e->PushNewEnv( libProList[ proIx], 1);
-	
+// 	e->PushNewEnv( libProList[ proIx], 1);
 	// make the call
-	EnvT* newEnv = static_cast<EnvT*>(e->Interpreter()->CallStack().back());
+// 	EnvT* newEnv = static_cast<EnvT*>(e->Interpreter()->CallStack().back());
+	EnvT* newEnv = e->NewEnv( libProList[ proIx], 1);
+	auto_ptr<EnvT> guard( newEnv);
 	static_cast<DLibPro*>(newEnv->GetPro())->Pro()(newEnv);
       }
     else
       {
+    StackGuard<EnvStackT> guard( e->Interpreter()->CallStack());
+
 	proIx = DInterpreter::GetProIx( callP);
 	
 	e->PushNewEnvUD( proList[ proIx], 1);
@@ -894,7 +1092,7 @@ namespace lib {
 
     BaseGDL** retLun = &e->GetPar( 0);
     
-    delete (*retLun); 
+    GDLDelete((*retLun)); 
     //            if( *retLun != e->Caller()->Object()) delete (*retLun); 
     
     *retLun = new DLongGDL( lun);
@@ -1024,7 +1222,7 @@ namespace lib {
       
       BaseGDL** err = &e->GetKW( errorIx);
       
-      delete (*err); 
+      GDLDelete((*err)); 
 //    if( *err != e->Caller()->Object()) delete (*err); 
       
       *err = new DLongGDL( 1);
@@ -1036,7 +1234,7 @@ namespace lib {
 	BaseGDL** err = &e->GetKW( errorIx);
       
 // 	if( *err != e->Caller()->Object()) delete (*err); 
-	delete (*err); 
+	GDLDelete((*err)); 
       
 	*err = new DLongGDL( 0);
       }
@@ -1078,19 +1276,19 @@ namespace lib {
 
     DUInt port;
     BaseGDL* p2 = e->GetParDefined( 2);
-    if (p2->Type() == STRING) {
+    if (p2->Type() == GDL_STRING) {
       // look up /etc/services
-    } else if (p2->Type() == UINT) {
+    } else if (p2->Type() == GDL_UINT) {
       e->AssureScalarPar<DUIntGDL>( 2, port);
-    } else if (p2->Type() == INT) {
+    } else if (p2->Type() == GDL_INT) {
       DInt p;
       e->AssureScalarPar<DIntGDL>( 2, p);
       port = p;
-    } else if (p2->Type() == LONG) {
+    } else if (p2->Type() == GDL_LONG) {
       DLong p;
       e->AssureScalarPar<DLongGDL>( 2, p);
       port = p;
-    } else if (p2->Type() == ULONG) {
+    } else if (p2->Type() == GDL_ULONG) {
       DULong p;
       e->AssureScalarPar<DULongGDL>( 2, p);
       port = p;
@@ -1137,7 +1335,7 @@ namespace lib {
       
       BaseGDL** err = &e->GetKW( errorIx);
       
-      delete (*err); 
+      GDLDelete((*err)); 
 //    if( *err != e->Caller()->Object()) delete (*err); 
       
       *err = new DLongGDL( 1);
@@ -1149,7 +1347,7 @@ namespace lib {
 	BaseGDL** err = &e->GetKW( errorIx);
       
 // 	if( *err != e->Caller()->Object()) delete (*err); 
-	delete (*err); 
+	GDLDelete((*err)); 
       
 	*err = new DLongGDL( 0);
       }
@@ -1511,7 +1709,7 @@ TRACEOMP( __FILE__, __LINE__)
 #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
 {
 #pragma omp for
-    for( SizeT i=0; i<nEl; ++i)
+    for( int i=0; i<nEl; ++i)
 	StrPut((*dest)[ i], source, pos);
 }
   }
@@ -1615,7 +1813,7 @@ TRACEOMP( __FILE__, __LINE__)
     else
       {
 	// not read only
-	delete oldVar;
+	GDLDelete(oldVar);
 	sysVar->Data() = newVar->Dup();
 
 	// only on first definition
@@ -1677,7 +1875,7 @@ TRACEOMP( __FILE__, __LINE__)
 
   void byteorderDo( EnvT* e, BaseGDL* pIn, SizeT swapSz, DLong p)
 {
-	if( pIn->Type() == STRUCT)
+	if( pIn->Type() == GDL_STRUCT)
 	{
 		DStructGDL* dS=static_cast<DStructGDL*>( pIn);
 		if( dS->Desc()->ContainsStringPtrObject())
@@ -1686,7 +1884,7 @@ TRACEOMP( __FILE__, __LINE__)
 		{
 			BaseGDL* par = dS->GetTag( t);
 			
-			if( par->Type() == STRUCT && par->N_Elements() == 1)
+			if( par->Type() == GDL_STRUCT && par->N_Elements() == 1)
 			{
 				// do tag by tag for scalar struct as memory might not be contigous (
 				byteorderDo( e, par, swapSz, p);
@@ -1717,11 +1915,11 @@ TRACEOMP( __FILE__, __LINE__)
 	}
 	else
 	{
-		if( pIn->Type() == STRING)
+		if( pIn->Type() == GDL_STRING)
 		e->Throw( "STRING type not allowed in this context: "+e->GetParString(p));		    
-		if( pIn->Type() == OBJECT)
+		if( pIn->Type() == GDL_OBJ)
 		e->Throw( "Object type not allowed in this context: "+e->GetParString(p));		    
-		if( pIn->Type() == PTR)
+		if( pIn->Type() == GDL_PTR)
 		e->Throw( "PTR type not allowed in this context: "+e->GetParString(p));		    
 	
 		BaseGDL*& par = pIn;
@@ -1801,13 +1999,13 @@ TRACEOMP( __FILE__, __LINE__)
 
  	byteorderDo( e, par, swapSz, p);
 
-/*	if( par->Type() == STRING)
+/*	if( par->Type() == GDL_STRING)
 	  e->Throw( "STRING type not allowed in this context: "+e->GetParString(p));		    
-	if( par->Type() == OBJECT)
+	if( par->Type() == GDL_OBJ)
 	  e->Throw( "Object type not allowed in this context: "+e->GetParString(p));		    
-	if( par->Type() == PTR)
+	if( par->Type() == GDL_PTR)
 	  e->Throw( "PTR type not allowed in this context: "+e->GetParString(p));		    
-	if( par->Type() == STRUCT)
+	if( par->Type() == GDL_STRUCT)
 	{
 		if( static_cast<DStructGDL*>( par)->Desc()->ContainsStringPtrObject())
 		  e->Throw( "Structs must not contain PTR, OBJECT or STRING tags: "+e->GetParString(p));		    
@@ -1859,7 +2057,7 @@ TRACEOMP( __FILE__, __LINE__)
       DString strArg = strEnv.substr(pos+1, len - pos - 1);
       strEnv = strEnv.substr(0, pos);
       // putenv() is POSIX unlike setenv()
-      #if defined(__hpux__)
+      #if defined(__hpux__) || defined(_MSC_VER)
       int ret = putenv((strEnv+"="+strArg).c_str());
       #else
       int ret = setenv(strEnv.c_str(), strArg.c_str(), 1);
@@ -1933,6 +2131,8 @@ TRACEOMP( __FILE__, __LINE__)
       }
   }
 
+#ifndef _MSC_VER
+ 
   // helper function for spawn_pro
   static void child_sighandler(int x){
     pid_t pid;
@@ -2108,7 +2308,7 @@ TRACEOMP( __FILE__, __LINE__)
              e->Throw( "SPAWN: Failed to open new LUN: Unit already open. Unit: "+i2s( unit_lun));
            fileUnits[ unit_lun-1].PutVarLenVMS( false);
  
-           // Here we invoke the black arts of converting from a C FILE*/fd to an fstream object
+           // Here we invoke the black arts of converting from a C FILE*fd to an fstream object
            __gnu_cxx::stdio_filebuf<char> *frb_p;
            frb_p = new __gnu_cxx::stdio_filebuf<char>(coutF, std::ios_base::in);
  
@@ -2207,6 +2407,7 @@ TRACEOMP( __FILE__, __LINE__)
         }
       }
   }
+#endif
 
   void replicate_inplace_pro( EnvT* e)
   {
@@ -2268,22 +2469,23 @@ TRACEOMP( __FILE__, __LINE__)
 	    p5 = e->GetNumericParDefined( 5);
 	  }
 
-	ArrayIndexVectorT* ixList = new ArrayIndexVectorT();
-	auto_ptr< ArrayIndexVectorT> ixList_guard( ixList);
+// 	ArrayIndexVectorT* ixList = new ArrayIndexVectorT();
+// 	auto_ptr< ArrayIndexVectorT> ixList_guard( ixList);
+	ArrayIndexVectorT ixList; 
 // 	BaseGDL* loc1 = p3->Dup();
 // 	loc1->SetDim (dimension( loc1->N_Elements()));
 //	ixList->reserve( p3->N_Elements());
 	for (size_t i=0; i<p3->N_Elements(); i++)
 	  if( (i+1) == d1)
-	    ixList->push_back( new ArrayIndexAll());
+	    ixList.push_back( new ArrayIndexAll());
 	  else if( (i+1) == d2)
-	    ixList->push_back( new CArrayIndexIndexed( p5, true));
+	    ixList.push_back( new CArrayIndexIndexed( p5, true));
 	  else
-	    ixList->push_back( new CArrayIndexScalar( (*p3)[ i]));//p3->NewIx(i)));
+	    ixList.push_back( new CArrayIndexScalar( (*p3)[ i]));//p3->NewIx(i)));
 	ArrayIndexListT* ixL;
-	MakeArrayIndex( ixList, &ixL);
+	MakeArrayIndex( &ixList, &ixL);
 	auto_ptr< ArrayIndexListT> ixL_guard( ixL);
-	ixL->AssignAt( p0, p1);	
+	ixL->AssignAt( p0, p1);
 	return;
       }
   }
@@ -2372,7 +2574,9 @@ TRACEOMP( __FILE__, __LINE__)
       e->Throw("Value of Julian date (" + i2s((*p0)[i]) + ") is out of allowed range.");
 
     // preparing output (loop order important when all parameters point the same variable)
-    BaseGDL** ret[nParam - 1];
+    //BaseGDL** ret[nParam - 1];
+    BaseGDL*** ret;
+    ret = (BaseGDL***)malloc((nParam-1)*sizeof(BaseGDL**));
     for (int i = nParam - 2; i >= 0; i--) if (global[i]) 
     {
       ret[i] = &e->GetPar(i + 1);
@@ -2380,7 +2584,7 @@ TRACEOMP( __FILE__, __LINE__)
       if 
       (
         *ret[i] == NULL || 
-        (*ret[i])->Type() != (i < 5 ? LONG : DOUBLE) || 
+        (*ret[i])->Type() != (i < 5 ? GDL_LONG : GDL_DOUBLE) || 
         (*ret[i])->N_Elements() != nEl 
       )
       {
@@ -2452,7 +2656,7 @@ TRACEOMP( __FILE__, __LINE__)
       if (global[6 - 1]) 
         (*static_cast<DDoubleGDL*>(*ret[6 - 1]))[i] = F * 86400;
     }
-    
+    free((void *)ret);
   }
   
 } // namespace
