@@ -69,6 +69,7 @@ namespace lib {
   void stop( EnvT* e);
 
   void heap_gc( EnvT* e);
+  void heap_free( EnvT* e);
 
   void defsysv( EnvT* e);
 
@@ -76,7 +77,9 @@ namespace lib {
 
   void byteorder( EnvT* e);
 
+#ifndef _MSC_VER
   void spawn_pro( EnvT* e);
+#endif
 
   // the following by Peter Messmer 
   // (messmer@users.sourceforge.net)
