@@ -27,6 +27,8 @@
 #include "str.hpp"
 #include "dcommon.hpp"
 #include "GDLInterpreter.hpp"
+#include "overload.hpp"
+#include "prognodeexpr.hpp"
 
 // print out AST tree
 //#define GDL_DEBUG
@@ -42,7 +44,7 @@ using namespace std;
 
 DCompiler::DCompiler(const string& f, EnvBaseT* e, const std::string& sub)
   : actualFile(f), subRoutine( sub), env(e), pro(NULL), 
-    activeProCompiled( false), tree( NULL)
+    activeProCompiled( false), nCompileErrors(0), tree( NULL)
 {
   // interactive mode?
   if( env != NULL) pro=dynamic_cast<DSubUD*>(env->GetPro());
@@ -193,7 +195,6 @@ void DCompiler::EndPro() // inserts in proList
 
   if( name != "$MAIN$" || o != "")
     {
-      
       ProListT *searchList;
       if( o == "")
 	searchList= &proList;
@@ -207,8 +208,19 @@ void DCompiler::EndPro() // inserts in proList
 	      structList.push_back( dStruct);
 	    }
 	  searchList = &dStruct->ProList();
+	  
+	  // operator overlaoding
+	  // consider only PRO operators
+	  int operatorIndex = OverloadOperatorIndexPro( name);
+	  // insert in operator list if operator
+	  if( operatorIndex != -1)
+	  {
+	    // will do nothing if not derived from GDL_OBJECT
+	    dStruct->SetOperator( operatorIndex, pro);
+	  }
 	}
       
+	  
       // search/replace in proList
       ProListT::iterator p=find_if((*searchList).begin(),(*searchList).end(),
 				   Is_eq<DPro>(name));
@@ -229,7 +241,7 @@ void DCompiler::EndPro() // inserts in proList
       else
       {
 	(*searchList).push_back(static_cast<DPro*>(pro));
-        WarnAboutObsoleteRoutine(pro->ObjectName());
+	WarnAboutObsoleteRoutine(pro->ObjectName());
       }
     }
 
@@ -263,6 +275,16 @@ void DCompiler::EndFun() // inserts in funList
 	  structList.push_back( dStruct);
 	}
       searchList = &dStruct->FunList();
+
+      // operator overlaoding
+      // consider only FUNCTION operators
+      int operatorIndex = OverloadOperatorIndexFun( name);
+      // insert in operator list if operator
+      if( operatorIndex != -1)
+      {
+	// will do nothing if not derived from GDL_OBJECT
+	dStruct->SetOperator( operatorIndex, pro);
+      }
     }
 
   // search/replace in funList
@@ -371,10 +393,35 @@ void DCompiler::CommonDecl(const string& N)
   pro->AddCommon(c);
 }
 
-BaseGDL* DCompiler::Constant( RefDNode n)
+// always returns new variable
+BaseGDL* DCompiler::ConstantIndex( RefDNode n)
 {
   if( n->getType() == CONSTANT)
-    return n->CData();
+  {
+    assert( n->CData() != NULL);
+    return n->CData()->Dup();
+  }
+  if( n->getType() == ARRAYDEF_CONST)
+  {
+//   cout << "RefDNode:" << endl;
+//   antlr::print_tree pt;
+//   pt.pr_tree(static_cast<antlr::RefAST>( n));
+//   cout << endl;
+
+    // must be compiled here
+    ARRAYDEFNode* c = new ARRAYDEFNode( n);
+    auto_ptr< ARRAYDEFNode> guard( c);
+    assert( c->ConstantArray());
+
+//   cout << "ProgNodeP:" << endl;
+//   pt.pr_tree(c);
+//   cout << endl;
+
+    BaseGDL* cData = c->Eval();
+    assert( cData != NULL);
+    return cData;
+  }
+  
   return NULL;
 }
 
@@ -420,8 +467,10 @@ RefDNode DCompiler::ByReference(RefDNode nIn)
   t=n->getType();
 
   // only var, common block var and deref ptr are passed by reference
+  // note: trinary op is REF_CHECK 
   // *** see AssignReplace(...)
-  if( !assignReplace && t != VAR && t != VARPTR && t != DEREF) return null; 
+  if( !assignReplace && t != VAR && t != VARPTR && t != DEREF) // && t != QUESTION) 
+    return null; 
 
 // #ifdef GDL_DEBUG
 //   cout << "ByReference: out:" << endl;
