@@ -201,8 +201,8 @@ SizeT nOp = kxLimit * kyLimit;
 	}
 
 	//std::cout << "XPutPixel: "<<kx<<"  "<< dev->height-ky-1 << std::endl;
-	//	if( ky < dev->height && kx < dev->width)
 	// TODO check if XPutPixel() and XGetPixel() are thread save
+	if( ky < dev->height && kx < dev->width)
 	XPutPixel(ximg, kx, dev->height-1-ky, curcolor.pixel);
       }
     }
@@ -265,6 +265,15 @@ SizeT nOp = kxLimit * kyLimit;
     if (wIx >= 0 && wIx < winList.size()) winList[ wIx]->Lower();
   }
 
+  void IconicWin( int wIx)
+  {
+    if (wIx >= 0 && wIx < winList.size()) winList[ wIx]->Iconic();
+  }
+  void DeIconicWin( int wIx)
+  {
+    if (wIx >= 0 && wIx < winList.size()) winList[ wIx]->DeIconic();
+  }
+
   // process user deleted windows
   // should be done in a thread
   void ProcessDeleted()
@@ -417,10 +426,19 @@ public:
     PLINT xoff; PLINT yoff;
     winList[ wIx]->gpage( xp, yp, xleng, yleng, xoff, yoff);
 
+    int debug=0;
+    if (debug) cout <<xp<<" "<<yp<<" "<<xleng<<" "<<yleng<<" "<<xoff<<" "<<yoff<<endl;
+
+    DLong xMaxSize, yMaxSize;
+    DeviceX::MaxXYSize(&xMaxSize, &yMaxSize);
+
     xleng = xSize;
     yleng = ySize;
     xoff  = xPos;
-    yoff  = yPos;
+    yoff  = yMaxSize-(yPos+ySize);
+    if (yoff <= 0) yoff=1;
+    
+    if (debug) cout <<xp<<" "<<yp<<" "<<xleng<<" "<<yleng<<" "<<xoff<<" "<<yoff<<endl;
 
     winList[ wIx]->spage( xp, yp, xleng, yleng, xoff, yoff);
 
@@ -509,8 +527,9 @@ public:
     int wLSize = winList.size();
     if (ix >= wLSize || ix < 0 || winList[ ix] == NULL) return false;
  
-    if (show) RaiseWin(ix);
-    else LowerWin(ix);
+    if (show) RaiseWin(ix); else LowerWin(ix);
+    
+    if (iconic) IconicWin(ix); else DeIconicWin(ix);
 
     return true;
   }
@@ -567,7 +586,18 @@ public:
     {
       Display* display = XOpenDisplay(NULL);
       if (display == NULL) ThrowGDLException("Cannot connect to X server");
-      decomposed = (DefaultDepth(display, DefaultScreen(display)) >= 15 ? true : false);
+
+      int Depth;
+      Depth=DefaultDepth(display, DefaultScreen(display));      
+      decomposed = (Depth >= 15 ? true : false);
+      DLong toto=16777216;
+      if (Depth == 24) 
+	(*static_cast<DLongGDL*>(dStruct->GetTag(n_colorsTag)))[0] = toto;
+      int debug=0;
+      if (debug) {
+	cout << "Display Depth " << Depth << endl;
+	cout << "n_colors " << toto << endl;
+      }
       XCloseDisplay(display);
     }
     if( decomposed) return 1;
@@ -579,7 +609,18 @@ public:
 
   BaseGDL* TVRD( EnvT* e)
   {
+    // AC 17 march 2012: needed to catch the rigth current window (wset ...)
+    DLong wIx = -1;
     Graphics* actDevice = Graphics::GetDevice();
+    wIx = actDevice->ActWin();
+    bool success = actDevice->WSet( wIx);
+    int debug=0;
+    if (debug) cout << "wIx :" << wIx << " " << success << endl;
+
+    //everywhere we use XGetImage we need to set an error handler, since GTK crashes on every puny
+    //BadMatch error, and if you read the XGetImage doc you'll see that such errors are prone to happen
+    //as soon as part of the window is obscured.
+    int (*oldErrorHandler)(Display*, XErrorEvent*);
 
     XwDev *dev = (XwDev *) plsc->dev;
     if( dev == NULL || dev->xwd == NULL)
@@ -597,20 +638,20 @@ public:
     if (e->KeywordSet("WORDS")) e->Throw( "WORDS keyword not yet supported.");
 
     /* this variable will contain the attributes of the window. */
-    //    XWindowAttributes win_attr;
+      XWindowAttributes win_attr;
 
     /* query the window's attributes. */
-    // Status rc = XGetWindowAttributes(xwd->display, dev->window, &win_attr);
-    // SizeT xSize = win_attr.width;
-    // SizeT ySize = win_attr.height;
-
-    int xSize, ySize, xPos, yPos;
+     Status rc = XGetWindowAttributes(xwd->display, dev->window, &win_attr);
+    unsigned int xSize = win_attr.width;
+    unsigned int ySize = win_attr.height;
+    //cout<<xSize<<" "<<ySize<<endl;
+    //int xPos, yPos;
     int actWin = /* actDevice-> */ ActWin();
-    bool success = /* actDevice-> */ WSize( actWin, &xSize, &ySize, &xPos, &yPos);
-
-    ximg = XGetImage( xwd->display, dev->window, 0, 0, 
+   // bool success = /* actDevice-> */ WSize( actWin, &xSize, &ySize, &xPos, &yPos);
+    oldErrorHandler = XSetErrorHandler(GetImageErrorHandler);
+    ximg = XGetImage( xwd->display, dev->window, (int) 0, (int) 0,
 		      xSize, ySize, AllPlanes, ZPixmap);
-
+    XSetErrorHandler(oldErrorHandler);
 
     SizeT dims[3];
     
@@ -623,57 +664,129 @@ public:
     e->AssureLongScalarKWIfPresent( "CHANNEL", channel);
     if (channel > 3) e->Throw("Value of Channel is out of allowed range.");
 
+    unsigned int x0u=0;
+    unsigned int y0u=0;
+    unsigned int Nxu=xSize-1;
+    unsigned int Nyu=ySize-1;
+    unsigned int Channelu=0;
+
+    int nParam = e->NParam();
+    if (nParam >= 1) {
+      DLongGDL* x0 = e->GetParAs<DLongGDL>(0);
+      x0u=(*x0)[0];
+    }
+    if (nParam >= 2) {
+      DLongGDL* y0 = e->GetParAs<DLongGDL>(1);
+      y0u=(*y0)[0];
+    }
+    if (nParam >= 3) {
+      DLongGDL* Nx = e->GetParAs<DLongGDL>(2);
+      Nxu=(*Nx)[0];
+    }
+    if (nParam >= 4) {
+      DLongGDL* Ny = e->GetParAs<DLongGDL>(3);
+      Nyu=(*Ny)[0];
+    }
+    if (nParam == 5) {
+      DLongGDL* Channel = e->GetParAs<DLongGDL>(4);
+      Channelu=(*Channel)[0];
+    }
+
+    if (debug) {
+      cout << "hello"<<endl;
+      //     cout << (*x0)[0] <<" "<< (*x1)[0] <<" "<< (*Nx)[0] <<" "<< (*Ny)[0] <<" "<< (*Channel)[0] <<endl;
+      cout << x0u <<" "<< y0u <<" "<< Nxu <<" "<< Nyu <<" "<< Channelu <<endl;
+    }
+
     if (tru == 0 || channel != -1) {
       dims[0] = xSize;
       dims[1] = ySize;
       dimension dim(dims, (SizeT) 2);
       res = new DByteGDL( dim, BaseGDL::ZERO);
 
-      if (channel <= 0) { 
-	DByte mx, mx1;
-	for( SizeT i=0; i<xSize*ySize; ++i) {
-	  mx = (DByte) ximg->data[4*i];
-	  for( SizeT j=1; j<=2; ++j) {
-	    mx1 = (DByte) ximg->data[4*i+j];
-	    if (mx1 > mx) mx = mx1;
-	  }
-	  (*res)[ xSize*ySize-1-i] = mx;
-	}
+      if (ximg == NULL) return res;
+
+      if (channel <= 0) {
+        DByte mx, mx1;
+        for (SizeT i = 0; i < xSize * ySize; ++i) {
+          mx = (DByte) ximg->data[4 * i];
+          for (SizeT j = 1; j <= 2; ++j) {
+            mx1 = (DByte) ximg->data[4 * i + j];
+            if (mx1 > mx) mx = mx1;
+          }
+          (*res)[ xSize * ySize - 1 - i] = mx;
+        }
       } else {
-	for( SizeT i=0; i<xSize*ySize; ++i) {
-	  (*res)[ xSize*ySize-1-i] = ximg->data[4*i+(3-channel)];
-	}
+        for (SizeT i = 0; i < xSize * ySize; ++i) {
+          (*res)[ xSize * ySize - 1 - i] = ximg->data[4 * i + (3 - channel)];
+        }
       }
 
       // Reflect about y-axis
-      for( SizeT i=0; i<ySize; ++i) {
-	for( SizeT j=0; j<xSize/2; ++j) {
-	  DByte c = (*res)[ i*xSize+(xSize-1-j)];
-	  (*res)[ i*xSize+(xSize-1-j)] = (*res)[ i*xSize+j];
-	  (*res)[ i*xSize+j] = c;
-	}
+      for (SizeT i = 0; i < ySize; ++i) {
+        for (SizeT j = 0; j < xSize / 2; ++j) {
+          DByte c = (*res)[ i * xSize + (xSize - 1 - j)];
+          (*res)[ i * xSize + (xSize - 1 - j)] = (*res)[ i * xSize + j];
+          (*res)[ i * xSize + j] = c;
+        }
       }
       return res;
 
     } else {
       if (tru > 3) e->Throw("Value of TRUE keyword is out of allowed range.");
 
-      dims[0] = 3;
-      dims[1] = xSize;
-      dims[2] = ySize;
-      dimension dim(dims, (SizeT) 3);
-      res = new DByteGDL( dim, BaseGDL::NOZERO);
+      // AC 17 march 2012
+      // below something is wrong
+      // window, 3,xsize=10, ysize=7
+      // plot, findgen(10), col='ff'x, back='ff'x ;; all in red
+      // uu=tvrd(/tru)
+      // plot, uu(0,*,*) ; should be at 255, but not all :(
+      // It seems to be related to refreshment of the screen !
 
-      for( SizeT i=0; i<ySize; ++i) {
-	for( SizeT j=0; j<xSize; ++j) {
-	  for( SizeT k=0; k<4; ++k) {
-	    if (k < 3) {
-	      (*res)[(ySize-i-1)*xSize*3 + j*3 + 2-k] = 
-		ximg->data[i*xSize*4 + j*4 + k];
+      if (nParam ==0)
+	{
+	  // AC 17 march 2012 : we may have a mirror effect here
+	  dims[0] = 3;
+	  dims[1] = xSize;
+	  dims[2] = ySize;
+	  dimension dim(dims, (SizeT) 3);
+	  res = new DByteGDL(dim, BaseGDL::NOZERO);
+	  if (ximg == NULL) return res;
+	  
+	  SizeT jj=0;
+	  for (SizeT i = 0; i < xSize; ++i) {
+	    for (SizeT j = 0; j < ySize; ++j) {
+	      for (SizeT k = 0; k < 4; ++k) {
+		if (k < 3) {
+		  (*res)[xSize*j * 3 + i * 3 + 2 - k] =
+		        ximg->data[j * xSize * 4 + i * 4 + k];
+		}
+	      }
+	    }
+	  }
+	}
+      else
+	{
+	  // AC 17 march 2012 : we may have a mirror effect here
+	  dims[0] = 3;
+	  dims[1] = Nxu;
+	  dims[2] = Nyu;
+	  //cout << dims[1] << " " << dims[2] <<endl;
+	  dimension dim(dims, (SizeT) 3);
+	  res = new DByteGDL(dim, BaseGDL::NOZERO);
+	  if (ximg == NULL) return res;
+	  
+	  for (SizeT i = x0u; i < x0u+Nxu; ++i) {
+	    for (SizeT j = y0u; j < y0u+Nyu; ++j) {
+	      for (SizeT k = 0; k < 4; ++k) {
+		if (k < 3) {		  
+		  (*res)[dims[1]*(j-y0u) * 3 + (i-x0u) * 3 + 2 - k] =
+		    ximg->data[j * xSize * 4 + i * 4 + k];
+		}
+	      }
 	    }
 	  }
 	}
-      }
     }
 
     XDestroyImage(ximg);
@@ -703,8 +816,8 @@ public:
     GDLGStream* actStream = GetStream();
     if( actStream == NULL)
       {
-	std::cerr << "TV: Internal error: plstream not set." << std::endl;
-	exit( EXIT_FAILURE);
+		std::cerr << "TV: Internal error: plstream not set." << std::endl;
+		exit( EXIT_FAILURE);
       }
 
     //    actStream->NextPlot( false);
@@ -729,8 +842,8 @@ public:
 
     //DByteGDL* p0B = e->GetParAs<DByteGDL>( 0);
     DByteGDL* p0B;
-    p0B =static_cast<DByteGDL*>(p0->Convert2(BYTE,BaseGDL::COPY));
-    //e->Guard( p0B);
+    p0B =static_cast<DByteGDL*>(p0->Convert2(GDL_BYTE,BaseGDL::COPY));
+    e->Guard( p0B);
     
     int width, height;
     DLong tru=0;
@@ -884,6 +997,21 @@ public:
     }   
 #endif
   }
+  
+  static void MaxXYSize(DLong *xSize, DLong *ySize)
+  {
+    *ySize = 640;
+    *ySize = 512;
+#ifdef HAVE_X
+    Display* display = XOpenDisplay(NULL);
+    if (display != NULL)
+    {
+      *xSize = DisplayWidth(display, DefaultScreen(display));
+      *ySize = DisplayHeight(display, DefaultScreen(display));
+      XCloseDisplay(display);
+    }
+#endif
+  }
 
 };
 
