@@ -27,6 +27,22 @@
 #define SETOPT setopt
 #endif
 
+#ifdef _MSC_VER
+/* replacement of Unix rint() for Windows */
+static int rint (double x)
+{
+char *buf;
+int i,dec,sig;
+
+buf = _fcvt(x, 0, &dec, &sig);
+i = atoi(buf);
+if(sig == 1) {
+i = i * -1;
+}
+return(i);
+}
+#endif
+
 class DeviceZ: public Graphics
 {
   GDLZStream*     actStream;
@@ -277,7 +293,7 @@ public:
     //    Graphics* actDevice = Graphics::GetDevice();
     SizeT nParam=e->NParam( 1); 
 
-    GDLGStream* actStream = /* actDevice-> */ GetStream();
+    GDLGStream* actStream = GetStream();
 
     //    actStream->NextPlot( false); // JMG
 
