@@ -18,7 +18,13 @@
 #include "includefirst.hpp"
 
 #include <iostream>
+#ifdef _MSC_VER
+#include <io.h> // isatty, windows
+#define feclearexcept(e)
+#define FE_ALL_EXCEPT
+#else
 #include <unistd.h> // isatty
+#endif
 
 //#include <wordexp.h>
 
@@ -37,7 +43,7 @@
 
 // print out AST tree
 //#define GDL_DEBUG
-#undef GDL_DEBUG
+//#undef GDL_DEBUG
 
 // print out an exspression entered at teh comand line
 //#define 	AUTO_PRINT_EXPR
@@ -109,11 +115,13 @@ RetCode GDLInterpreter::NewInterpreterInstance( SizeT lineOffset)
 
 DStructGDL* GDLInterpreter::ObjectStruct( BaseGDL* self, ProgNodeP mp)
 {
-  DObjGDL* obj=dynamic_cast<DObjGDL*>(self);
-  if( obj == NULL) 
+  DType selfType = self->Type();
+  if( selfType != GDL_OBJ) 
     throw GDLException( mp, "Object reference type"
 			" required in this context: "+Name(self));
-  
+
+  DObjGDL* obj=static_cast<DObjGDL*>(self);
+
   SizeT o;
   if( !obj->Scalar( o))
     throw GDLException( mp, "Object reference"
@@ -585,7 +593,6 @@ DInterpreter::CommandCode DInterpreter::CmdRun( const string& command)
   //  return CC_OK;
 }
 
-
 // execute GDL command (.run, .step, ...)
 DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
 {
@@ -655,30 +662,46 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
     }
   if( cmd( "SKIP"))
     {
-      cout << "SKIP not implemented yet." << endl;
-      return CC_OK;
+      DLong sCount;
+      if( args == "")
+      {
+	  sCount = 1;
+      }
+      else
+      {
+	const char* cStart=args.c_str();
+	char* cEnd;
+	sCount = strtol(cStart,&cEnd,10);
+	if( cEnd == cStart)
+	{
+	  cout << "Type conversion error: Unable to convert given STRING: '"+args+"' to LONG." << endl;
+	  return CC_OK;
+	}
+      }
+      stepCount = sCount;
+      return CC_SKIP;
     }
   if( cmd( "STEP"))
     {
-      	    DLong sCount;
-      	    if( args == "")
-      	    {
-				sCount = 1;
-      	    }
-      	    else
-      	    {
-				const char* cStart=args.c_str();
-				char* cEnd;
-				sCount = strtol(cStart,&cEnd,10);
-				if( cEnd == cStart)
-				{
-				cout << "Type conversion error: Unable to convert given STRING: '"+args+"' to LONG." << endl;
-				return CC_OK;
-				}
-      	      }
-      	      stepCount = sCount;
-			  debugMode = DEBUG_STEP;
-			  return CC_STEP;
+      DLong sCount;
+      if( args == "")
+      {
+	  sCount = 1;
+      }
+      else
+      {
+	  const char* cStart=args.c_str();
+	  char* cEnd;
+	  sCount = strtol(cStart,&cEnd,10);
+	  if( cEnd == cStart)
+	  {
+	    cout << "Type conversion error: Unable to convert given STRING: '"+args+"' to LONG." << endl;
+	    return CC_OK;
+	  }
+	}
+	stepCount = sCount;
+	debugMode = DEBUG_STEP;
+	return CC_STEP;
     }
   if( cmd( "STEPOVER"))
     {
@@ -949,7 +972,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
     {
 	  env->ResizeForLoops( nForLoopsIn);
       
-      cerr << "Unhandled GDL exception: " <<  e.toString() << endl;;
+      cerr << "Unhandled GDL exception: " <<  e.toString() << endl;
       return CC_OK;
     }
   catch( ANTLRException& e)
@@ -1046,7 +1069,9 @@ string DInterpreter::GetLine()
     // make a string
     line = cline;
     
+#ifndef _MSC_VER
     free(cline);        // done here for compatibility with readline
+#endif
   
     StrTrim(line);
   } while( line == "");
@@ -1089,30 +1114,36 @@ else
 // inner loop (called via Control-C, STOP, error)
 RetCode DInterpreter::InnerInterpreterLoop(SizeT lineOffset)
 {
-
-  bool runCmd = false;
+  ProgNodeP retTreeSave = _retTree;
   for (;;) {
     feclearexcept(FE_ALL_EXCEPT);
 
-//     try
-//       {
-	DInterpreter::CommandCode ret=ExecuteLine(NULL, lineOffset);
-	if( ret == CC_RETURN) return RC_RETURN;
-	if( ret == CC_CONTINUE) return RC_OK; 
-	if( ret == CC_STEP) return RC_OK;
-//       }
-//     catch( RetAllException&)
-//       {
-//  	throw;
-//       }
-    //     catch( exception& e)
-    //       {
-    // 	cerr << "InnerInterpreterLoop: Exception: " << e.what() << endl;
-    //       }
-    //     catch (...)
-    //       {	
-    // 	cerr << "InnerInterpreterLoop: Unhandled Error." << endl;
-    //       }
+    DInterpreter::CommandCode ret=ExecuteLine(NULL, lineOffset);
+
+    _retTree = retTreeSave; // on return, _retTree should be kept
+
+    if( ret == CC_SKIP)
+    {
+      for( int s=0; s<stepCount; ++s)
+      {
+	if( _retTree == NULL)
+	  break;
+	
+	_retTree = _retTree->getNextSibling();
+      }
+//       cout << ".SKIP " << stepCount << "   " << _retTree << endl;
+
+      stepCount = 0;
+      retTreeSave = _retTree;
+      // we stay at the command line here
+      if( _retTree == NULL)
+	Message( "Can't continue from this point.");
+      else
+	DebugMsg( _retTree, "Skipped to: ");
+    }
+    else if( ret == CC_RETURN) return RC_RETURN;
+    else if( ret == CC_CONTINUE) return RC_OK; 
+    else if( ret == CC_STEP) return RC_OK;
   }
 }
 
@@ -1392,11 +1423,15 @@ historyIntialized = true;
 	  {
 	    DInterpreter::CommandCode ret=ExecuteLine();
 
-		// stop steppig when at main level
-		stepCount = 0;
-		debugMode = DEBUG_CLEAR;
+	    // stop steppig when at main level
+	    stepCount = 0;
+	    debugMode = DEBUG_CLEAR;
 
-	    if( ret == CC_CONTINUE)
+	    if( ret == CC_SKIP)
+	    {
+	      Message( "Can't continue from this point.");
+	    }		    
+	    else if( ret == CC_CONTINUE)
 	      {
 		if( static_cast<DSubUD*>
 		    (callStack.back()->GetPro())->GetTree() != NULL)
