@@ -117,6 +117,7 @@ protected:
   IDList              key;    // keyword names (IDList: typedefs.hpp)
                               // (KEYWORD_NAME=keyword_value)
   int                 nPar;   // number of parameters (-1 = infinite)
+  int                 nParMin;  // minimum number of parameters (-1 = infinite)
 
   ExtraType           extra;
   int                 extraIx; // index of extra keyword
@@ -126,7 +127,7 @@ protected:
 
 public:
   DSub( const std::string& n, const std::string& o=""): 
-    name(n), object(o), nPar(0), extra(NONE), extraIx(-1) 
+    name(n), object(o), key(), nPar(0), nParMin(0), extra(NONE), extraIx(-1), warnKey() 
   {}
 
   virtual ~DSub(); // polymorphism
@@ -168,12 +169,14 @@ public:
 
   int   NKey() const { return key.size();}
   int   NPar() const { return nPar;}
+  int   NParMin() const { return nParMin;}
 
   //  bool AKey() { return aKey;} // additional keywords allowed
 
   friend class EnvBaseT;
   friend class EnvT;
   friend class ExtraT;
+  friend void SetupOverloadSubroutines(); // overload base class methods
 };
 
 // Lib pro/fun ********************************************************
@@ -181,16 +184,22 @@ class EnvT;
 
 typedef void     (*LibPro)(EnvT*);
 typedef BaseGDL* (*LibFun)(EnvT*);
+typedef BaseGDL* (*LibFunDirect)(BaseGDL* param,bool canGrab);
 
 // library procedure/function (in cases both are handled the same way)
 class DLib: public DSub
 {
+  bool hideHelp; // if set HELP,/LIB will not list this subroutine
+  
 public:
   DLib( const std::string& n, const std::string& o, const int nPar_,
 	const std::string keyNames[],
-	const std::string warnKeyNames[]);
+	const std::string warnKeyNames[], const int nParMin_);
 
   virtual const std::string ToString() = 0;
+  
+  bool GetHideHelp() const { return hideHelp;}
+  void SetHideHelp( bool v) { hideHelp = v;}
 };
 
 // library procedure
@@ -207,12 +216,12 @@ public:
   // on which a value is returned.
   DLibPro( LibPro p, const std::string& n, const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
-	   const std::string warnKeyNames[]=NULL);
+	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
   DLibPro( LibPro p, const std::string& n, const std::string& o, 
 	   const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
-	   const std::string warnKeyNames[]=NULL);
+	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
   LibPro Pro() { return pro;}
 
@@ -226,40 +235,56 @@ class DLibFun: public DLib
 public:
   DLibFun( LibFun f, const std::string& n, const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
-	   const std::string warnKeyNames[]=NULL);
+	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
   DLibFun( LibFun f, const std::string& n, const std::string& o, 
 	   const int nPar_=0, 
 	   const std::string keyNames[]=NULL,
-	   const std::string warnKeyNames[]=NULL);
+	   const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
   LibFun Fun() { return fun;}
 
   const std::string ToString();
 
   virtual bool RetNew() { return false;}
+  virtual bool DirectCall() { return false;}
 };
 
 // library function which ALWAYS return a new value
 // (as opposite to returning an input value)
 class DLibFunRetNew: public DLibFun
 {
-  bool   retConstant;
+  bool   retConstant; // means: can be pre-evaluated with constant input 
 public:
   DLibFunRetNew( LibFun f, const std::string& n, const int nPar_=0, 
 		 const std::string keyNames[]=NULL,
-		 const std::string warnKeyNames[]=NULL, bool rConstant=false);
+		 const std::string warnKeyNames[]=NULL, bool rConstant=false, const int nParMin_=0);
 
 
   DLibFunRetNew( LibFun f, const std::string& n, const std::string& o, 
 		 const int nPar_=0, 
 		 const std::string keyNames[]=NULL,
-		 const std::string warnKeyNames[]=NULL);
+		 const std::string warnKeyNames[]=NULL, const int nParMin_=0);
 
   bool RetNew() { return true;}
   bool RetConstant() { return this->retConstant;}
 };
 
+// direct call functions must have:
+// ony one parameter, no keywords
+// these functions are called "direct", no environment is created
+class DLibFunDirect: public DLibFunRetNew
+{
+  LibFunDirect funDirect;
+public:
+  DLibFunDirect( LibFunDirect f, const std::string& n, bool retConstant_=true);
+
+  LibFunDirect FunDirect() { return funDirect;}
+
+//   bool RetNew() { return true;}
+  bool DirectCall() { return true;}
+};
+
 // UD pro/fun ********************************************************
 // function/procedure (differ because they are in different lists)
 // User Defined
@@ -286,6 +311,7 @@ public:
 
   void Reset();
   void DelTree();
+  void SetTree( ProgNodeP t) { tree = t;}
 
   void AddCommon(DCommonBase* c) { common.push_back(c);}
   
@@ -306,7 +332,7 @@ public:
   }
 
   // add variables
-  void     AddPar(const std::string&); // add paramter
+  DSubUD*  AddPar(const std::string&); // add paramter
   unsigned AddVar(const std::string&); // add local variable
   void     AddKey(const std::string&, const std::string&); // add keyword=value
 
