@@ -40,7 +40,18 @@ DStructBase::~DStructBase()
 // {}
 
 DStructDesc::~DStructDesc()
-{}
+{
+  assert( !isUnnamed || (operatorList == NULL));
+  if( !isUnnamed) // only named structs have members and overloaded operators 
+  // (usually they are never deleted only with .RESET_SESSION and .FULL_RESET_SESSION dot commands)
+  {
+    delete operatorList;
+    for(FunListT::iterator i = this->fun.begin(); i != this->fun.end(); ++i) 
+      { delete *i;}
+    for(ProListT::iterator i = this->pro.begin(); i != this->pro.end(); ++i) 
+      { delete *i;}
+  }
+}
 
 DStructDesc* FindInStructList(StructListT v, const string& s)
 {
@@ -53,10 +64,10 @@ DStructDesc* FindInStructList(StructListT v, const string& s)
   {
    for( SizeT t=0; t<tags.size(); ++t)
 	{
-		if( tags[t]->Type() == STRING) return true;
-		if( tags[t]->Type() == PTR) return true;
-		if( tags[t]->Type() == OBJECT) return true;
-		if( tags[t]->Type() == STRUCT)
+		if( tags[t]->Type() == GDL_STRING) return true;
+		if( tags[t]->Type() == GDL_PTR) return true;
+		if( tags[t]->Type() == GDL_OBJ) return true;
+		if( tags[t]->Type() == GDL_STRUCT)
 		{
 			if( static_cast<DStructGDL*>( tags[t])->Desc()->ContainsStringPtrObject()) return true;
 		} 
@@ -81,6 +92,30 @@ void DStructDesc::AddParent( DStructDesc* p)
   for( SizeT t=0; t < nTags; t++)
     AddTag( p->TagName(t), (*p)[t]);
   parent.push_back(p);
+  OperatorList* parentOperatorList = p->GetOperatorList(); 
+  if( parentOperatorList != NULL)
+  {
+    assert( this->operatorList == NULL); // GDL_OBJECT can only be inherited once
+    operatorList = new OperatorList(*parentOperatorList);
+  }
+}
+
+// copy appropiate member subroutines from fun and pro lists
+void DStructDesc::SetupOperators()
+{
+  assert( this->operatorList != NULL);
+  for( FunListT::iterator f = this->fun.begin(); f != this->fun.end(); ++f)
+  {
+    int ix = OverloadOperatorIndexFun( (*f)->Name());
+    if( ix != -1)
+      operatorList->SetOperator(ix,*f);      
+  }
+  for( ProListT::iterator p = this->pro.begin(); p != this->pro.end(); ++p)
+  {
+    int ix = OverloadOperatorIndexPro( (*p)->Name());
+    if( ix != -1)
+      operatorList->SetOperator(ix,*p);      
+  }
 }
 
 // more sophisticated error messages than operator==
@@ -121,7 +156,7 @@ void DStructDesc::AssureIdentical( DStructDesc* d)
 	  throw GDLException( "STRUCT: "+name+": "+TagName(i)+
 			      " tag type differs in redefinition.");
 	}
-      if( tags[i]->Type() == STRUCT)
+      if( tags[i]->Type() == GDL_STRUCT)
 	{
 	  SpDStruct* castLeft= 
 	    dynamic_cast<SpDStruct*>(tags[i]);
@@ -165,7 +200,7 @@ bool operator==(const DStructDesc& left,
       //      if( left.TagName(i) != right.TagName(i)) return false;
       if( left.tags[i]->Dim() != right.tags[i]->Dim()) return false;
       if( left.tags[i]->Type() != right.tags[i]->Type()) return false;
-      if( left.tags[i]->Type() == STRUCT)
+      if( left.tags[i]->Type() == GDL_STRUCT)
 	{
 	  SpDStruct* castLeft= 
 	    dynamic_cast<SpDStruct*>(left.tags[i]);
