@@ -26,6 +26,7 @@
 #include "basegdl.hpp"
 #include "dpro.hpp"
 #include "typedefs.hpp"
+#include "overload.hpp"
 
 class DStructBase
 {
@@ -94,6 +95,7 @@ typedef std::deque<DStructDesc*> StructListT;
 
 // descriptor of structs layout ************************************************
 // unnamed struct
+// this is never used directly, only DStructDesc (see below)
 class DUStructDesc: public DStructBase
 {
 private:
@@ -131,7 +133,16 @@ private:
 
   SizeT refCount;
 
-
+  bool isUnnamed;
+  
+  // operatorList != NULL means inherited from GDL_OBJECT
+  OperatorList* operatorList;
+  // avoid extra allocation
+//   char operatorListBuffer[ sizeof(operatorList)];
+  // ONLY FOR GDL_OBJECT
+  void InitOperatorList() { assert( operatorList == NULL); operatorList = new OperatorList();}
+  friend void InitStructs(); // restrict usage to definition of GDL_OBECT
+  
 private:
 
   std::string              name;
@@ -142,22 +153,23 @@ private:
   ProListT                 pro; // member procedures
 
   DStructDesc( const DStructDesc&) {} // disabeld
- 
 
 public:
-  DStructDesc( const std::string n): DUStructDesc(), refCount( 1)
+  DStructDesc( const std::string& n): DUStructDesc(), refCount( 1), operatorList( NULL), name(n)
   {
-    name=n;
+//     name=n;
+    // if this is to be changed, see also:
+    // DStructGDL::DStructGDL( const string& name_) // (dstructgdl.cpp)
+    isUnnamed = (name[0] == '$');
   }
 
-private:
-  // this is only used for unnamed structs -> only copy name from 'this'
-  DStructDesc( const DStructDesc* d_): 
-    DUStructDesc( d_), 
-    name( d_->name) // must be "$..."
-  {}
-
-public:
+// private:
+//   // this is only used for unnamed structs -> only copy name from 'this'
+//   DStructDesc( const DStructDesc* d_): 
+//     DUStructDesc( d_), 
+//     name( d_->name) // must be "$..."
+//   {}
+// public:
   ~DStructDesc();
 
   friend bool operator==(const DStructDesc& left, const DStructDesc& right);
@@ -165,10 +177,6 @@ public:
 
   const std::string& Name() const { return name;}
 
-  // if this is to be changed, see also:
-  // DStructGDL::DStructGDL( const string& name_) // (dstructgdl.cpp)
-  bool IsUnnamed() const { return (name[0] == '$');}
-
   FunListT& FunList()
   {
     return fun;
@@ -176,9 +184,9 @@ public:
 
   DFun* FindInFunList( const std::string& n)
   {
-  FunListT::iterator p=std::find_if(fun.begin(),fun.end(),Is_eq<DFun>(n));
-  if( p != fun.end()) return *p;
-  return NULL;
+    FunListT::iterator p=std::find_if(fun.begin(),fun.end(),Is_eq<DFun>(n));
+    if( p != fun.end()) return *p;
+    return NULL;
   }
   
   ProListT& ProList()
@@ -198,6 +206,22 @@ public:
   DFun* GetFun( const std::string& pName);
   DFun* GetFun( const std::string& pName, const std::string& parentName);
 
+  // if this is to be changed, see also:
+  // DStructGDL::DStructGDL( const string& name_) // (dstructgdl.cpp)
+  bool IsUnnamed() const { return isUnnamed;}
+
+//   bool InheritsGDL_OBJECT() const { return (operatorList != NULL);}
+  OperatorList* GetOperatorList() const { return operatorList;}
+  
+  DSubUD* GetOperator( SizeT i) const
+  { if( operatorList == NULL) return NULL; return (*operatorList)[ i];}
+  
+  void SetOperator( SizeT i, DSubUD* op)
+  { if( operatorList == NULL) return; operatorList->SetOperator( i, op);}
+  
+  // copy appropiate member subroutines from fun and pro lists
+  void SetupOperators();
+  
   bool IsParent( const std::string& p)
   {
     if( p == name) return true;
@@ -224,6 +248,7 @@ public:
   void AssureIdentical( DStructDesc* d);
   //  DStructDesc* FindEqual( const StructListT& sL);
 
+  // for unnamed structs (typetraits.cpp)
   void AddRef()
   {
     ++refCount;
