@@ -202,9 +202,10 @@ void DStructGDL::SetBufferSize( SizeT s)
 }
 
 
-DStructGDL* DStructGDL::CShift( DLong d)
+DStructGDL* DStructGDL::CShift( DLong d) const
 {
-  DStructGDL* sh = new DStructGDL( Desc(), dim, BaseGDL::NOZERO);
+  // must be nulled for correct reference counting
+  DStructGDL* sh = new DStructGDL( Desc(), dim);//, BaseGDL::NOZERO);
 
   SizeT nEl = N_Elements();
   SizeT nTag = NTags();
@@ -221,9 +222,10 @@ DStructGDL* DStructGDL::CShift( DLong d)
 }
 
 // TODO: Implement fast 2D version (see datatypes.cpp)
-DStructGDL* DStructGDL::CShift( DLong s[MAXRANK])
+DStructGDL* DStructGDL::CShift( DLong s[MAXRANK]) const
 {
-  DStructGDL* sh = new DStructGDL( Desc(), dim, BaseGDL::NOZERO);
+  // must be nulled for correct reference counting
+  DStructGDL* sh = new DStructGDL( Desc(), dim);//, BaseGDL::NOZERO);
 
   SizeT nDim = Rank();
   SizeT nEl = N_Elements();
@@ -672,7 +674,7 @@ DStructGDL* DStructGDL::CatArray( ExprListT& exprList,
   SizeT at=0;
   for( i=exprList.begin(); i != exprList.end(); i++)
     {
-      // for STRUCT all types are compatible structs already (array_def in gdlc.i.g)
+      // for GDL_STRUCT all types are compatible structs already (array_def in gdlc.i.g)
       //      if( (*i)->Type() != t) (*i)=(*i)->Convert2( t);
       catArr->CatInsert(static_cast<DStructGDL*>( (*i)),
 			catRankIx,at); // advances 'at'
@@ -914,10 +916,10 @@ void DStructGDL::CatInsert( const DStructGDL* srcArr, const SizeT atDim, SizeT&
 BaseGDL* DStructGDL::Get( SizeT tag)
 {
   DotAccessDescT aD( 2); // like a.b (=2 levels)
-  aD.Root( this);  // set root, implicit no index
-  aD.Add( tag);    // tag to extract
-  aD.AddIx( NULL); // no index -> ALL
+  aD.ADRoot( this);  // set root, implicit no index
+  aD.ADAdd( tag);    // tag to extract
+  aD.ADAddIx( NULL); // no index -> ALL
   
-  return aD.Resolve();
+  return aD.ADResolve();
 }
 
