@@ -145,7 +145,7 @@ public:
   // assignment. 
   DStructGDL& operator=(const BaseGDL& r)
   {
-    assert( r.Type() == STRUCT);
+    assert( r.Type() == GDL_STRUCT);
     const DStructGDL& right = static_cast<const DStructGDL&>( r);
 
     assert( *Desc() == *right.Desc());
@@ -168,7 +168,7 @@ public:
   
   void InitFrom(const BaseGDL& r)
   {
-    assert( r.Type() == STRUCT);
+    assert( r.Type() == GDL_STRUCT);
     const DStructGDL& right = static_cast<const DStructGDL&>( r);
 
     assert( *Desc() == *right.Desc());
@@ -217,8 +217,8 @@ return Buf();
    DStructGDL* SetBuffer( const void* b);
   void SetBufferSize( SizeT s);
 
-  DStructGDL* CShift( DLong d);
-  DStructGDL* CShift( DLong d[MAXRANK]);
+  DStructGDL* CShift( DLong d) const;
+  DStructGDL* CShift( DLong d[MAXRANK]) const;
 
   // for use by MIN and MAX functions
   void MinMax( DLong* minE, DLong* maxE, 
@@ -487,6 +487,8 @@ DStructGDL* NewResult() const
 			const SizeT catRank, 
 			const SizeT rank);
 
+  template< typename To> typename Data_<To>::Ty GetAs( SizeT i);
+
   BaseGDL* Convert2( DType destTy, 
 		     BaseGDL::Convert2Mode mode = BaseGDL::CONVERT);
   
@@ -635,12 +637,12 @@ DStructGDL* NewResult() const
 
 
   
-  Data_<SpDByte>* EqOp( BaseGDL* r);
-  Data_<SpDByte>* NeOp( BaseGDL* r);
-  Data_<SpDByte>* GtOp( BaseGDL* r);
-  Data_<SpDByte>* GeOp( BaseGDL* r);
-  Data_<SpDByte>* LtOp( BaseGDL* r);
-  Data_<SpDByte>* LeOp( BaseGDL* r);
+  BaseGDL* EqOp( BaseGDL* r);
+  BaseGDL* NeOp( BaseGDL* r);
+  BaseGDL* GtOp( BaseGDL* r);
+  BaseGDL* GeOp( BaseGDL* r);
+  BaseGDL* LtOp( BaseGDL* r);
+  BaseGDL* LeOp( BaseGDL* r);
 
   // assigns srcIn to this at ixList, if ixList is NULL does linear copy
   // assumes: ixList has this already set as variable
