@@ -282,8 +282,10 @@ EnvT::EnvT( EnvT* pEnv, DSub* newPro, BaseGDL** self):
 
 
 
-EnvUDT::EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self):
-  EnvBaseT( pEnv->CallingNode(), newPro),
+//EnvUDT::EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self):
+EnvUDT::EnvUDT( ProgNodeP callingNode_, DSub* newPro, BaseGDL** self):
+//   EnvBaseT( pEnv->CallingNode(), newPro),
+  EnvBaseT( callingNode_, newPro),
   ioError(NULL), 
   onError( -1), 
   catchVar(NULL), 
@@ -322,7 +324,7 @@ void EnvBaseT::AddStruct( DPtrListT& ptrAccessible,
   SizeT nTags = desc->NTags();
   for( SizeT t=0; t<nTags; ++t)
     {
-      if( (*desc)[ t]->Type() == PTR)
+      if( (*desc)[ t]->Type() == GDL_PTR)
 	{
 	  for( SizeT e = 0; e<nEl; ++e)
 	    {
@@ -330,7 +332,7 @@ void EnvBaseT::AddStruct( DPtrListT& ptrAccessible,
 	      AddPtr( ptrAccessible, objAccessible, ptr);
 	    }
 	}
-      else if( (*desc)[ t]->Type() == STRUCT)
+      else if( (*desc)[ t]->Type() == GDL_STRUCT)
 	{
 	  for( SizeT e = 0; e<nEl; ++e)
 	    {
@@ -338,7 +340,7 @@ void EnvBaseT::AddStruct( DPtrListT& ptrAccessible,
 	      AddStruct( ptrAccessible, objAccessible, ptr);
 	    }
 	}
-     else if( (*desc)[ t]->Type() == OBJECT)
+     else if( (*desc)[ t]->Type() == GDL_OBJ)
 	{
 	  for( SizeT e = 0; e<nEl; ++e)
 	    {
@@ -468,7 +470,7 @@ void EnvT::HeapGC( bool doPtr, bool doObj, bool verbose)
 
 	AddToDestroy( ptrAccessible, objAccessible);  
 
-    // do OBJ first as the cleanup might need the PTR be valid
+    // do OBJ first as the cleanup might need the GDL_PTR be valid
     if( doObj)
       {
 	std::vector<DObj>* heap = interpreter->GetAllObjHeapSTL();
@@ -552,50 +554,50 @@ void EnvBaseT::ObjCleanup( DObj actID)
     {
       DStructGDL* actObj;
       try{
-// 		actObj=GetObjHeap( actID);
- 		GDLInterpreter::ObjHeapT::iterator it;
-		actObj=GDLInterpreter::GetObjHeap( actID, it);
-      }
+	actObj=GetObjHeap( actID);
+// 	GDLInterpreter::ObjHeapT::iterator it;
+// 	actObj=GDLInterpreter::GetObjHeap( actID, it);
+    }
       catch( GDLInterpreter::HeapException){
 		actObj=NULL;
       }
 	    
     if( actObj != NULL)
-		{
-		try{
-				// call CLEANUP function
-				DPro* objCLEANUP= actObj->Desc()->GetPro( "CLEANUP");
-			
-				if( objCLEANUP != NULL)
-				{
-					BaseGDL* actObjGDL = new DObjGDL( actID);
-					auto_ptr<BaseGDL> actObjGDL_guard( actObjGDL);
-					GDLInterpreter::IncRefObj( actID); // set refcount to 1
-				
-					PushNewEmptyEnvUD( objCLEANUP, &actObjGDL);
-				
-					inProgress.insert( actID);
-				
-					interpreter->call_pro( objCLEANUP->GetTree());
-				
-					inProgress.erase( actID);
+	    {
+	      try{
+		  // call CLEANUP function
+		  DPro* objCLEANUP= actObj->Desc()->GetPro( "CLEANUP");
+	  
+		  if( objCLEANUP != NULL)
+		  {
+		    BaseGDL* actObjGDL = new DObjGDL( actID);
+		    auto_ptr<BaseGDL> actObjGDL_guard( actObjGDL);
+		    GDLInterpreter::IncRefObj( actID); // set refcount to 1
+	    
+		    PushNewEmptyEnvUD( objCLEANUP, &actObjGDL);
+	    
+		    inProgress.insert( actID);
+	    
+		    interpreter->call_pro( objCLEANUP->GetTree());
+	    
+		    inProgress.erase( actID);
 
-					EnvBaseT* callStackBack =  interpreter->CallStack().back();
-					interpreter->CallStack().pop_back();
-					delete callStackBack;
+		    EnvBaseT* callStackBack =  interpreter->CallStack().back();
+		    interpreter->CallStack().pop_back();
+		    delete callStackBack;
 
-					FreeObjHeap( actID); // make sure actObj is freed
-					// actObjGDL goes out of scope -> refcount is (would be) decreased
-				}
-			}
-		catch( ...)
-			{
-				FreeObjHeap( actID); // make sure actObj is freed
-				throw; // rethrow
-			}		
-		}
-	else		
-		FreeObjHeap( actID); // the actual freeing
+		    FreeObjHeap( actID); // make sure actObj is freed
+		    // actObjGDL goes out of scope -> refcount is (would be) decreased
+		  }
+	      }
+	    catch( ...)
+	      {
+		FreeObjHeap( actID); // make sure actObj is freed
+		throw; // rethrow
+	      }		
+	    }
+    else		
+	FreeObjHeap( actID); // the actual freeing
     }
 }
 
@@ -607,9 +609,9 @@ void EnvT::ObjCleanup( DObj actID)
     {
       DStructGDL* actObj;
       try{
-// 		actObj=GetObjHeap( actID);
- 		GDLInterpreter::ObjHeapT::iterator it;
-		actObj=GDLInterpreter::GetObjHeap( actID, it);
+ 		actObj=GetObjHeap( actID);
+//  		GDLInterpreter::ObjHeapT::iterator it;
+// 		actObj=GDLInterpreter::GetObjHeap( actID, it);
       }
       catch( GDLInterpreter::HeapException){
 		actObj=NULL;
@@ -733,7 +735,7 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
 	os << p->TypeStr() << right;
 	
 	// Data display
-	if( p->Type() == STRUCT)
+	if( p->Type() == GDL_STRUCT)
 	{
 /*		DStructGDL* s = static_cast<DStructGDL*>( p);
 		os << "-> ";
@@ -743,7 +745,7 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
 	else if( p->Dim( 0) == 0)
 	{
 		os << "(";
-		if (p->Type() == STRING)
+		if (p->Type() == GDL_STRING)
 		{
 		// trim string larger than 45 characters
 		DString dataString = (*static_cast<DStringGDL*>(p))[0];
@@ -815,7 +817,7 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
 // 	os << p->TypeStr() << right;
 // 	
 // 	// Data display
-// 	if( p->Type() == STRUCT)
+// 	if( p->Type() == GDL_STRUCT)
 // 	{
 // 		DStructGDL* s = static_cast<DStructGDL*>( p);
 // 		os << "-> ";
@@ -825,7 +827,7 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
 // 	else if( p->Dim( 0) == 0)
 // 	{
 // 		os << "(";
-// 		if (p->Type() == STRING)
+// 		if (p->Type() == GDL_STRING)
 // 		{
 // 		// trim string larger than 45 characters
 // 		DString dataString = (*static_cast<DStringGDL*>(p))[0];
@@ -907,30 +909,43 @@ void EnvBaseT::SetKeyword( const string& k, BaseGDL** const val) // reference
 }
 
 // called after parameter definition
-void EnvBaseT::Extra()
+void EnvBaseT::ResolveExtra()
 {
-  if( extra != NULL) extra->Resolve();
+  if( extra != NULL) extra->ResolveExtra( NULL);
 }
 
+// // for internal non-library routines (e.g. operator overloads) ('this' is on the stack)
+// EnvUDT* EnvUDT::CallingEnv()
+// {
+//   EnvStackT& callStack=interpreter->CallStack();
+//   assert( callStack.size() >= 2); // must be: "$MAIN$" and the EnvUDT of the internal routine
+//   return callStack[callStack.size()-2];
+// }
+
+// for library subroutines, get the EnvUDT from which they are called
 EnvBaseT* EnvBaseT::Caller()
 {
   EnvStackT& callStack=interpreter->CallStack();
 
-  if( callStack.size() <= 1) return NULL;
+  //if( callStack.size() <= 1) return NULL;
+  // library environments are no longer on the call stack
+  assert( callStack.back() != this);
 
-  if( callStack.back() != this) 
-    return callStack.back();
-//     return static_cast< EnvUDT*>( callStack.back());
-
-  return callStack[ callStack.size()-2];
-//   return static_cast< EnvUDT*>( callStack[ callStack.size()-2]);
+  return callStack.back();
+  
+//   if( callStack.back() != this) 
+//     return callStack.back();
+// //     return static_cast< EnvUDT*>( callStack.back());
+// 
+//   return callStack[ callStack.size()-2];
+// //   return static_cast< EnvUDT*>( callStack[ callStack.size()-2]);
 }
 
 // used by obj_new (basic_fun.cpp)
 // and obj_destroy (basic_pro.cpp)
 void EnvBaseT::PushNewEmptyEnvUD(  DSub* newPro, BaseGDL** newObj)
 {
-  EnvUDT* newEnv= new EnvUDT( this, newPro, newObj);
+  EnvUDT* newEnv= new EnvUDT( this->CallingNode(), newPro, newObj);
 
   // pass the parameters, skip the first 'skipP'
 //   SizeT nParam = NParam();
@@ -949,9 +964,11 @@ void EnvBaseT::PushNewEmptyEnvUD(  DSub* newPro, BaseGDL** newObj)
 
 // used by obj_new (basic_fun.cpp)
 // and obj_destroy (basic_pro.cpp)
+// and call_function (basic_fun.cpp)
+// and call_procedure (basic_pro.cpp)
 void EnvT::PushNewEnvUD(  DSub* newPro, SizeT skipP, BaseGDL** newObj)
 {
-  EnvUDT* newEnv= new EnvUDT( this, newPro, newObj);
+  EnvUDT* newEnv= new EnvUDT( this->CallingNode(), newPro, newObj);
 
   // pass the parameters, skip the first 'skipP'
   SizeT nParam = NParam();
@@ -960,18 +977,20 @@ void EnvT::PushNewEnvUD(  DSub* newPro, SizeT skipP, BaseGDL** newObj)
       newEnv->SetNextPar( &GetPar( p)); // pass as global
     }
 
-  interpreter->CallStack().push_back( newEnv); 
+//   interpreter->CallStack().push_back( newEnv); // problem with call_function if done here s. b.
 
   // _REF_EXTRA is set to the keyword string array
   newEnv->extra = new ExtraT( newEnv);
   newEnv->extra->Set( &env[0]);
-  newEnv->extra->Resolve();
+  newEnv->extra->ResolveExtra( this); // s. a. problem caused here due to a call to EnvBaseT::Caller() in Resolve()
+
+  interpreter->CallStack().push_back( newEnv); 
 }
 // used by obj_new (basic_fun.cpp)
 // and obj_destroy (basic_pro.cpp)
 // and call_function (basic_fun.cpp)
 // and call_procedure (basic_pro.cpp)
-void EnvT::PushNewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj)
+EnvT* EnvT::NewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj)
 {
   EnvT* newEnv= new EnvT( this, newPro, newObj);
 
@@ -982,12 +1001,14 @@ void EnvT::PushNewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj)
       newEnv->SetNextPar( &GetPar( p)); // pass as global
     }
 
-  interpreter->CallStack().push_back( newEnv); 
+//   interpreter->CallStack().push_back( newEnv); 
 
   // _REF_EXTRA is set to the keyword string array
   newEnv->extra = new ExtraT( newEnv);
   newEnv->extra->Set( &env[0]);
-  newEnv->extra->Resolve();
+  newEnv->extra->ResolveExtra( this);
+
+  return newEnv;
 }
 
 void EnvT::AssureGlobalPar( SizeT pIx)
@@ -1056,7 +1077,7 @@ void EnvT::Catch()
 			  "in this context: " + GetParString(0));
     caller->catchNode = callingNode->getNextSibling();
     caller->catchVar = &GetPar( 0);
-    delete *caller->catchVar;
+    GDLDelete(*caller->catchVar);
     *caller->catchVar = new DLongGDL( 0);
   }
 
@@ -1100,7 +1121,7 @@ const string EnvBaseT::GetString( SizeT ix)
     return subUD->GetVarName( ix);
   }
 
-// SA: used by STRING() for VMS-compat hack
+// SA: used by GDL_STRING() for VMS-compat hack
 void EnvT::ShiftParNumbering(int n)
 {
   assert(abs(n) == 1); // currently the code below works for n = +/- 1 only
@@ -1183,7 +1204,7 @@ SizeT EnvT::NParam( SizeT minPar)
 
 void EnvBaseT::SetNextParUnchecked( BaseGDL* const nextP) // by value (reset loc)
 {
-  if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar))
+//   if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar))
   assert( static_cast<int>(parIx - pro->key.size()) < pro->nPar);
   env.Set(parIx++,nextP); // check done in parameter_def
 }
@@ -1337,11 +1358,12 @@ bool EnvT::KeywordSet( SizeT ix)
 {
   BaseGDL* keyword=env[ix];
   if( keyword == NULL) return false;
-  if( !keyword->Scalar()) return false;
+  if( !keyword->Scalar()) return true;
+  if( keyword->Type() == GDL_STRUCT) return true;
   return keyword->LogTrue();
 }
 
-// returns the ix'th parameter
+// returns the ix'th parameter (for library function API only)
 BaseGDL*& EnvT::GetPar(SizeT i)
 {
   static BaseGDL* null=NULL;
@@ -1354,11 +1376,16 @@ BaseGDL*& EnvT::GetPar(SizeT i)
     }
   return env[ ix];
 }
+// BaseGDL*& EnvT::GetParUnchecked(SizeT i)
+// {
+//   SizeT ix= i + pro->key.size();
+//   return env[ ix];
+// }
 
 void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong64& scalar)
 {
   BaseGDL* p = GetParDefined( pIx);
-  DLong64GDL* lp = static_cast<DLong64GDL*>(p->Convert2( LONG64, BaseGDL::COPY));
+  DLong64GDL* lp = static_cast<DLong64GDL*>(p->Convert2( GDL_LONG64, BaseGDL::COPY));
   auto_ptr<DLong64GDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
     Throw("Parameter must be a scalar in this context: "+
@@ -1367,7 +1394,7 @@ void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong64& scalar)
 void EnvBaseT::AssureLongScalarPar( SizeT pIx, DLong& scalar)
 {
   BaseGDL* p = GetParDefined( pIx);
-  DLongGDL* lp = static_cast<DLongGDL*>(p->Convert2( LONG, BaseGDL::COPY));
+  DLongGDL* lp = static_cast<DLongGDL*>(p->Convert2( GDL_LONG, BaseGDL::COPY));
   auto_ptr<DLongGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
     Throw("Parameter must be a scalar in this context: "+
@@ -1407,7 +1434,7 @@ void EnvT::AssureLongScalarKW( SizeT eIx, DLong& scalar)
   if( p == NULL)
     Throw("Expression undefined: "+GetString(eIx));
   
-  DLongGDL* lp= static_cast<DLongGDL*>(p->Convert2( LONG, BaseGDL::COPY));
+  DLongGDL* lp= static_cast<DLongGDL*>(p->Convert2( GDL_LONG, BaseGDL::COPY));
   
   auto_ptr<DLongGDL> guard_lp( lp);
 
@@ -1419,7 +1446,7 @@ void EnvT::AssureLongScalarKW( SizeT eIx, DLong& scalar)
 void EnvT::AssureDoubleScalarPar( SizeT pIx, DDouble& scalar)
 {
   BaseGDL* p = GetParDefined( pIx);
-  DDoubleGDL* lp = static_cast<DDoubleGDL*>(p->Convert2( DOUBLE, BaseGDL::COPY));
+  DDoubleGDL* lp = static_cast<DDoubleGDL*>(p->Convert2( GDL_DOUBLE, BaseGDL::COPY));
   auto_ptr<DDoubleGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
     Throw("Parameter must be a scalar in this context: "+
@@ -1449,7 +1476,7 @@ void EnvT::AssureDoubleScalarKW( SizeT eIx, DDouble& scalar)
   if( p == NULL)
     Throw("Expression undefined: "+GetString(eIx));
   
-  DDoubleGDL* lp= static_cast<DDoubleGDL*>(p->Convert2( DOUBLE, BaseGDL::COPY));
+  DDoubleGDL* lp= static_cast<DDoubleGDL*>(p->Convert2( GDL_DOUBLE, BaseGDL::COPY));
   
   auto_ptr<DDoubleGDL> guard_lp( lp);
 
@@ -1462,7 +1489,7 @@ void EnvT::AssureDoubleScalarKW( SizeT eIx, DDouble& scalar)
 void EnvT::AssureFloatScalarPar( SizeT pIx, DFloat& scalar)
 {
   BaseGDL* p = GetParDefined( pIx);
-  DFloatGDL* lp = static_cast<DFloatGDL*>(p->Convert2( FLOAT, BaseGDL::COPY));
+  DFloatGDL* lp = static_cast<DFloatGDL*>(p->Convert2( GDL_FLOAT, BaseGDL::COPY));
   auto_ptr<DFloatGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
     Throw("Parameter must be a scalar in this context: "+
@@ -1492,7 +1519,7 @@ void EnvT::AssureFloatScalarKW( SizeT eIx, DFloat& scalar)
   if( p == NULL)
     Throw("Expression undefined: "+GetString(eIx));
   
-  DFloatGDL* lp= static_cast<DFloatGDL*>(p->Convert2( FLOAT, BaseGDL::COPY));
+  DFloatGDL* lp= static_cast<DFloatGDL*>(p->Convert2( GDL_FLOAT, BaseGDL::COPY));
   
   auto_ptr<DFloatGDL> guard_lp( lp);
 
@@ -1505,7 +1532,7 @@ void EnvT::AssureFloatScalarKW( SizeT eIx, DFloat& scalar)
 void EnvT::AssureStringScalarPar( SizeT pIx, DString& scalar)
 {
   BaseGDL* p = GetParDefined( pIx);
-  DStringGDL* lp = static_cast<DStringGDL*>(p->Convert2( STRING, BaseGDL::COPY));
+  DStringGDL* lp = static_cast<DStringGDL*>(p->Convert2( GDL_STRING, BaseGDL::COPY));
   auto_ptr<DStringGDL> guard_lp( lp);
   if( !lp->Scalar( scalar))
     Throw("Parameter must be a scalar in this context: "+
@@ -1534,7 +1561,7 @@ void EnvT::AssureStringScalarKW( SizeT eIx, DString& scalar)
   if( p == NULL)
     Throw("Expression undefined: "+GetString(eIx));
   
-  DStringGDL* lp= static_cast<DStringGDL*>(p->Convert2( STRING, BaseGDL::COPY));
+  DStringGDL* lp= static_cast<DStringGDL*>(p->Convert2( GDL_STRING, BaseGDL::COPY));
   auto_ptr<DStringGDL> guard_lp( lp);
 
   if( !lp->Scalar( scalar))
@@ -1547,7 +1574,7 @@ void EnvT::SetKW( SizeT ix, BaseGDL* newVal)
   // can't use Guard here as data has to be released
   auto_ptr<BaseGDL> guard( newVal);
   AssureGlobalKW( ix);
-  delete GetKW( ix);
+  GDLDelete(GetKW( ix));
   GetKW( ix) = guard.release();
 }
 void EnvT::SetPar( SizeT ix, BaseGDL* newVal)
@@ -1555,7 +1582,7 @@ void EnvT::SetPar( SizeT ix, BaseGDL* newVal)
   // can't use Guard here as data has to be released
   auto_ptr<BaseGDL> guard( newVal);
   AssureGlobalPar( ix);
-  delete GetPar( ix);
+  GDLDelete(GetPar( ix));
   GetPar( ix) = guard.release();
 }
 
