@@ -112,6 +112,7 @@ public:
 
   // for CLEANUP calls due to reference counting
   void PushNewEmptyEnvUD(  DSub* newPro, BaseGDL** newObj = NULL);
+  void PushNewEmptyEnvUDWithExtra(  DSub* newPro, BaseGDL** newObj = NULL);
   
   void AddEnv( DPtrListT& ptrAccessible, DPtrListT& objAccessible);
   void AddToDestroy( DPtrListT& ptrAccessible, DPtrListT& objAccessible);
@@ -140,28 +141,28 @@ public:
   static DStructGDL* GetObjHeap( DObj ID);
   static BaseGDL* GetHeap( DPtr ID);
 
-int GetLineNumber()
-{
-return lineNumber;
-}
-void SetLineNumber(int l)
-{
-lineNumber = l;
-}
-unsigned int CompileOpt()
-{
-DSubUD* proUD = dynamic_cast<DSubUD*>(pro);
-if( proUD == NULL)
-	throw GDLException("Intenal error: CompileOpt called non DSub object.");
-return proUD->GetCompileOpt();
-}
-void SetCompileOpt( unsigned int cOpt)
-{
-DSubUD* proUD = dynamic_cast<DSubUD*>(pro);
-if( proUD == NULL)
-	throw GDLException("Intenal error: CompileOpt called non DSub object.");
-proUD->SetCompileOpt( cOpt);
-}
+  int GetLineNumber()
+  {
+  return lineNumber;
+  }
+  void SetLineNumber(int l)
+  {
+  lineNumber = l;
+  }
+  unsigned int CompileOpt()
+  {
+    DSubUD* proUD = dynamic_cast<DSubUD*>(pro);
+    if( proUD == NULL)
+	    throw GDLException("Intenal error: CompileOpt called non DSub object.");
+    return proUD->GetCompileOpt();
+  }
+  void SetCompileOpt( unsigned int cOpt)
+  {
+    DSubUD* proUD = dynamic_cast<DSubUD*>(pro);
+    if( proUD == NULL)
+	    throw GDLException("Intenal error: CompileOpt called non DSub object.");
+    proUD->SetCompileOpt( cOpt);
+  }
 
   // raise an exception from within a library function
   // automatically cares for adding line/column info and the
@@ -179,7 +180,7 @@ proUD->SetCompileOpt( cOpt);
   void RemoveLoc( BaseGDL* p) { env.RemoveLoc( p);}
 
   // called after parameter definition
-  void Extra();
+  void ResolveExtra();
   friend class ExtraT;
 
   // used by compiler and from EnvT (for variable number of paramters)
@@ -199,12 +200,13 @@ proUD->SetCompileOpt( cOpt);
   // a EnvT must have always a /*EnvUDT*/ caller
   // i.e. library functions never call each other
   // with a new EnvT environment
+  // for library subroutines, get the EnvUDT from which they are called
   EnvBaseT* Caller();
 
   // returns environment data, by value (but that by C++ reference)
   BaseGDL*& GetKW(SizeT ix) { return env[ix];}
 
-  // used by HELP and SetNextPar(...)
+  // used by HELP, SetNextPar(...)
   SizeT EnvSize() const { return env.size();}
 
   // next four are used by Parameter...(...) functions
@@ -416,7 +418,8 @@ public:
 
 
   // for obj_new and obj_destroy
-  EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self); 
+  //EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self); 
+  EnvUDT( ProgNodeP callindNode_, DSub* newPro, BaseGDL** self); 
 
   DLong GetOnError() const { return onError;}
 
@@ -446,6 +449,9 @@ public:
     return subUD->GetFilename();
   }
 
+//   // for internal non-library routines (e.g. operator overloads) ('this' is on the stack)
+//   EnvUDT* CallingEnv();
+
   friend class DInterpreter;
   friend class EnvT;
 };
@@ -478,7 +484,7 @@ public:
   void ObjCleanup( DObj actID);
 
   // used by obj_new (basic_fun.cpp)
-  void PushNewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj=NULL);
+  EnvT* NewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj=NULL);
   void PushNewEnvUD(  DSub* newPro, SizeT skipP, BaseGDL** newObj=NULL);
   // for exclusive use by lib::on_error
   void OnError();
@@ -519,29 +525,34 @@ public:
 
 
   // returns environment data, by value (but that by C++ reference)
-  BaseGDL*& GetKW(SizeT ix) { return env[ix];}
+  // in EnvBaseT
+//   BaseGDL*& GetKW(SizeT ix) { return env[ix];}
+
+  // it is now possible to define a niminum number of parameters for library subroutines
+  // if this is done the next function can be used
+//   BaseGDL*& GetParUnchecked(SizeT i);
 
   // returns the ix'th parameter (NULL if not defined)
-  BaseGDL*& GetPar(SizeT i);
+  BaseGDL*& GetPar( SizeT i);  
 
   // get i'th parameter
   // throws if not defined (ie. never returns NULL)
   BaseGDL*& GetParDefined(SizeT i); //, const std::string& subName = "");
 
-  // throw for STRING, STRUCT, PTR and OBJECT
+  // throw for GDL_STRING, GDL_STRUCT, GDL_PTR and GDL_OBJ
   BaseGDL*& GetNumericParDefined( SizeT ix)
   {
     BaseGDL*& p0 = GetParDefined( ix);
     if( NumericType( p0->Type()))
 	return p0;
 
-    if( p0->Type() == STRING)
+    if( p0->Type() == GDL_STRING)
       Throw( "String expression not allowed in this context: "+GetParString(ix));
-    if( p0->Type() == STRUCT)
+    if( p0->Type() == GDL_STRUCT)
       Throw( "Struct expression not allowed in this context: "+GetParString(ix));
-    if( p0->Type() == PTR)
+    if( p0->Type() == GDL_PTR)
       Throw( "Pointer expression not allowed in this context: "+GetParString(ix));
-    if( p0->Type() == OBJECT)
+    if( p0->Type() == GDL_OBJ)
       Throw( "Object reference not allowed in this context: "+GetParString(ix));
 
     assert( false);
@@ -572,9 +583,11 @@ public:
   T* GetParAs( SizeT pIx)
   {
     BaseGDL* p = GetParDefined( pIx);
-    T* res = dynamic_cast<T*>( p);
-    if( res != NULL) return res;
-    res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
+	if( p->Type() == T::t)
+		return static_cast<T*>( p);
+//     T* res = dynamic_cast<T*>( p);
+//     if( res != NULL) return res;
+    T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
     Guard( res);
     return res;
   }
@@ -585,9 +598,11 @@ public:
     BaseGDL* p = GetKW( ix);
     if( p == NULL)
       Throw( "Keyword is undefined: "+GetString( ix));
-    T* res = dynamic_cast<T*>( p);
-    if( res != NULL) return res;
-    res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
+	if( p->Type() == T::t)
+		return static_cast<T*>( p);
+//     T* res = dynamic_cast<T*>( p);
+//     if( res != NULL) return res;
+    T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
     Guard( res);
     return res;
   }
@@ -598,9 +613,11 @@ public:
   {
     BaseGDL* p = GetPar( pIx);
     if( p == NULL) return NULL;
-    T* res = dynamic_cast<T*>( p);
-    if( res != NULL) return res;
-    res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
+	if( p->Type() == T::t)
+		return static_cast<T*>( p);
+//     T* res = dynamic_cast<T*>( p);
+//     if( res != NULL) return res;
+    T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
     Guard( res);
     return res;
   }
@@ -610,9 +627,11 @@ public:
   {
     BaseGDL* p = GetKW( ix);
     if( p == NULL) return NULL;
-    T* res = dynamic_cast<T*>( p);
-    if( res != NULL) return res;
-    res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
+	if( p->Type() == T::t)
+		return static_cast<T*>( p);
+//     T* res = dynamic_cast<T*>( p);
+//     if( res != NULL) return res;
+    T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
     Guard( res);
     return res;
   }
@@ -766,12 +785,13 @@ public:
 
   // to be extended on demand for other data types  
 
-  // SA: used by STRING() for VMS-compat hack
+  // SA: used by GDL_STRING() for VMS-compat hack
   void ShiftParNumbering(int n);
 };
 
 
-typedef std::deque<EnvBaseT*> EnvStackT;
+// typedef std::deque<EnvBaseT*> EnvStackT;
+typedef std::deque<EnvUDT*> EnvStackT;
 
 #endif
 
