@@ -26,7 +26,9 @@
 #include "file.hpp"
 #include "objects.hpp"
 
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 #include <sys/stat.h>
 #include <climits> // PATH_MAX
 
@@ -34,9 +36,28 @@
 // #include <wx/file.h>
 // #include <wx/dir.h>
 
+#ifndef _MSC_VER
 #include <glob.h>
-#include <dirent.h>
 #include <fnmatch.h>
+#include <dirent.h>
+#else
+#include <io.h>
+#define access _access
+#define R_OK    4       /* Test for read permission.  */
+#define W_OK    2       /* Test for write permission.  */
+//#define   X_OK    1       /* execute permission - unsupported in windows*/
+#define F_OK    0       /* Test for existence.  */
+#define PATH_MAX 255
+
+#include <direct.h>
+
+#if !defined(S_ISDIR)
+#define __S_ISTYPE(mode, mask)	(((mode) & S_IFMT) == (mask))
+#define S_ISDIR(mode)	 __S_ISTYPE((mode), S_IFDIR)
+#define S_ISREG(mode)    __S_ISTYPE((mode), S_IFREG)
+#endif
+#include <shlwapi.h>
+#endif
 
 // workaround for HP-UX. A better solution is needed i think
 //#if defined(__hpux__) || defined(__sun__)
@@ -54,6 +75,162 @@
 #  define GLOB_PERIOD 0
 #endif
 
+#ifdef _MSC_VER
+
+/*
+
+    Implementation of POSIX directory browsing functions and types for Win32.
+
+    Author:  Kevlin Henney (kevlin@acm.org, kevlin@curbralan.com)
+    History: Created March 1997. Updated June 2003 and July 2012.
+    Rights:  See end of file.
+
+*/
+
+#include <errno.h>
+#include <io.h> /* _findfirst and _findnext set errno iff they return -1 */
+#include <stdlib.h>
+#include <string.h>
+
+struct dirent
+{
+    char *d_name;
+};
+
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+typedef ptrdiff_t handle_type; /* C99's intptr_t not sufficiently portable */
+
+struct DIR
+{
+    handle_type         handle; /* -1 for failed rewind */
+    struct _finddata_t  info;
+    struct dirent       result; /* d_name null iff first time */
+    char                *name;  /* null-terminated char string */
+};
+
+DIR *opendir(const char *name)
+{
+    DIR *dir = 0;
+
+    if(name && name[0])
+    {
+        size_t base_length = strlen(name);
+        const char *all = /* search pattern must end with suitable wildcard */
+            strchr("/\\", name[base_length - 1]) ? "*" : "/*";
+
+        if((dir = (DIR *) malloc(sizeof *dir)) != 0 &&
+           (dir->name = (char *) malloc(base_length + strlen(all) + 1)) != 0)
+        {
+            strcat(strcpy(dir->name, name), all);
+
+            if((dir->handle =
+                (handle_type) _findfirst(dir->name, &dir->info)) != -1)
+            {
+                dir->result.d_name = 0;
+            }
+            else /* rollback */
+            {
+                free(dir->name);
+                free(dir);
+                dir = 0;
+            }
+        }
+        else /* rollback */
+        {
+            free(dir);
+            dir   = 0;
+            errno = ENOMEM;
+        }
+    }
+    else
+    {
+        errno = EINVAL;
+    }
+
+    return dir;
+}
+
+int closedir(DIR *dir)
+{
+    int result = -1;
+
+    if(dir)
+    {
+        if(dir->handle != -1)
+        {
+            result = _findclose(dir->handle);
+        }
+
+        free(dir->name);
+        free(dir);
+    }
+
+    if(result == -1) /* map all errors to EBADF */
+    {
+        errno = EBADF;
+    }
+
+    return result;
+}
+
+struct dirent *readdir(DIR *dir)
+{
+    struct dirent *result = 0;
+
+    if(dir && dir->handle != -1)
+    {
+        if(!dir->result.d_name || _findnext(dir->handle, &dir->info) != -1)
+        {
+            result         = &dir->result;
+            result->d_name = dir->info.name;
+        }
+    }
+    else
+    {
+        errno = EBADF;
+    }
+
+    return result;
+}
+
+void rewinddir(DIR *dir)
+{
+    if(dir && dir->handle != -1)
+    {
+        _findclose(dir->handle);
+        dir->handle = (handle_type) _findfirst(dir->name, &dir->info);
+        dir->result.d_name = 0;
+    }
+    else
+    {
+        errno = EBADF;
+    }
+}
+
+#ifdef __cplusplus
+}
+#endif
+
+/*
+
+    Copyright Kevlin Henney, 1997, 2003, 2012. All rights reserved.
+
+    Permission to use, copy, modify, and distribute this software and its
+    documentation for any purpose is hereby granted without fee, provided
+    that this copyright and permissions notice appear in all copies and
+    derivatives.
+    
+    This software is supplied "as is" without express or implied warranty.
+
+    But that said, if there are any problems please get in touch.
+
+*/
+
+#endif
 
 namespace lib {
 
@@ -151,7 +328,11 @@ namespace lib {
 	
 
 	struct stat statStruct;
+#ifdef _MSC_VER
+	int actStat = stat( actFile, &statStruct);
+#else
 	int actStat = lstat( actFile, &statStruct);
+#endif
 	if( actStat != 0) 
 	  continue;
 
@@ -169,20 +350,16 @@ namespace lib {
 	  continue;
 	if( write && access( actFile, W_OK) != 0)
 	  continue;
-	if( executable && access( actFile, X_OK) != 0)
-	  continue;
 
 	if( zero_length && statStruct.st_size != 0) 
 	  continue;
+#ifndef _MSC_VER
+	if( executable && access( actFile, X_OK) != 0)
+	  continue;
 
 	if( get_mode)
 	  (*getMode)[ f] = statStruct.st_mode & 
 	    (S_IRWXU | S_IRWXG | S_IRWXO);
-	    
-	if( directory && S_ISDIR(statStruct.st_mode) == 0) 
-	  continue;
-	if( regular && S_ISREG(statStruct.st_mode) == 0) 
-	  continue;
 	if( block_special && S_ISBLK(statStruct.st_mode) == 0) 
 	  continue;
 	if( character_special && S_ISCHR(statStruct.st_mode) == 0) 
@@ -193,6 +370,11 @@ namespace lib {
 	  continue;
 	if( symlink && S_ISLNK(statStruct.st_mode) == 0) 
 	  continue;
+#endif
+	if( directory && S_ISDIR(statStruct.st_mode) == 0) 
+	  continue;
+	if( regular && S_ISREG(statStruct.st_mode) == 0) 
+	  continue;
 	
 	(*res)[ f] = 1;
       }
@@ -286,10 +468,18 @@ namespace lib {
 	if( entryStr != "." && entryStr != "..")
 	  {
 	    DString testFile = root + entryStr;
+#ifdef _MSC_VER
+	    int actStat = stat( testFile.c_str(), &statStruct);
+#else
 	    int actStat = lstat( testFile.c_str(), &statStruct);
+#endif
 	    if( S_ISDIR(statStruct.st_mode) == 0)
 	      { // only test non-dirs
+#ifdef _MSC_VER
+		int match = PathMatchSpecEx(entryStr.c_str(), pat.c_str(), 0);
+#else
 		int match = fnmatch( pat.c_str(), entryStr.c_str(), 0);
+#endif
 		if( match == 0)
 		  {
 		    closedir( dir);
@@ -339,14 +529,22 @@ namespace lib {
 	if( entryStr != "." && entryStr != "..")
 	  {
 	    DString testDir = root + entryStr;
+#ifdef _MSC_VER
+	    int actStat = stat( testDir.c_str(), &statStruct);
+#else
 	    int actStat = lstat( testDir.c_str(), &statStruct);
+#endif
 	    if( S_ISDIR(statStruct.st_mode) != 0)
 	      {
 		recurDir.push_back( testDir);
 	      }
 	    else if( notAdded)
 	      {
+#ifdef _MSC_VER
+		int match = PathMatchSpec(entryStr.c_str(), pat.c_str());
+#else
 		int match = fnmatch( pat.c_str(), entryStr.c_str(), 0);
+#endif
 		if( match == 0)
 		  notAdded = false;
 	      }
@@ -395,6 +593,10 @@ namespace lib {
 
     // dirN == "+DIRNAME"
 
+#ifdef _MSC_VER
+	// Windows does not use '~' as a home directory alias
+	DString initDir = dirN;
+#else
     // do first a glob because of '~'
     int flags = GLOB_TILDE | GLOB_NOSORT;
     glob_t p;
@@ -403,12 +605,13 @@ namespace lib {
     int gRes = glob( dirN.substr(offset_tilde).c_str(), flags, NULL, &p);
     if( gRes != 0 || p.gl_pathc == 0)
       {
-	globfree( &p);
-	return;
+		globfree( &p);
+		return;
       }
 
     DString initDir = p.gl_pathv[ 0];
     globfree( &p);
+#endif
     
     if (dirN[0] == '+')
       {
@@ -486,11 +689,13 @@ namespace lib {
   {
     int fnFlags = 0;
 
+#ifndef _MSC_VER
     if( !match_dot)
       fnFlags |= FNM_PERIOD;
 
     if( !quote)
       fnFlags |= FNM_NOESCAPE;
+#endif
 
     DString root = dirN;
     if( root != "")
@@ -550,13 +755,21 @@ namespace lib {
 	    if( root != "") // dirs for current ("") already included
 	      {
 		DString testDir = root + entryStr;
+#ifdef _MSC_VER
+		int actStat = stat( testDir.c_str(), &statStruct);
+#else
 		int actStat = lstat( testDir.c_str(), &statStruct);
+#endif
 		if( S_ISDIR(statStruct.st_mode) != 0)
 		    recurDir.push_back( testDir);
 	      }
 
 	    // dirs are also returned if they match
+#ifdef _MSC_VER
+	    int match = PathMatchSpec(entryStr.c_str(), pat.c_str());
+#else
 	    int match = fnmatch( pat.c_str(), entryStr.c_str(), fnFlags);
+#endif
 	    if( match == 0)
 	      fL.push_back( prefix + entryStr);
 	  }
@@ -621,6 +834,7 @@ DString makeInsensitive(const DString &s)
 	return insen;
 }
 
+#ifndef _MSC_VER
   void FileSearch( FileListT& fL, const DString& s, 
 		   bool environment,
 		   bool tilde,
@@ -869,6 +1083,7 @@ DString makeInsensitive(const DString &s)
 
     return res;
   }
+#endif
 
   BaseGDL* file_same( EnvT* e)
   {
@@ -999,27 +1214,32 @@ DString makeInsensitive(const DString &s)
 
         // stating the file (and moving on to the next file if failed)
 	struct stat statStruct;
+#ifdef _MSC_VER
+	if (stat(actFile, &statStruct) != 0) continue;
+#else
 	if (lstat(actFile, &statStruct) != 0) continue;
-
+#endif
         // checking struct tag indices (once)
         if (!indices_known) 
         {
           tExists =           res->Desc()->TagIndex("EXISTS"); 
           tRead =             res->Desc()->TagIndex("READ"); 
           tWrite =            res->Desc()->TagIndex("WRITE"); 
-          tExecute =          res->Desc()->TagIndex("EXECUTE"); 
           tRegular =          res->Desc()->TagIndex("REGULAR"); 
           tDirectory =        res->Desc()->TagIndex("DIRECTORY");
+#ifndef _MSC_VER
           tBlockSpecial =     res->Desc()->TagIndex("BLOCK_SPECIAL");
           tCharacterSpecial = res->Desc()->TagIndex("CHARACTER_SPECIAL");
           tNamedPipe =        res->Desc()->TagIndex("NAMED_PIPE");
+          tExecute =          res->Desc()->TagIndex("EXECUTE"); 
           tSetuid =           res->Desc()->TagIndex("SETUID");
           tSetgid =           res->Desc()->TagIndex("SETGID");
-          tSocket =           res->Desc()->TagIndex("SOCKET");
+		  tSocket =           res->Desc()->TagIndex("SOCKET");
           tStickyBit =        res->Desc()->TagIndex("STICKY_BIT");
           tSymlink =          res->Desc()->TagIndex("SYMLINK");
           tDanglingSymlink =  res->Desc()->TagIndex("DANGLING_SYMLINK");
           tMode =             res->Desc()->TagIndex("MODE");
+#endif
           tAtime =            res->Desc()->TagIndex("ATIME");
           tCtime =            res->Desc()->TagIndex("CTIME");
           tMtime =            res->Desc()->TagIndex("MTIME");
@@ -1033,25 +1253,29 @@ DString makeInsensitive(const DString &s)
         // READ, WRITE, EXECUTE
         *(res->GetTag(tRead, f)) =    DByteGDL(access(actFile, R_OK) == 0);
         *(res->GetTag(tWrite, f)) =   DByteGDL(access(actFile, W_OK) == 0);
+#ifndef _MSC_VER
         *(res->GetTag(tExecute, f)) = DByteGDL(access(actFile, X_OK) == 0);
+#endif
 
         // REGULAR, DIRECTORY, BLOCK_SPECIAL, CHARACTER_SPECIAL, NAMED_PIPE, SOCKET
         *(res->GetTag(tRegular, f)) =          DByteGDL(S_ISREG( statStruct.st_mode) != 0);
         *(res->GetTag(tDirectory, f)) =        DByteGDL(S_ISDIR( statStruct.st_mode) != 0);
+#ifndef _MSC_VER
         *(res->GetTag(tBlockSpecial, f)) =     DByteGDL(S_ISBLK( statStruct.st_mode) != 0);
         *(res->GetTag(tCharacterSpecial, f)) = DByteGDL(S_ISCHR( statStruct.st_mode) != 0);
         *(res->GetTag(tNamedPipe, f)) =        DByteGDL(S_ISFIFO(statStruct.st_mode) != 0);
         *(res->GetTag(tSocket, f)) =           DByteGDL(S_ISSOCK(statStruct.st_mode) != 0);
-  
+#endif  
         // SETUID, SETGID, STICKY_BIT
+#ifndef _MSC_VER
         *(res->GetTag(tSetuid, f)) =           DByteGDL((S_ISUID & statStruct.st_mode) != 0);
         *(res->GetTag(tSetgid, f)) =           DByteGDL((S_ISGID & statStruct.st_mode) != 0);
         *(res->GetTag(tStickyBit, f)) =        DByteGDL((S_ISVTX & statStruct.st_mode) != 0);
-
         // MODE
         *(res->GetTag(tMode, f)) = DLongGDL(
           statStruct.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID | S_ISGID | S_ISVTX)
         );
+#endif
 
         // ATIME, CTIME, MTIME
         *(res->GetTag(tAtime, f)) = DLong64GDL(statStruct.st_atime);
@@ -1061,14 +1285,15 @@ DString makeInsensitive(const DString &s)
         // SIZE
 	*(res->GetTag(tSize, f)) = DLong64GDL(statStruct.st_size);
 
-        // SYMLINK, DANLING_SYMLINK 
+        // SYMLINK, DANLING_SYMLINK
+#ifndef _MSC_VER // No symlinks in windows
         if (S_ISLNK(statStruct.st_mode) != 0)
         {
           *(res->GetTag(tSymlink, f)) = DByteGDL(1);
           // warning: statStruct now describes the linked file
           *(res->GetTag(tDanglingSymlink, f)) = DByteGDL(stat(actFile, &statStruct) != 0);
         }
-	
+#endif
       }
 
     return res;
@@ -1087,8 +1312,11 @@ DString makeInsensitive(const DString &s)
 
     static int noexpand_pathIx = e->KeywordIx( "NOEXPAND_PATH");
     bool noexpand_path = e->KeywordSet( noexpand_pathIx);
-
+#ifdef _MSC_VER
+    string cmd = "md"; // windows always creates all of the non-existing directories
+#else
     string cmd = "mkdir -p";
+#endif
     for (int i=0; i<nParam; i++)
     {
       DStringGDL* pi = dynamic_cast<DStringGDL*>(e->GetParDefined(i));
@@ -1099,7 +1327,9 @@ DString makeInsensitive(const DString &s)
         cmd.append(" " + tmp);
       }
     }
+#ifndef _MSC_VER
     cmd.append(" 2>&1 | awk '{print \"% FILE_MKDIR: \" $0; exit 1}'");
+#endif
     // SA: calling system(), mkdir and awk is surely not the most efficient way, 
     //     but copying a bunch of code from coreutils does not seem elegant either
     if (system(cmd.c_str()) != 0) e->Throw("failed to create a directory (or execute mkdir).");
