@@ -167,16 +167,17 @@ void GDLXStream::Clear( DLong bColor)
   GDLCT* actCT = Graphics::GetCT();
   actCT->Get( bColor, rb, gb, bb);
 
-  // Convert to PLINT from BYTE
+  // Convert to PLINT from GDL_BYTE
   r1 = (PLINT) rb;
   g1 = (PLINT) gb;
   b1 = (PLINT) bb;
-
+// this mimics better the *DL behaviour.
+  ::c_plbop();
   plscolbg (r1, g1, b1);
 
-  ::c_plclear();
-
-  plscolbg (r0, g0, b0);
+//  ::c_plclear();
+//
+//  plscolbg (r0, g0, b0);
 }
 
 void GDLXStream::Raise()
@@ -193,4 +194,22 @@ void GDLXStream::Lower()
   XLowerWindow(dev->xwd->display, dev->window);
 }
 
+// note by AC on 2012-Aug-16    Help/suggestions welcome
+// I don't know how to find the sub-window number (third parametre
+// in call XIconifyWindow())
+
+void GDLXStream::Iconic()
+{
+  XwDev *dev = (XwDev *) pls->dev;
+  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+  XIconifyWindow(dev->xwd->display, dev->window,0);
+}
+
+void GDLXStream::DeIconic()
+{
+  XwDev *dev = (XwDev *) pls->dev;
+  XwDisplay *xwd = (XwDisplay *) dev->xwd;
+  XMapWindow(dev->xwd->display, dev->window);
+}
+
 #endif
