@@ -49,6 +49,12 @@ namespace lib {
   void inplacemxradixfft(double a[], double b[], 
 			 int ntot, int n, int nspan, int isn);
 
+  // the following by AC
+  BaseGDL* qromb_fun(EnvT* e);
+  BaseGDL* qromo_fun(EnvT* e);
+  BaseGDL* fz_roots_fun(EnvT* e);
+  BaseGDL* fx_root_fun(EnvT* e);
+
   // the following by SA
   BaseGDL* newton_broyden(EnvT* e);
   BaseGDL* constant(EnvT* e);
