@@ -22,6 +22,163 @@
 #include "objects.hpp"
 #include "io.hpp"
 
+#ifdef _MSC_VER
+#define NS_INT16SZ       2
+#define NS_INADDRSZ      4
+#define NS_IN6ADDRSZ    16
+
+static int inet_pton4( const char *src, unsigned char *dst )
+{
+  static const char digits[] = "0123456789";
+  int saw_digit, octets, ch;
+  unsigned char tmp[NS_INADDRSZ], *tp;
+
+  saw_digit = 0;
+  octets = 0;
+  *(tp = tmp) = 0;
+  while( (ch = *src++) != ' ' )
+  {
+    const char *pch;
+
+    if( (pch = strchr(digits, ch)) != NULL )
+    {
+      unsigned int iNew = (unsigned int)(*tp * 10 + (pch - digits));
+
+      if( iNew > 255 ) return (0);
+      *tp = iNew;
+      if( !saw_digit )
+      {
+        if( ++octets > 4 ) return (0);
+        saw_digit = 1;
+      }
+    }
+    else if( ch == '.' && saw_digit )
+    {
+      if( octets == 4 ) return (0);
+      *++tp = 0;
+      saw_digit = 0;
+    }
+    else
+    {
+      return (0);
+    }
+  }
+
+  if( octets < 4 ) return (0);
+
+  memcpy( dst, tmp, NS_INADDRSZ );
+
+  return (1);
+}
+
+static int inet_pton6( const char *src, unsigned char * dst )
+{
+  static const char xdigits_l[] = "0123456789abcdef",
+                    xdigits_u[] = "0123456789ABCDEF";
+  unsigned char tmp[NS_IN6ADDRSZ], *tp, *endp, *colonp;
+  const char *xdigits, *curtok;
+  int ch, saw_xdigit;
+  unsigned int val;
+
+  memset((tp = tmp), ' ', NS_IN6ADDRSZ);
+  endp = tp + NS_IN6ADDRSZ;
+  colonp = NULL;
+
+  // Leading :: requires some special handling.
+  if( *src == ':' && *++src != ':' ) return (0);
+
+  curtok = src;                                                                              
+  saw_xdigit = 0;
+  val = 0;
+  while( ( ch = *src++ ) != ' ' )
+  {
+    const char *pch;
+
+    if( ( pch = strchr( ( xdigits = xdigits_l ), ch ) ) == NULL )
+      pch = strchr( (xdigits = xdigits_u), ch );
+
+    if( pch != NULL )
+    {
+      val <<= 4;
+      val |= (pch - xdigits);
+      if( val > 0xffff ) return (0);
+      saw_xdigit = 1;
+      continue;
+    }
+
+    if( ch == ':' )
+    {
+      curtok = src;
+      if( !saw_xdigit )
+      {
+        if( colonp ) return (0);
+        colonp = tp;
+        continue;
+      }
+
+      if( tp + NS_INT16SZ > endp ) return (0);
+
+      *tp++ = (unsigned char) (val >> 8) & 0xff;
+      *tp++ = (unsigned char) val & 0xff;
+      saw_xdigit = 0;
+      val = 0;
+      continue;
+    }
+
+    if( ch == '.' && ((tp + NS_INADDRSZ) <= endp) && inet_pton4(curtok, tp) > 0 )
+    {
+      tp += NS_INADDRSZ;
+      saw_xdigit = 0;
+      break;  /* ' ' was seen by inet_pton4(). */
+    }
+    return (0);
+  }
+
+  if( saw_xdigit )
+  {
+    if( tp + NS_INT16SZ > endp ) return (0);
+
+    *tp++ = (unsigned char) (val >> 8) & 0xff;
+    *tp++ = (unsigned char) val & 0xff;
+  }
+
+  if( colonp != NULL )
+  {
+    // Since some memmove()'s erroneously fail to handle overlapping regions, we'll do the shift by hand.
+    const int n = (int)(tp - colonp);
+    int i;
+
+    for( i = 1; i <= n; i++ )
+    {
+      endp[- i] = colonp[n - i];
+      colonp[n - i] = 0;
+    }
+    tp = endp;
+  }
+
+  if( tp != endp ) return (0);
+  
+  memcpy(dst, tmp, NS_IN6ADDRSZ);
+  return (1);
+}                                                                                                                            
+
+int inet_pton( int af, const char *src, void *dst )
+{
+  switch (af)
+  {
+  case AF_INET:
+    return inet_pton4( src, (unsigned char *)dst );
+  case AF_INET6:
+    return inet_pton6( src, (unsigned char *)dst );
+  default:
+    errno = EAFNOSUPPORT;
+    return (-1);
+  }
+}
+// This source code is from http://blog.naver.com/PostView.nhn?blogId=websearch&logNo=70089324416&parentCategoryNo=4&viewDate=&currentPage=1&listtype=0
+#endif
+
+
 using namespace std;
 
 const string StreamInfo( ios* searchStream)
@@ -295,6 +452,8 @@ void GDLStream::Free()
 
   delete recvBuf;
   recvBuf = NULL;
+  
+  getLunLock = false;
 }
  
 igzstream& GDLStream::IgzStream()
