@@ -81,6 +81,9 @@ void LibInit()
   LibInit_ng(); 
   const char KLISTEND[] = "";
 
+  const string scope_varfetchKey[]={"LEVEL", KLISTEND};
+  new DLibFun(lib::scope_varfetch_value,string("SCOPE_VARFETCH"),-1,scope_varfetchKey);
+
   const string cpuKey[]={ "RESET","RESTORE","TPOOL_MAX_ELTS", "TPOOL_MIN_ELTS",
 					"TPOOL_NTHREADS","VECTOR_ENABLE",KLISTEND};
   new DLibPro(lib::cpu,string("CPU"),0,cpuKey);
@@ -91,15 +94,17 @@ void LibInit()
   const string svdcKey[]={"COLUMN","ITMAX","DOUBLE",KLISTEND};
   new DLibPro(lib::svdc,string("SVDC"),4,svdcKey);
 
-  new DLibFun(lib::temporary,string("TEMPORARY"),1);
+  new DLibFunRetNew(lib::temporary,string("TEMPORARY"),1);
 
   const string routine_infoKey[]={"FUNCTIONS","SYSTEM","DISABLED","ENABLED",
 				  "PARAMETERS", KLISTEND};
   new DLibFun(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
 
+#ifndef _MSC_VER
   const string spawnKey[]={ "COUNT","EXIT_STATUS","PID",
 			    "SH","NOSHELL","UNIT",KLISTEND};
   new DLibPro(lib::spawn_pro,string("SPAWN"),3,spawnKey);
+#endif
 
   const string bytsclKey[]={"MIN","MAX","TOP","NAN",KLISTEND};
   new DLibFun(lib::bytscl,string("BYTSCL"),3,bytsclKey);
@@ -126,12 +131,14 @@ void LibInit()
   const string convolKey[]={"CENTER","EDGE_TRUNCATE","EDGE_WRAP",KLISTEND};
   new DLibFunRetNew(lib::convol,string("CONVOL"),3,convolKey);
 
+#ifndef _MSC_VER
   const string file_searchKey[]={"COUNT","EXPAND_ENVIRONMENT","EXPAND_TILDE",
 				 "FOLD_CASE","ISSUE_ACCESS_ERROR",
 				 "MARK_DIRECTORY","NOSORT","QUOTE",
 				 "MATCH_INITIAL_DOT",
 				 "MATCH_ALL_INITIAL_DOT","FULLY_QUALIFY_PATH",KLISTEND};
   new DLibFunRetNew(lib::file_search,string("FILE_SEARCH"),2,file_searchKey);
+#endif
 
   const string expand_pathKey[]={"ARRAY","ALL_DIRS","COUNT",KLISTEND};
   new DLibFunRetNew(lib::expand_path,string("EXPAND_PATH"),1,expand_pathKey);
@@ -193,8 +200,8 @@ void LibInit()
   
   const string helpKey[]={"STRUCTURES","ROUTINES","BRIEF",
 			  "OUTPUT","PROCEDURES","FUNCTIONS",
-			  "INFO","LIB","CALLS","RECALL_COMMANDS",
-                          "MEMORY", KLISTEND};
+			  "INFO","LIB","LIB_GDL_INTERNAL","CALLS","RECALL_COMMANDS",
+                          "MEMORY","PREFERENCES", KLISTEND};
   new DLibPro(lib::help,string("HELP"),-1,helpKey);
 
   const string memoryKey[]={"CURRENT","HIGHWATER","NUM_ALLOC",
@@ -227,6 +234,7 @@ void LibInit()
 
   const string heap_gcKey[]={"PTR","OBJ","VERBOSE",KLISTEND};
   new DLibPro(lib::heap_gc,string("HEAP_GC"),0,heap_gcKey); 
+  new DLibPro(lib::heap_free,string("HEAP_FREE"),1,heap_gcKey); 
 
 
   new DLibPro(lib::ptr_free,string("PTR_FREE"),-1);
@@ -282,7 +290,7 @@ void LibInit()
   new DLibFunRetNew(lib::cindgen,string("CINDGEN"),MAXRANK,NULL,NULL,true);
   new DLibFunRetNew(lib::dcindgen,string("DCINDGEN"),MAXRANK,NULL,NULL,true);
 
-  new DLibFunRetNew(lib::n_elements,string("N_ELEMENTS"),1,NULL,NULL,true);
+  new DLibFunRetNew(lib::n_elements,string("N_ELEMENTS"),1,NULL,NULL,true,1);
 
   new DLibFun(lib::execute,string("EXECUTE"),2);
 
@@ -320,7 +328,7 @@ void LibInit()
   new DLibPro(lib::writeu,string("WRITEU"),-1,writeuKey);
   new DLibPro(lib::readu,string("READU"),-1,writeuKey);
 
-  const string resolve_routineWarnKey[]={"EITHER","IS_FUNCTION","NO_RECOMPILE",KLISTEND};
+  const string resolve_routineWarnKey[]={"EITHER","IS_FUNCTION","NO_RECOMPILE","COMPILE_FULL_FILE",KLISTEND};
   const string resolve_routineKey[]={KLISTEND};
   new DLibPro(lib::resolve_routine,string("RESOLVE_ROUTINE"),1,
 	      NULL, resolve_routineWarnKey);
@@ -336,7 +344,6 @@ void LibInit()
   new DLibFun(lib::string_fun,string("STRING"),-1,stringKey,NULL);
   new DLibFun(lib::byte_fun,string("BYTE"),10,NULL,NULL);
 
-  const string fixKey[]={"TYPE","PRINT",KLISTEND};
 /*
   new DLibFunRetNew(lib::fix_fun,string("FIX"),10,fixKey,NULL,true);
   new DLibFunRetNew(lib::uint_fun,string("UINT"),10,NULL,NULL,true);
@@ -350,7 +357,9 @@ void LibInit()
   new DLibFunRetNew(lib::dcomplex_fun,string("DCOMPLEX"),MAXRANK+2,NULL,NULL,true);
 */
 // that's apparently the desired bahaviour, see bug no. 3151760
+  const string fixKey[]={"TYPE","PRINT",KLISTEND};
   new DLibFun(lib::fix_fun,string("FIX"),10,fixKey,NULL);
+
   new DLibFun(lib::uint_fun,string("UINT"),10,NULL,NULL);
   new DLibFun(lib::long_fun,string("LONG"),10,NULL,NULL);
   new DLibFun(lib::ulong_fun,string("ULONG"),10,NULL,NULL);
@@ -358,36 +367,40 @@ void LibInit()
   new DLibFun(lib::ulong64_fun,string("ULONG64"),10,NULL,NULL);
   new DLibFun(lib::float_fun,string("FLOAT"),10,NULL,NULL);
   new DLibFun(lib::double_fun,string("DOUBLE"),10,NULL,NULL);
-  new DLibFun(lib::complex_fun,string("COMPLEX"),MAXRANK+2,NULL,NULL);
+
+  const string complexKey[]={"DOUBLE",KLISTEND};
+  new DLibFun(lib::complex_fun,string("COMPLEX"),MAXRANK+2,complexKey,NULL);
   new DLibFun(lib::dcomplex_fun,string("DCOMPLEX"),MAXRANK+2,NULL,NULL);
 
   new DLibFunRetNew(lib::gdl_logical_and,string("LOGICAL_AND"),2,NULL,NULL,true);
   new DLibFunRetNew(lib::gdl_logical_or,string("LOGICAL_OR"),2,NULL,NULL,true);
-  new DLibFunRetNew(lib::logical_true,string("LOGICAL_TRUE"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::logical_true,string("LOGICAL_TRUE"));
 
   new DLibFunRetNew(lib::replicate,string("REPLICATE"),9,NULL,NULL,true);
   new DLibPro(lib::replicate_inplace_pro,string("REPLICATE_INPLACE"),6);
 
-  new DLibFunRetNew(lib::sin_fun,string("SIN"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::cos_fun,string("COS"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::tan_fun,string("TAN"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::sin_fun,string("SIN"));
+  new DLibFunDirect(lib::cos_fun,string("COS"));
+  new DLibFunDirect(lib::tan_fun,string("TAN"));//,1,NULL,NULL,true);
 
-  new DLibFunRetNew(lib::sinh_fun,string("SINH"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::cosh_fun,string("COSH"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::tanh_fun,string("TANH"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::sinh_fun,string("SINH"));//,1,NULL,NULL,true);
+  new DLibFunDirect(lib::cosh_fun,string("COSH"));//,1,NULL,NULL,true);
+  new DLibFunDirect(lib::tanh_fun,string("TANH"));//,1,NULL,NULL,true);
 
-  new DLibFunRetNew(lib::asin_fun,string("ASIN"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::acos_fun,string("ACOS"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::asin_fun,string("ASIN"));
+  new DLibFunDirect(lib::acos_fun,string("ACOS"));
   const string atanKey[] = {"PHASE", KLISTEND};
   new DLibFunRetNew(lib::atan_fun,string("ATAN"),2,atanKey,NULL,true);
 
-  new DLibFunRetNew(lib::alog_fun,string("ALOG"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::alog10_fun,string("ALOG10"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::alog_fun,string("ALOG"));
+  new DLibFunDirect(lib::alog10_fun,string("ALOG10"));
+//   new DLibFunRetNew(lib::alog_fun,string("ALOG"),1,NULL,NULL,true,1);
+//   new DLibFunRetNew(lib::alog10_fun,string("ALOG10"),1,NULL,NULL,true,1);
 
-  new DLibFunRetNew(lib::sqrt_fun,string("SQRT"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::abs_fun,string("ABS"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::sqrt_fun,string("SQRT"));
+  new DLibFunDirect(lib::abs_fun,string("ABS"));
 
-  new DLibFunRetNew(lib::exp_fun,string("EXP"),1,NULL,NULL,true);
+  new DLibFunDirect(lib::exp_fun,string("EXP"));
 
   const string roundKey[]={"L64",KLISTEND};
   // retConstant: check definition of the rounding functions if they depend 
@@ -398,22 +411,24 @@ void LibInit()
   new DLibFunRetNew(lib::ceil_fun,string("CEIL"),1,ceilfloorKey);
   new DLibFunRetNew(lib::floor_fun,string("FLOOR"),1,ceilfloorKey);
 
-  new DLibFunRetNew(lib::conj_fun,string("CONJ"),1);
-  new DLibFunRetNew(lib::imaginary_fun,string("IMAGINARY"),1);
+  new DLibFunDirect(lib::conj_fun,string("CONJ"));
+  new DLibFunDirect(lib::imaginary_fun,string("IMAGINARY"));
 
   const string strcompressKey[]={"REMOVE_ALL",KLISTEND};
   new DLibFunRetNew(lib::strcompress,string("STRCOMPRESS"),1,strcompressKey,NULL,true);
-  new DLibFunRetNew(lib::strlowcase,string("STRLOWCASE"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::strupcase,string("STRUPCASE"),1,NULL,NULL,true);
-  new DLibFunRetNew(lib::strlen,string("STRLEN"),1,NULL,NULL,true);
+  
+  new DLibFunDirect(lib::strlowcase,string("STRLOWCASE"));
+  new DLibFunDirect(lib::strupcase,string("STRUPCASE"));
+  new DLibFunDirect(lib::strlen,string("STRLEN"));
+
   const string strmidKey[]={"REVERSE_OFFSET",KLISTEND};
   new DLibFunRetNew(lib::strmid,string("STRMID"),3,strmidKey,NULL,true);
   new DLibFunRetNew(lib::strtrim,string("STRTRIM"),2,NULL,NULL,true);
   const string strposKey[]={"REVERSE_OFFSET","REVERSE_SEARCH",KLISTEND};
-  new DLibFunRetNew(lib::strpos,string("STRPOS"),3,strposKey,NULL,true);
-  new DLibPro(lib::strput,string("STRPUT"),3);
+  new DLibFunRetNew(lib::strpos,string("STRPOS"),3,strposKey,NULL,true,2);
+  new DLibPro(lib::strput,string("STRPUT"),3,NULL,NULL,2);
   
-  const string whereKey[]={"COMPLEMENT","NCOMPLEMENT",KLISTEND};
+  const string whereKey[]={"COMPLEMENT","NCOMPLEMENT","NULL",KLISTEND};
   new DLibFunRetNew(lib::where,string("WHERE"),2,whereKey);
 
   const string totalKey[]={"CUMULATIVE","DOUBLE","NAN","INTEGER","PRESERVE_TYPE",KLISTEND};
@@ -463,8 +478,9 @@ void LibInit()
   new DLibPro(lib::window,string("WINDOW"),1,windowKey);
   new DLibPro(lib::wdelete,string("WDELETE"),-1);
   new DLibPro(lib::wset,string("WSET"),1);
-  const string wshowWarnKey[]={"ICONIC", KLISTEND};
-  new DLibPro(lib::wshow,string("WSHOW"),2,NULL,wshowWarnKey);
+
+  const string wshowKey[]={"ICONIC", KLISTEND};
+  new DLibPro(lib::wshow,string("WSHOW"),2,wshowKey);
 
   const string cursorKey[]={"CHANGE","DOWN","NOWAIT","UP","WAIT",
 				"DATA","DEVICE","NORMAL",KLISTEND};
@@ -490,13 +506,20 @@ void LibInit()
   const string deviceWarnKey[] = {"RETAIN", "SET_FONT", "HELVETICA", 
     "AVANTGARDE", "BKMAN", "COURIER", "PALATINO", 
     "SCHOOLBOOK", "TIMES", "ZAPFCHANCERY", "ZAPFDINGBATS", "BITS_PER_PIXEL", 
-    "ITALIC", "BOLD", KLISTEND};
+    "ITALIC", "BOLD", "TRUE_COLOR", KLISTEND};
   new DLibPro(lib::device,string("DEVICE"),0, deviceKey, deviceWarnKey);
 
+  const string usersymKey[]= 
+  {
+    "COLOR","FILL","THICK",
+    KLISTEND
+   };
+  new DLibPro(lib::usersym,string("USERSYM"),2, usersymKey);
+
   const string plotKey[]=
     {
      "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-     "COLOR",     "DATA",    "DEVICE", 
+     "COLOR",     "DATA",    "DEVICE", "ISOTROPIC",
      "LINESTYLE", "NOCLIP",  "NODATA",   "NOERASE", 
      "NORMAL",    "POSITION","PSYM",     "SUBTITLE",
      "SYMSIZE",   "THICK",    "TICKLEN", "TITLE",
@@ -508,12 +531,12 @@ void LibInit()
      "YTICKS",    "YTITLE", "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
      "ZRANGE",    "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
      "ZTICKLEN",  "ZTICKS",    "ZTITLE", "ZVALUE", "MAX_VALUE", "MIN_VALUE",
-     "XLOG",      "YLOG",     "YNOZERO", "XTYPE", "YTYPE", 
+     "XLOG",      "YLOG",     "YNOZERO", "XTYPE", "YTYPE", "POLAR", "NSUM",
      KLISTEND
     };
-  //"POLAR",
+  //
   const string plotWarnKey[]= {
-    "ISOTROPIC", "FONT", "T3D", "YTICKUNITS", "NSUM", "XTICKLAYOUT", "YTICKLAYOUT", 
+    "FONT", "T3D", "YTICKUNITS", "XTICKLAYOUT", "YTICKLAYOUT", 
     "ZTICKLAYOUT", "XGRIDSTYLE", "YGRIDSTYLE", "XTICKUNITS", "XTICKV", "XTICK_GET", "YTICKV",
     "YTICK_GET", "XTICKNAME", "ZTICKNAME", "ZTICKUNITS", "ZTICKV",  "ZTICK_GET",
     "XTICKINTERVAL", "YTICKINTERVAL", "YTICKNAME", "ZTICKINTERVAL", 
@@ -546,10 +569,19 @@ void LibInit()
      "ZVALUE", 
 
      "SAVE", "XAXIS", "YAXIS", "XLOG", "YLOG", "XTYPE", "YTYPE", "YNOZERO", "THICK",
-     "XTICKINTERVAL", "YTICKINTERVAL",
-     KLISTEND
-    };
-  new DLibPro(lib::axis,string("AXIS"),3,axisKey);
+      KLISTEND
+     };
+   const string axisWarnKey[]=
+   {
+   "ZAXIS","ZLOG", 
+   "XTICKINTERVAL", "YTICKINTERVAL", "ZTICKINTERVAL",
+   "XTICKLAYOUT", "YTICKLAYOUT", "ZTICKLAYOUT",
+   "XTICKUNITS", "YTICKUNITS", "ZTICKUNITS",
+   "XTICKV", "YTICKV", "ZTICKV",
+   "XTICK_GET", "YTICK_GET", "ZTICK_GET",
+   KLISTEND
+  };
+  new DLibPro(lib::axis,string("AXIS"),3,axisKey,axisWarnKey);
 
   const string oplotKey[]=
     {
@@ -559,7 +591,7 @@ void LibInit()
       // 4
       "PSYM", "SYMSIZE",  "T3D",  "THICK",
       // 8
-      "MAX_VALUE", "MIN_VALUE","NSUM",  "POLAR",
+      "MAX_VALUE", "MIN_VALUE", "NSUM", "POLAR",
       KLISTEND
     };
   new DLibPro(lib::oplot, string("OPLOT"),2,oplotKey);
@@ -627,13 +659,12 @@ void LibInit()
 
      // SURFACE keywords
      // 73
-     "MAX_VALUE", "MIN_VALUE","AX", "AZ",
-     "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE",
-     KLISTEND
+     "MAX_VALUE",  "MIN_VALUE", "AX", "AZ", "XLOG", "YLOG", "ZLOG", KLISTEND
     };
   const string surfaceWarnKey[]=
   {
-    "SAVE", KLISTEND 
+    "SAVE", "BOTTOM", "HORIZONTAL", "LOWER_ONLY", "UPPER_ONLY",
+    "SHADES", "SKIRT", "ZAXIS", "LEGO", KLISTEND
   };
   new DLibPro(lib::surface,string("SURFACE"),3,surfaceKey, surfaceWarnKey);
 
@@ -663,21 +694,31 @@ void LibInit()
      "ZVALUE", 
      "LEVELS", "NLEVELS",
      "MAX_VALUE", "MIN_VALUE",
-     "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE", "FILL",  "ISOTROPIC",
+     "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE", "FILL", "ISOTROPIC",
      "FOLLOW", "C_CHARSIZE","OVERPLOT","C_COLORS","C_LINESTYLE",
      KLISTEND
     };
-  new DLibPro(lib::contour,string("CONTOUR"),3,contourKey);
+   const string contourWarnKey[]=
+    {
+        "CELL_FILL","C_LABELS","C_ANNOTATIONS", "C_CHARTHICK","C_ORIENTATION","C_SPACING","C_THICK",KLISTEND 
+    };
+   new DLibPro(lib::contour,string("CONTOUR"),3,contourKey,contourWarnKey);
 
+  // note that although  SIZE is not in IDL6+ documentation, it is widely used
+  // in place of CHARSIZE....
   const string xyoutsKey[]=
     {
       "COLOR","DATA","NORMAL","DEVICE","CLIP",
-      "ORIENTATION","ALIGNMENT","CHARSIZE","CHARTHICK",
-      "NOCLIP","Z","WIDTH",KLISTEND
+      "ORIENTATION","ALIGNMENT","CHARSIZE","SIZE","CHARTHICK",
+      "NOCLIP","Z","WIDTH","FONT",KLISTEND
     };
-  new DLibPro(lib::xyouts, string("XYOUTS"), 3, xyoutsKey);
-
-  const string polyfillKey[]=
+   const string xyoutsWarnKey[]=
+    {
+      "TEXT_AXES","T3D", KLISTEND 
+    };  
+   new DLibPro(lib::xyouts, string("XYOUTS"), 3, xyoutsKey, xyoutsWarnKey);
+   
+   const string polyfillKey[]=
     {
       "COLOR","DATA","NORMAL","DEVICE","CLIP","NOCLIP",
       "LINE_FILL","SPACING","LINESTYLE","ORIENTATION","THICK",
@@ -685,7 +726,8 @@ void LibInit()
     };
   new DLibPro(lib::polyfill, string("POLYFILL"), 3, polyfillKey);
 
-  new DLibPro(lib::erase,string("ERASE"),1);
+  const string eraseKey[]= {"CHANNEL","COLOR",KLISTEND};
+  new DLibPro(lib::erase,string("ERASE"),1, eraseKey);
   
   const string laguerreKey[]={"DOUBLE","COEFFICIENTS",KLISTEND};
   new DLibFun(lib::laguerre,string("LAGUERRE"),3,laguerreKey);
