@@ -23,6 +23,8 @@
 #endif
 
 #include "math_fun_ac.hpp"
+#include "gsl_matrix.hpp"
+#include "gsl_fun.hpp"
 
 using namespace std;
 
@@ -31,6 +33,8 @@ void LibInit_ac()
   const char KLISTEND[] = "";
 
 #if defined(HAVE_LIBGSL)
+
+  // Besel functions family
   const string beseliKey[]={"DOUBLE","ITER","HELP",KLISTEND};
   new DLibFun(lib::beseli_fun,string("BESELI"),2,beseliKey);
   const string beseljKey[]={"DOUBLE","ITER","HELP",KLISTEND};
@@ -39,6 +43,33 @@ void LibInit_ac()
   new DLibFun(lib::beselk_fun,string("BESELK"),2,beselkKey);
   const string beselyKey[]={"DOUBLE","ITER","HELP",KLISTEND};
   new DLibFun(lib::besely_fun,string("BESELY"),2,beselyKey);
+
+  // Matrix functions family
+  const string ludcKey[]={"COLUMN","DOUBLE","INTERCHANGES",KLISTEND};
+  new DLibPro(lib::ludc_pro,string("LUDC"),2,ludcKey);
+  const string lusolKey[]={"COLUMN","DOUBLE",KLISTEND};
+  new DLibFun(lib::lusol_fun,string("LUSOL"),3,lusolKey);
+  const string determKey[]={"DOUBLE","CHECK","ZERO",KLISTEND};
+  new DLibFun(lib::determ_fun,string("DETERM"),1,determKey);
+
+  const string trisolKey[]={"DOUBLE",KLISTEND};
+  new DLibFun(lib::trisol_fun,string("TRISOL"),4,trisolKey);
+
+  // QSIMP/QROMB/QROMO (we used same code for QSIMP and QROMB)
+  const string qsimpKey[]={"DOUBLE","EPS","JMAX",KLISTEND};
+  new DLibFun(lib::qromb_fun,string("QSIMP"),3,qsimpKey);
+  const string qrombKey[]={"DOUBLE","EPS","JMAX","K",KLISTEND};
+  new DLibFun(lib::qromb_fun,string("QROMB"),3,qrombKey);
+  const string qromoKey[]={"DOUBLE","EPS","JMAX","K","MIDEXP",
+			   "MIDINF","MIDPNT","MIDSQL","MIDSQU",KLISTEND};
+  new DLibFun(lib::qromo_fun,string("QROMO"),3,qromoKey);
+  const string fz_rootsKey[]={"DOUBLE","EPS","NO_POLISH",KLISTEND};
+  new DLibFun(lib::fz_roots_fun,string("FZ_ROOTS"),3,fz_rootsKey);
+  const string fx_rootKey[]={"DOUBLE","ITMAX","STOP","TOL",KLISTEND};
+  new DLibFun(lib::fx_root_fun,string("FX_ROOT"),2,fx_rootKey);
+  
+
+
 #endif
   
   const string spl1Key[]={"YP0","YPN_1","DOUBLE","HELP",KLISTEND};
