@@ -207,6 +207,9 @@ void LibInit_cl()
   const string magick_pingKey[] = {"INFO", "CHANNELS", "DIMENSIONS", "HAS_PALETTE", "IMAGE_INDEX",
     "NUM_IMAGES", "PIXEL_TYPE",/* "SUPPORTED_READ", "SUPPORTED_WRITE", */"TYPE"};
   new DLibFun(lib::magick_ping, string("MAGICK_PING"), 2, magick_pingKey);
+
+  // see bug no. 3376577
+  Magick::InitializeMagick(NULL); 
 #endif
 }
 
