@@ -165,10 +165,14 @@ namespace lib {
       DString filename;
       e->AssureScalarPar<DStringGDL>(0, filename);
       if (filename.length() == 0) return new DLongGDL(0);
+
+      WordExp( filename);
+
       Image a;
       try 
       {
-        a.ping(filename);
+	a.ping(filename);
+	//a.read(filename);
       }
       catch (WarningCoder &warning_ )
       {
@@ -182,12 +186,30 @@ namespace lib {
         if (a.magick() != magick) return new DLongGDL(0); 
       }
   
-      DInt has_palette, pixel_type;
-      DLong channels, num_images, image_index;
-      DString type;
-     
+      int debug=0;
+      if (debug == 1) {
+	cout << "a.type()      :" << a.type() << endl;
+	cout << "a.classType() :" << a.classType() << endl;
+	cout << "a.matte()     :" << a.matte() << endl;
+	// no useful info here:cout << "a.colorSpace()     :" << a.colorSpace() << endl;
+	// Always 8:cout << "a.depth()     :" << a.depth() << endl;
+	// Always 1: cout << "a.colorSpace() :" << a.colorSpace() << endl;
+      }
+      
+      // AC 2012-May-10
+      // http://www.graphicsmagick.org/Magick++/Image.html#type
       // relevant information that, in some cases, is provided after pinging:
-      // a.type(), a.matte(), a.classType(), a.colorSpace()
+      // a.matte(), a.classType() [and a.type() for Palette info only]
+      DLong channels=0;
+      if (a.classType() == 1) channels=3; // DirectColor
+      if (a.classType() == 2) channels=1; // PseudoColor
+      if (channels ==0) cout << "no ClassType found for current Image" << endl;
+
+      // AC 2012-May-10 this is NOT working with only a a.ping()
+      // a.type() is FULLY reliable if and only if a.read() was done before !!! 
+      //http://www.graphicsmagick.org/Magick++/Image.html#type
+      // (should be OK with a a.read())
+      /*
       channels = a.classType() == PseudoClass 
         ? 1      // color palette
         : a.type() == GrayscaleType 
@@ -195,22 +217,34 @@ namespace lib {
           : a.type() == ColorSeparationType 
             ? 4  // CMYK
             : 3; // RGB
+      */
+
+      // AC 2012-May-10 this is OK (reliable), see exemple "589 Lavandula mono"
       if (a.matte()) channels += 1;
 
       // TODO! multiple images (using the Magick++ STL interface)
+      DLong image_index, num_images;
       image_index = 0; 
       num_images = 1;
 
+      DInt pixel_type;
       pixel_type = a.depth() == 16 ? 2 : 1;
 
-      has_palette = a.classType() == PseudoClass ? 1 : 0;
+      // AC 2012-May-10 Palette only if type == 4 OR 5
+      // Despite Type is NOT useful without a a.read(), it is OK for Palette !
+      // This should be reliable (OK with ImageMagick AND GraphicsMagick)
+      DInt has_palette=0;
+      if (a.type() == 4 | a.type() == 5) has_palette=1;
 
       // TODO: 
-      // - JP2->JPEG2000 ?
+      // - JP2->JPEG2000 ?      
+      DString type;
       type = a.magick() == "PNM" ? "PPM" : 
              a.magick() == "DCM" ? "DICOM" :
              a.magick();
 
+      if (debug ==1) cout << "Type (via a.magick()) : " << type <<endl;
+
       static int infoIx = e->KeywordIx("INFO");
       if (e->KeywordPresent(infoIx))
       {
@@ -378,23 +412,40 @@ namespace lib {
 
 	columns=image.columns();
 	rows=image.rows();
-	SizeT c[2];
-	c[0]=columns;
-	c[1]=rows;
-	dimension dim(c,2);
-	DByteGDL *bImage=new DByteGDL(dim,BaseGDL::NOZERO);
 
-	PixelPacket* pixel;
-	IndexPacket* index;
-	pixel=image.getPixels(0,0,columns,rows);
-	index=image.getIndexes();
-        if (index == NULL) e->Throw("(FIXME!) Magick's getIndexes() returned NULL for: " +e->GetParString(0));
-	unsigned int cx, cy;
-	for (cy=0;cy<rows;++cy)
-	  for (cx=0;cx<columns;++cx)
-	    (*bImage)[cx+(rows-cy-1)*columns]= index[cx+(cy)*columns];
-	
-	return bImage;
+	if (image.matte() == 0){
+	  
+	  SizeT c[2];
+	  c[0]=columns;
+	  c[1]=rows;
+	  dimension dim(c,2);
+	  DByteGDL *bImage=new DByteGDL(dim,BaseGDL::NOZERO);
+	  
+	  PixelPacket* pixel;
+	  IndexPacket* index;
+	  pixel=image.getPixels(0,0,columns,rows);
+	  index=image.getIndexes();
+	  string txt="(FIXME!) Magick's getIndexes() returned NULL for: ";
+	  if (index == NULL) e->Throw(txt + e->GetParString(0));
+	  unsigned int cx, cy;
+	  for (cy=0;cy<rows;++cy)
+	    for (cx=0;cx<columns;++cx)
+	      // note by AC, 07Feb2012: why this transpose here ??
+	      // (*bImage)[cx+(rows-cy-1)*columns]= index[cx+(cy)*columns];
+	      (*bImage)[cx+cy*columns]= index[cx+cy*columns];
+	  return bImage;
+	} else {
+	  // we do have to manage an extra channel for transparency
+	  string map="RA";
+	  SizeT c[3];
+	  c[0] = map.length(); // see code "magick_read" below
+	  c[1] = columns;
+	  c[2] = rows;
+	  dimension dim(c,3);
+	  DByteGDL *bImage=new DByteGDL(dim,BaseGDL::NOZERO);
+	  image.write(0,0,columns,rows,map, CharPixel,&(*bImage)[0]);
+	  return bImage;
+	} 
       }
     catch (Exception &error_ )
       {
@@ -402,7 +453,6 @@ namespace lib {
       }
   }
 
-
   void magick_readcolormapRGB(EnvT* e)
   {
     try{
@@ -412,7 +462,6 @@ namespace lib {
       Image image=magick_image(e,mid);
       if(image.classType()==DirectClass)
 	e->Throw("Not an indexed image: " +e->GetParString(0));
-      
 
       if(image.classType()==PseudoClass)
 	{
@@ -469,14 +518,13 @@ namespace lib {
 		  (*B)[i]=(col.blueQuantum())*scale/Quant;
 		}
 	      if(nParam > 1) e->SetPar(1,R);
-		if(nParam > 2) e->SetPar(2,G);
-		if(nParam > 3) e->SetPar(3,B);
+	      if(nParam > 2) e->SetPar(2,G);
+	      if(nParam > 3) e->SetPar(3,B);
 	    }
-	    else
-	      {
-		e->Throw("Uknown Image type, too many colors");
-	      }
-	  
+	  else
+	    {
+	      e->Throw("Uknown Image type, too many colors");
+	    }	  
 	}
       else
 	{
@@ -522,6 +570,7 @@ namespace lib {
 		map="BGR";
 	      }
 	  }
+
 	if(image.matte()) map=map+"A";
 
 	if(e->KeywordSet(2)) //MAP
@@ -531,7 +580,7 @@ namespace lib {
 	if(e->GetKW(1) != NULL)//SUB_RECT
 	  {
 	    BaseGDL* sr=e->GetKW(1);
-	    DULongGDL * subrect=static_cast<DULongGDL*>(sr->Convert2(ULONG,BaseGDL::COPY));
+	    DULongGDL * subrect=static_cast<DULongGDL*>(sr->Convert2(GDL_ULONG,BaseGDL::COPY));
 	    if(subrect->N_Elements() != 4)
 	      e->Throw("Not enough elements in SUB_RECT, expected 4.");
 	    lx=(*subrect)[0];//guaranteed to be >0
@@ -621,13 +670,13 @@ namespace lib {
 		    {*/
 
 		DByteGDL * bImage=
-		  static_cast<DByteGDL*>(GDLimage->Convert2(BYTE,BaseGDL::COPY));
+		  static_cast<DByteGDL*>(GDLimage->Convert2(GDL_BYTE,BaseGDL::COPY));
 		image.read(columns,rows,map, CharPixel,&(*bImage)[0]);
 		/*	      }
 	    else if(image.depth() == 16)
 	      {
 		DUIntGDL * iImage=
-		  static_cast<DUIntGDL*>(GDLimage->Convert2(UINT,BaseGDL::COPY));
+		  static_cast<DUIntGDL*>(GDLimage->Convert2(GDL_UINT,BaseGDL::COPY));
 
 		image.read(columns,rows,map, ShortPixel,&(*iImage)[0]);
 	      }
@@ -662,6 +711,8 @@ namespace lib {
 	
 	DString filename;
 	e->AssureScalarPar<DStringGDL>(1,filename);    
+	WordExp( filename);
+
 	if(nParam==3)
 	  {
 	    DString imagetype;
@@ -914,49 +965,39 @@ namespace lib {
   void magick_quantize(EnvT* e)
   {
     try{
-      size_t nParam=e->NParam(2);
+      size_t nParam=e->NParam();
+
       DUInt mid;
       e->AssureScalarPar<DUIntGDL>(0,mid);    
       Image image=magick_image(e,mid);
-      DLong ncol;      
-      //set the number of colors;
-      if(!e->KeywordSet(0)) 
-	{
-	  if(nParam>1)
-	    e->AssureLongScalarPar(1,ncol);
-	  else
-	      ncol=256;
 
-	  image.quantizeColors(ncol);
-	  if(e->KeywordSet(1))//YUV
+      //set the number of colors;
+      DLong ncol=256;
+      if(nParam>1) e->AssureLongScalarPar(1,ncol);
+      image.quantizeColors(ncol);
+
+      if(e->KeywordSet("TRUECOLOR")) 
+	{
+	  image.quantizeColorSpace(RGBColorspace);
+	  image.quantizeColors((long)256*(long)256*(long)256-1);
+	  if(e->KeywordSet("DITHER")) 
+	    image.quantizeDither(true);
+	  image.quantize();
+	  image.classType(DirectClass);
+	}
+      else
+	{
+	  if(e->KeywordSet("YUV"))//YUV
 	    image.quantizeColorSpace(YUVColorspace);
-	  else if(e->KeywordSet(2))//Grayscale
+	  else if(e->KeywordSet("GRAYSCALE"))//Grayscale
 	    image.quantizeColorSpace(GRAYColorspace);
 	  else
 	    image.quantizeColorSpace(RGBColorspace);
-	  if(e->KeywordSet(3)) 
+	  if(e->KeywordSet("DITHER")) 
 	    image.quantizeDither(true);
 	  image.quantize();
 	  image.classType(PseudoClass);
 	}
-      else
-	{
-	  //truecolor
-	  if(nParam>1)
-	    e->AssureLongScalarPar(1,ncol);
-	  else
-	    ncol=256;
-
-	  image.quantizeColors(ncol);
-	  image.quantizeColorSpace(RGBColorspace);
-	  image.quantizeColors((long)256*(long)256*(long)256-1);
-	  if(e->KeywordSet(3)) 
-	    image.quantizeDither(true);
-	  image.quantize();
-	  image.classType(DirectClass);
-	  
-	}
-      
       
       magick_replace(e,mid,image);      
     }
@@ -983,7 +1024,7 @@ namespace lib {
     DUInt mid;
     e->AssureScalarPar<DUIntGDL>(0,mid);    
     BaseGDL* GDLimage=e->GetParDefined(1);
-    DByteGDL * bImage=static_cast<DByteGDL*>(GDLimage->Convert2(BYTE,BaseGDL::COPY));
+    DByteGDL * bImage=static_cast<DByteGDL*>(GDLimage->Convert2(GDL_BYTE,BaseGDL::COPY));
 
     Image image=magick_image(e,mid);
 
@@ -1026,15 +1067,15 @@ SizeT nEl = columns*rows;
     Image image=magick_image(e,mid);
 
     BaseGDL* GDLCol=e->GetParDefined(1);
-    DByteGDL * Red=static_cast<DByteGDL*>(GDLCol->Convert2(BYTE,BaseGDL::COPY));
+    DByteGDL * Red=static_cast<DByteGDL*>(GDLCol->Convert2(GDL_BYTE,BaseGDL::COPY));
     //e->Guard( Red);
     auto_ptr<BaseGDL> r_guard( Red);
     GDLCol=e->GetParDefined(2);
-    DByteGDL * Green=static_cast<DByteGDL*>(GDLCol->Convert2(BYTE,BaseGDL::COPY));
+    DByteGDL * Green=static_cast<DByteGDL*>(GDLCol->Convert2(GDL_BYTE,BaseGDL::COPY));
     //e->Guard( Green);
     auto_ptr<BaseGDL> g_guard( Green);
     GDLCol=e->GetParDefined(3);
-    DByteGDL *Blue= static_cast<DByteGDL*>(GDLCol->Convert2(BYTE,BaseGDL::COPY));
+    DByteGDL *Blue= static_cast<DByteGDL*>(GDLCol->Convert2(GDL_BYTE,BaseGDL::COPY));
     //e->Guard( Blue);
     auto_ptr<BaseGDL> b_guard( Blue);
 
