@@ -32,7 +32,21 @@
 //#define GDL_DEBUG
 #undef GDL_DEBUG
 
-#define COMPLEX2 COMPLEX
+#define COMPLEX2 GDL_COMPLEX
+
+#ifdef _MSC_VER
+#define fetestexcept(e) false
+#define feclearexcept(e)
+#define FE_DIVBYZERO
+#define FE_UNDERFLOW
+#define FE_OVERFLOW
+#define FE_INVALID
+#define round(f) floor(f+0.5)
+#define isnan _isnan
+#define isinf !_finite
+#define isfinite _finite
+#define signbit(d) (d < 0.0)? 1:0
+#endif
 
 using namespace std;
 
@@ -103,9 +117,9 @@ namespace lib {
 //     SizeT nEl = p0->N_Elements();
 //     for( SizeT i=0; i<nEl; ++i)
 //       {
-// 	if( p0->Type() == COMPLEX) {
+// 	if( p0->Type() == GDL_COMPLEX) {
 // 	  float* dptr = (float*) &(*p0C)[0];
-// 	 } else if( p0->Type() == COMPLEXDBL) {
+// 	 } else if( p0->Type() == GDL_COMPLEXDBL) {
 // 	    int a=0;
 // 	} else {
 // 	  int out = isfinite((*p0C)[ i]); 
@@ -131,7 +145,7 @@ namespace lib {
 
 //     DByteGDL* res = new DByteGDL( p0->Dim(), BaseGDL::NOZERO);
 
-//     if( p0->Type() == COMPLEX) {
+//     if( p0->Type() == GDL_COMPLEX) {
 //       DComplexGDL* p0C = static_cast<DComplexGDL*>( p0);
 //       for( SizeT i=0; i<nEl; ++i) {
 // // 	float* dptr = (float*) &(*p0C)[ i];
@@ -142,7 +156,7 @@ namespace lib {
 // 	if (isfinite(r_part) == 0 || isfinite(i_part) == 0) 
 // 	  (*res)[ i] = 0; else (*res)[ i] = 1;
 //       }
-//     } else if ( p0->Type() == COMPLEXDBL) {
+//     } else if ( p0->Type() == GDL_COMPLEXDBL) {
 //       DComplexDblGDL* p0C = static_cast<DComplexDblGDL*>( p0);
 //       for( SizeT i=0; i<nEl; ++i) {
 // //         double* dptr = (double*) &(*p0C)[ i];
@@ -153,17 +167,17 @@ namespace lib {
 // 	if (isfinite(r_part) == 0 || isfinite(i_part) == 0) 
 // 	  (*res)[ i] = 0; else (*res)[ i] = 1;
 //       }
-//     } else if( p0->Type() == DOUBLE) {
+//     } else if( p0->Type() == GDL_DOUBLE) {
 //       DDoubleGDL* p0D = static_cast<DDoubleGDL*>( p0);
 //       for( SizeT i=0; i<nEl; ++i)
 // 	if (isfinite((*p0D)[ i]) == 0) (*res)[ i] = 0; else (*res)[ i] = 1;
-//     } else if( p0->Type() == FLOAT) {
+//     } else if( p0->Type() == GDL_FLOAT) {
 //       DFloatGDL* p0F = static_cast<DFloatGDL*>( p0);
 //       for( SizeT i=0; i<nEl; ++i)
 // 	if (isfinite((*p0F)[ i]) == 0) (*res)[ i] = 0; else (*res)[ i] = 1;
 //     } else {
 //       DFloatGDL* p0F = static_cast<DFloatGDL*>
-// 	(p0->Convert2( FLOAT, BaseGDL::COPY));
+// 	(p0->Convert2( GDL_FLOAT, BaseGDL::COPY));
 //       for( SizeT i=0; i<nEl; ++i)
 // 	if (isfinite((*p0F)[ i]) == 0) (*res)[ i] = 0; else (*res)[ i] = 1;
 //     }
@@ -192,7 +206,7 @@ namespace lib {
      }
    };
 
-   // partial specialization for COMPLEX, DCOMPLEX
+   // partial specialization for GDL_COMPLEX, DCOMPLEX
    template< typename T> struct finite_helper<T, true>
    {
      inline static BaseGDL* do_it(T* src, bool kwNaN, bool kwInfinity)
@@ -220,6 +234,76 @@ namespace lib {
        do_it(static_cast<T*>(src), kwNaN, kwInfinity);
    };
 
+   template< typename T, bool> struct finite_helper_sign
+   {
+     inline static BaseGDL* do_it(T* src, bool kwNaN, bool kwInfinity, DLong kwSign)
+     {
+
+// #pragma omp parallel if (nEl >= CpuTPOOL_MIN_ELTS && (CpuTPOOL_MAX_ELTS == 0 || CpuTPOOL_MAX_ELTS <= nEl))
+
+       DByteGDL* res = new DByteGDL( src->Dim(), BaseGDL::NOZERO); // ::ZERO is not working
+       SizeT nEl = src->N_Elements();
+
+       //       for ( SizeT i=0; i<nEl; ++i) (*res)[i]=0;
+
+       if (kwInfinity) {
+	 if (kwSign > 0) {
+// #pragma omp for
+	   for ( SizeT i=0; i<nEl; ++i) {
+	     if (isinf((*src)[ i]) && (signbit((*src)[ i]) == 0)) (*res)[i]=1; else (*res)[i]=0;
+	   }
+	 } else {
+// #pragma omp for
+	   for ( SizeT i=0; i<nEl; ++i) {
+	     if (isinf((*src)[ i]) && (signbit((*src)[ i]) != 0)) (*res)[i]=1; else (*res)[i]=0;
+	   }
+	 }
+	 return res;	 
+       }
+       if (kwNaN) {
+	 if (kwSign > 0) {
+// #pragma omp for
+	   for ( SizeT i=0; i<nEl; ++i) {
+	     if (isnan((*src)[ i]) && (signbit((*src)[ i]) == 0)) (*res)[i]=1; else (*res)[i]=0;
+	   }
+	 } else {
+// #pragma omp for
+	   for ( SizeT i=0; i<nEl; ++i) {
+	     if (isnan((*src)[ i]) && (signbit((*src)[ i]) != 0)) (*res)[i]=1; else (*res)[i]=0;
+	   }
+	 }
+	 return res;
+       }
+     }
+   };
+
+   // partial specialization for GDL_COMPLEX, DCOMPLEX
+   template< typename T> struct finite_helper_sign<T, true>
+   {
+     inline static BaseGDL* do_it(T* src, bool kwNaN, bool kwInfinity, DLong kwSign)
+     {
+       DByteGDL* res = new DByteGDL( src->Dim(), BaseGDL::NOZERO);
+       SizeT nEl = src->N_Elements();
+       
+       for ( SizeT i=0; i<nEl; ++i)
+	 {
+	   (*res)[i]=0;
+	   if      ((kwInfinity && isinf((*src)[ i].real()) || kwNaN && isnan((*src)[ i].real())) && signbit((*src)[ i].real())==0 && kwSign > 0) (*res)[i]=1;
+	   else if ((kwInfinity && isinf((*src)[ i].imag()) || kwNaN && isnan((*src)[ i].imag())) && signbit((*src)[ i].imag())==0 && kwSign > 0) (*res)[i]=1;
+	   else if ((kwInfinity && isinf((*src)[ i].real()) || kwNaN && isnan((*src)[ i].real())) && signbit((*src)[ i].real())==1 && kwSign < 0) (*res)[i]=1;
+	   else if ((kwInfinity && isinf((*src)[ i].imag()) || kwNaN && isnan((*src)[ i].imag())) && signbit((*src)[ i].imag())==1 && kwSign < 0) (*res)[i]=1;	 
+	 }
+       return res;
+     }
+   };
+
+   template< typename T, bool IS_COMPLEX>
+   inline BaseGDL* finite_template( BaseGDL* src, bool kwNaN, bool kwInfinity, DLong kwSign)
+   {
+     return finite_helper_sign<T, IS_COMPLEX>::
+       do_it(static_cast<T*>(src), kwNaN, kwInfinity, kwSign);
+   };
+
    BaseGDL* finite_fun( EnvT* e)
    {
      e->NParam( 1);
@@ -233,52 +317,107 @@ namespace lib {
      static int infinityIx = e->KeywordIx( "INFINITY");
      bool kwInfinity = e->KeywordSet( infinityIx);
 
+     static int signIx = e->KeywordIx( "SIGN");
+     DLong kwSign = 0;
+     e->AssureLongScalarKWIfPresent( signIx, kwSign); 
+
      if( kwNaN && kwInfinity)
        e->Throw("Conflicting keywords.");
-
-     switch (p0->Type()) 
+     
+     if(kwSign==0 || (kwInfinity==0 && kwNaN==0))
        {
-       case FLOAT: 
-	 {
-	   return finite_template<DFloatGDL, false>(p0, kwNaN, kwInfinity);
-	 }
-       case DOUBLE:
-	 {
-	   return finite_template<DDoubleGDL, false>(p0, kwNaN, kwInfinity);
-	 }
-       case COMPLEX:
-	 {
-	   return finite_template<DComplexGDL, true>(p0, kwNaN, kwInfinity);
-	 }
-       case COMPLEXDBL:
-	 {
-	   return finite_template<DComplexDblGDL, true>(p0, kwNaN, kwInfinity);
-	 }
-       case STRING:
-	 {
-	   DFloatGDL* p0F = 
-	     static_cast<DFloatGDL*>(p0->Convert2(FLOAT,BaseGDL::COPY));
-	   guard.reset( p0F);
-	   return finite_template<DFloatGDL, false>(p0F, kwNaN, kwInfinity);
-	 }
-       case STRUCT:
-       case PTR:
-       case OBJECT:
-	 {
-	   e->Throw( p0->TypeStr() + " not allowed in this context: " +
-		     e->GetParString( 0));
-	 }
-       default: // integer types
-	 {
-	   if( kwNaN || kwInfinity)
-	     return new DByteGDL( p0->Dim()); // zero
-
+	 switch (p0->Type()) 
+	   {
+	   case GDL_FLOAT: 
+	     {
+	       return finite_template<DFloatGDL, false>(p0, kwNaN, kwInfinity);
+	     }
+	   case GDL_DOUBLE:
+	     {
+	       return finite_template<DDoubleGDL, false>(p0, kwNaN, kwInfinity);
+	     }
+	   case GDL_COMPLEX:
+	     {
+	       return finite_template<DComplexGDL, true>(p0, kwNaN, kwInfinity);
+	     }
+	   case GDL_COMPLEXDBL:
+	     {
+	       return finite_template<DComplexDblGDL, true>(p0, kwNaN, kwInfinity);
+	     }
+	   case GDL_STRING:
+	     {
+	       DFloatGDL* p0F = 
+		 static_cast<DFloatGDL*>(p0->Convert2(GDL_FLOAT,BaseGDL::COPY));
+	       guard.reset( p0F);
+	       return finite_template<DFloatGDL, false>(p0F, kwNaN, kwInfinity);
+	     }
+	   case GDL_STRUCT:
+	   case GDL_PTR:
+	   case GDL_OBJ:
+	     {
+	       e->Throw( p0->TypeStr() + " not allowed in this context: " +
+			 e->GetParString( 0));
+	     }
+	   default: // integer types
+	     {
+	       if( kwNaN || kwInfinity)
+		 return new DByteGDL( p0->Dim()); // zero
+	       
 	   DByteGDL* res = new DByteGDL( p0->Dim(), BaseGDL::NOZERO); 
 	   SizeT nEl = p0->N_Elements();
 	   for (SizeT i=0; i<nEl; i++)
 	     (*res)[i] = 1;
 	   return res;
-	 }
+	     }
+	   }
+       }
+     // Sign
+     else
+       {
+	 switch (p0->Type()) 
+	   {
+	   case GDL_FLOAT: 
+	     {
+	       return finite_template<DFloatGDL, false>(p0, kwNaN, kwInfinity, kwSign);
+	     }
+	   case GDL_DOUBLE:
+	     {
+	       return finite_template<DDoubleGDL, false>(p0, kwNaN, kwInfinity, kwSign);
+	     }
+	   case GDL_COMPLEX:
+	     {
+	       return finite_template<DComplexGDL, true>(p0, kwNaN, kwInfinity, kwSign);
+	     }
+	   case GDL_COMPLEXDBL:
+	     {
+	       return finite_template<DComplexDblGDL, true>(p0, kwNaN, kwInfinity, kwSign);
+	     }
+	   case GDL_STRING:
+	     {
+	       DFloatGDL* p0F = 
+		 static_cast<DFloatGDL*>(p0->Convert2(GDL_FLOAT,BaseGDL::COPY));
+	       guard.reset( p0F);
+	       return finite_template<DFloatGDL, false>(p0F, kwNaN, kwInfinity, kwSign);
+	     }
+	   case GDL_STRUCT:
+	   case GDL_PTR:
+	   case GDL_OBJ:
+	     {
+	       e->Throw( p0->TypeStr() + " not allowed in this context: " +
+			 e->GetParString( 0));
+	     }
+	   default: // integer types
+	     {
+	       if( kwNaN || kwInfinity)
+		 return new DByteGDL( p0->Dim()); // zero
+	       
+	       DByteGDL* res = new DByteGDL( p0->Dim(), BaseGDL::NOZERO);
+	       SizeT nEl = p0->N_Elements();
+	       for (SizeT i=0; i<nEl; i++)
+	       (*res)[i] = 0;
+	       return res;
+	     }
+	   }
        }
    }
 
@@ -295,12 +434,16 @@ namespace lib {
     DLong flag_print=0, flag_noclear=0, flag_clear=1;
 
     DLong value=0;
-    static DLong cumul_value;
+    static DLong cumul_value; // auto init to 0
     DLong mask=255;
 
     //
-    flag_print=e->KeywordSet( "PRINT");
-    flag_noclear=e->KeywordSet( "NOCLEAR");
+    static int printKwIx = e->KeywordIx("PRINT");
+    static int noClearKwIx = e->KeywordIx("NOCLEAR");
+    static int maskKwIx = e->KeywordIx("MASK");
+
+    flag_print = e->KeywordSet( printKwIx);
+    flag_noclear = e->KeywordSet( noClearKwIx);
     //
     // if Params are provides (first: print, second: noclear)
     // they do overwrite the same provided by Keyword
@@ -314,15 +457,15 @@ namespace lib {
     
     if (flag_noclear > 0) flag_clear=0;
     
-    int debug=0;
+    const int debug=0;
     if (debug) {
       cout << "Flag Print  : " << flag_print << endl;
       cout << "Flag NoClear: " << flag_noclear << endl;
       cout << "Flag Clear: " << flag_clear << endl;
     }
 
-    if( e->KeywordSet( "MASK"))
-      e->AssureLongScalarKWIfPresent( "MASK", mask);	
+    if( e->KeywordSet( maskKwIx))
+      e->AssureLongScalarKWIfPresent( maskKwIx, mask);	
 
     if (mask & 16) {
       if (fetestexcept(FE_DIVBYZERO)) {
@@ -354,10 +497,11 @@ namespace lib {
       }
     }
 
-    if (mask & 128 && value == 0) {
+    
+    if (mask & 128) {// && value == 0) {
       if (fetestexcept(FE_INVALID)) {
 	value = value | 128;
-	if ( flag_print)
+	if ( flag_print) // avoid double message
 	  cout << 
 	    "% Program caused arithmetic error: Floating illegal operand" << endl;
 	if ( flag_clear) feclearexcept(FE_INVALID); 
@@ -371,7 +515,7 @@ namespace lib {
     
     if (flag_noclear) {
        if (debug) cout << "noclear == 1" << endl;
-      cumul_value=cumul_value+value;
+      cumul_value |= value;
       value=cumul_value;
     } else {
        if (debug) cout << "noclear == 0" << endl;
@@ -382,7 +526,7 @@ namespace lib {
       cout << "cumul_value " <<cumul_value<< endl;
     }
 
-    return new DLongGDL( value );
+    return new DLongGDL( value);
   }
 
 
@@ -398,7 +542,7 @@ namespace lib {
       e->Throw( "Array must have 2 dimensions: "+e->GetParString(0));
 
     DFloatGDL* p0F = static_cast<DFloatGDL*>
-      (p0->Convert2( FLOAT, BaseGDL::COPY));
+      (p0->Convert2( GDL_FLOAT, BaseGDL::COPY));
 
     DFloat fpi=4*atan(1.0);
 
@@ -658,7 +802,7 @@ namespace lib {
 	e->Throw( "Array must have 2 elements: "
 		  +e->GetParString(4));
       GS = static_cast<DDoubleGDL*>
-	(p4->Convert2( DOUBLE, BaseGDL::COPY));
+	(p4->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 
       if( nParam == 6) {
 	BaseGDL* p5 = e->GetParDefined( 5);
@@ -669,7 +813,7 @@ namespace lib {
 	  e->Throw( "Array must have 4 elements: "
 		    +e->GetParString(5));
 	limits = static_cast<DDoubleGDL*>
-	  (p5->Convert2( DOUBLE, BaseGDL::COPY));
+	  (p5->Convert2( GDL_DOUBLE, BaseGDL::COPY));
       }
     }
 
@@ -717,13 +861,13 @@ namespace lib {
     }
 
     DDoubleGDL* x_tri = static_cast<DDoubleGDL*>
-      (p0->Convert2( DOUBLE, BaseGDL::COPY));
+      (p0->Convert2( GDL_DOUBLE, BaseGDL::COPY));
     DDoubleGDL* y_tri = static_cast<DDoubleGDL*>
-      (p1->Convert2( DOUBLE, BaseGDL::COPY));
+      (p1->Convert2( GDL_DOUBLE, BaseGDL::COPY));
     DDoubleGDL* z = static_cast<DDoubleGDL*>
-      (p2->Convert2( DOUBLE, BaseGDL::COPY));
+      (p2->Convert2( GDL_DOUBLE, BaseGDL::COPY));
     DLongGDL* triangles = static_cast<DLongGDL*>
-      (p3->Convert2( LONG, BaseGDL::COPY));
+      (p3->Convert2( GDL_LONG, BaseGDL::COPY));
 
 
     //    bool sphere=false;
@@ -751,7 +895,7 @@ namespace lib {
 		   "must have 4 elements.");
 	auto_ptr<DDoubleGDL> guard;
 	DDoubleGDL* mapD = static_cast<DDoubleGDL*>
-	  ( Map->Convert2( DOUBLE, BaseGDL::COPY));
+	  ( Map->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 	guard.reset( mapD);
 	xvsx[0] = (*mapD)[0];
 	xvsx[1] = (*mapD)[1];
@@ -967,19 +1111,25 @@ namespace lib {
 
   template< typename T1, typename T2>
   BaseGDL* poly_2d_shift_template( BaseGDL* p0, DLong nCol, DLong nRow, 
-				   int shift_y, int shift_x)
+				   int shift_y, int shift_x, DDouble missing)
   {
     dimension dim(nCol,nRow);  
 //     dim.Set(0, nCol);
 //     dim.Set(1, nRow);
+//    cout<<"shifting"<<endl;
     T1* res = new T1( dim, BaseGDL::NOZERO);
-
+    T2 missed=missing;
+    
     int lx = (int) p0->Dim(0);
     int ly = (int) p0->Dim(1);
     int lx_out = (int) nCol;
     int ly_out = (int) nRow;
 
     char *p_out = (char *) res->DataAddr();
+    T2 *resAddr = (T2 *) res->DataAddr();
+    for ( SizeT i=0; i<nCol*nRow; i++) {
+      resAddr[i] = missed;
+    }
     char *p_in  = (char *) p0->DataAddr();
 
     SizeT sz = sizeof(T2);
@@ -1067,10 +1217,10 @@ namespace lib {
     DLong nc = (nDegree + 1) * (nDegree + 1);
 
     DDoubleGDL* P = static_cast<DDoubleGDL*>
-      (p1->Convert2( DOUBLE, BaseGDL::COPY));
+      (p1->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 
     DDoubleGDL* Q = static_cast<DDoubleGDL*>
-      (p2->Convert2( DOUBLE, BaseGDL::COPY));
+      (p2->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 
     char kernel_name[32];
     kernel_name[0] = 0;
@@ -1092,42 +1242,42 @@ namespace lib {
 	if (((int) (*P)[0] == 0) && ((int) (*Q)[0] == 0)) {
 	  return p0->Dup();
 	} else {
-	  if (p0->Type() == BYTE) {
+	  if (p0->Type() == GDL_BYTE) {
 	    return poly_2d_shift_template< DByteGDL, DByte>( p0, nCol, nRow,  
 							     (int) (*Q)[0], 
-							     (int) (*P)[0]);
-	  } else if (p0->Type() == INT) {
+							     (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_INT) {
 	    return poly_2d_shift_template< DIntGDL, DInt>( p0, nCol, nRow,  
 							    (int) (*Q)[0], 
-							    (int) (*P)[0]);
-	  } else if (p0->Type() == UINT) {
+							    (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_UINT) {
 	    return poly_2d_shift_template< DUIntGDL, DUInt>( p0, nCol, nRow,  
 							     (int) (*Q)[0], 
-							     (int) (*P)[0]);
-	  } else if (p0->Type() == LONG) {
+							     (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_LONG) {
 	    return poly_2d_shift_template< DLongGDL, DLong>( p0, nCol, nRow,  
 							     (int) (*Q)[0], 
-							     (int) (*P)[0]);
-	  } else if (p0->Type() == ULONG) {
+							     (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_ULONG) {
 	    return poly_2d_shift_template< DULongGDL, DULong>( p0, nCol, nRow,  
 							       (int) (*Q)[0], 
-							       (int) (*P)[0]);
-	  } else if (p0->Type() == LONG64) {
+							       (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_LONG64) {
 	    return poly_2d_shift_template< DLong64GDL, DLong64>( p0, nCol, nRow,  
 								 (int) (*Q)[0], 
-								 (int) (*P)[0]);
-	  } else if (p0->Type() == ULONG64) {
+								 (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_ULONG64) {
 	    return poly_2d_shift_template< DULong64GDL, DULong64>( p0, nCol, nRow,  
 								   (int) (*Q)[0], 
-								   (int) (*P)[0]);
-	  } else if (p0->Type() == FLOAT) {
+								   (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_FLOAT) {
 	    return poly_2d_shift_template< DFloatGDL, DFloat>( p0, nCol, nRow,  
 							       (int) (*Q)[0], 
-							       (int) (*P)[0]);
-	  } else if (p0->Type() == DOUBLE) {
+							       (int) (*P)[0],missing);
+	  } else if (p0->Type() == GDL_DOUBLE) {
 	    return poly_2d_shift_template< DDoubleGDL, DDouble>( p0, nCol, nRow,  
 								 (int) (*Q)[0], 
-								 (int) (*P)[0]);
+								 (int) (*P)[0],missing);
 	  }
 	}
       } else {
@@ -1141,7 +1291,7 @@ namespace lib {
 	warped = image_warp(p0->Dim(1), p0->Dim(0), nRow, nCol, p0->Type(), 
 			    p0->DataAddr(), kernel_name,
 			    lineartrans, poly_v, poly_u,
-			    interp, cubic, LINEAR);
+			    interp, cubic, LINEAR, missing);
       }
     } else {
       // Polynomial
@@ -1172,7 +1322,7 @@ namespace lib {
       warped = image_warp(p0->Dim(1), p0->Dim(0), nRow, nCol, p0->Type(), 
 			  p0->DataAddr(), kernel_name, 
 			  lineartrans, poly_v, poly_u, 
-			  interp, cubic, GENERIC);
+			  interp, cubic, GENERIC, missing);
 
       if (poly_u->px != NULL) free(poly_u->px);
       if (poly_u->py != NULL) free(poly_u->py);
@@ -1187,27 +1337,27 @@ namespace lib {
       free(poly_v);
     }
 
-    if (p0->Type() == BYTE) {
+    if (p0->Type() == GDL_BYTE) {
       for ( SizeT i=0; i<nCol*nRow; ++i) {
 	if (warped->data[i] < 0)   warped->data[i] = 0;
 	if (warped->data[i] > 255) warped->data[i] = 255;
       }
       return poly_2d_fun_template< DByteGDL, DByte>( nCol, nRow, warped);
-    } else if (p0->Type() == INT) {
+    } else if (p0->Type() == GDL_INT) {
       return poly_2d_fun_template< DIntGDL, DInt>( nCol, nRow, warped);
-    } else if (p0->Type() == UINT) {
+    } else if (p0->Type() == GDL_UINT) {
       return poly_2d_fun_template< DUIntGDL, DUInt>( nCol, nRow, warped);
-    } else if (p0->Type() == LONG) {
+    } else if (p0->Type() == GDL_LONG) {
       return poly_2d_fun_template< DLongGDL, DLong>( nCol, nRow, warped);
-    } else if (p0->Type() == ULONG) {
+    } else if (p0->Type() == GDL_ULONG) {
       return poly_2d_fun_template< DULongGDL, DULong>( nCol, nRow, warped);
-    } else if (p0->Type() == LONG64) {
+    } else if (p0->Type() == GDL_LONG64) {
       return poly_2d_fun_template< DLong64GDL, DLong64>( nCol, nRow, warped);
-    } else if (p0->Type() == ULONG64) {
+    } else if (p0->Type() == GDL_ULONG64) {
       return poly_2d_fun_template< DULong64GDL, DULong64>( nCol, nRow, warped);
-    } else if (p0->Type() == FLOAT) {
+    } else if (p0->Type() == GDL_FLOAT) {
       return poly_2d_fun_template< DFloatGDL, DFloat>( nCol, nRow, warped);
-    } else if (p0->Type() == DOUBLE) {
+    } else if (p0->Type() == GDL_DOUBLE) {
       return poly_2d_fun_template< DDoubleGDL, DDouble>( nCol, nRow, warped);
     }
   }
@@ -1427,7 +1577,7 @@ double * generate_interpolation_kernel(char * kernel_type, DDouble cubic)
  */
 /*--------------------------------------------------------------------------*/
 
-image_t * image_warp(
+image_t * image_warp (
 		     SizeT  lx,		     
 		     SizeT  ly,		     
 		     SizeT  lx_out,		     
@@ -1440,7 +1590,8 @@ image_t * image_warp(
 		     poly2d		*	poly_v,
 		     DLong interp,
 		     DDouble cubic,
-		     DLong warpType)
+		     DLong warpType,
+                     DDouble initvalue)
 {
     image_t    *	image_out ;
     int         	i, j, k ;
@@ -1484,7 +1635,7 @@ image_t * image_warp(
       }
     }
 
-    image_out = image_new(lx_out, ly_out) ;
+    image_out = image_new(lx_out, ly_out, initvalue) ;
 
     /* Pre compute leaps for 16 closest neighbors positions */
 
@@ -1531,7 +1682,10 @@ image_t * image_warp(
 	      (px > (lx-1)) ||
 	      (py < 1) ||
 	      (py > (ly-1)))
-	    image_out->data[i+j*lx_out] = (pixelvalue)0.0 ;
+          {
+          //already initialised to 'missing' value. No need to put zero here.
+          //	    image_out->data[i+j*lx_out] = (pixelvalue)0.0 ;
+          }
 	  else {
 	    /* Now feed the positions for the closest 16 neighbors  */
 	    pos = px + py * lx ;
@@ -1541,47 +1695,47 @@ image_t * image_warp(
 
 	      int row = (pos+leaps[k]) / lx;
 	      int col = (pos+leaps[k]) - row*lx;
-	      if (type == BYTE) {
+	      if (type == GDL_BYTE) {
 		memcpy(&data_b, &ptr[sizeof(char)*(col*ly+row)], 
 		       sizeof(char));
 		neighbors[k] = (double) data_b;
 	      } 
-	      if (type == INT) {
+	      if (type == GDL_INT) {
 		memcpy(&data_i, &ptr[sizeof(DInt)*(col*ly+row)], 
 		       sizeof(DInt));
 		neighbors[k] = (double) data_i;
 	      } 
-	      if (type == UINT) {
+	      if (type == GDL_UINT) {
 		memcpy(&data_ui, &ptr[sizeof(DUInt)*(col*ly+row)], 
 		       sizeof(DUInt));
 		neighbors[k] = (double) data_ui;
 	      } 
-	      if (type == LONG) {
+	      if (type == GDL_LONG) {
 		memcpy(&data_l, &ptr[sizeof(DLong)*(col*ly+row)], 
 		       sizeof(DLong));
 		neighbors[k] = (double) data_l;
 	      } 
-	      if (type == ULONG) {
+	      if (type == GDL_ULONG) {
 		memcpy(&data_ul, &ptr[sizeof(DULong)*(col*ly+row)], 
 		       sizeof(DULong));
 		neighbors[k] = (double) data_ul;
 	      } 
-	      if (type == LONG64) {
+	      if (type == GDL_LONG64) {
 		memcpy(&data_l64, &ptr[sizeof(DLong64)*(col*ly+row)], 
 		       sizeof(DLong64));
 		neighbors[k] = (double) data_l64;
 	      } 
-	      if (type == ULONG64) {
+	      if (type == GDL_ULONG64) {
 		memcpy(&data_ul64, &ptr[sizeof(DULong64)*(col*ly+row)], 
 		       sizeof(DULong64));
 		neighbors[k] = (double) data_ul64;
 	      } 
-	      if (type == FLOAT) {
+	      if (type == GDL_FLOAT) {
 		memcpy(&data_f, &ptr[sizeof(float)*(col*ly+row)], 
 		       sizeof(float));
 		neighbors[k] = (double) data_f;
 	      } 
-	      if (type == DOUBLE) {
+	      if (type == GDL_DOUBLE) {
 		memcpy(&data_d, &ptr[sizeof(double)*(col*ly+row)], 
 		       sizeof(double));
 		neighbors[k] = data_d;
@@ -1700,7 +1854,8 @@ image_t * image_warp(
 /*--------------------------------------------------------------------------*/
 image_t * image_new(
 		int 	size_x, 
-		int 	size_y)
+		int 	size_y,
+                DDouble  initvalue)
 {
     image_t    *	image_new ;
 
@@ -1717,6 +1872,9 @@ image_t * image_new(
     image_new->lx = size_x ;
     image_new->ly = size_y ;
     image_new->data = (pixelvalue *) calloc(size_x * size_y, sizeof(pixelvalue));
+ 
+    for (SizeT i=0; i < size_x * size_y; i++) (image_new->data)[i]=initvalue;
+    
     return image_new ;
 }
 
@@ -1780,7 +1938,7 @@ void image_del(image_t * d)
     DString tag = time_str;
 
     SizeT nParam = e->NParam(5);
-    // Result = RK4( Y, Dydx, X, H, Derivs [, /DOUBLE] )
+    // Result = RK4( Y, Dydx, X, H, Derivs [, /GDL_DOUBLE] )
 
     BaseGDL* par = e->GetParDefined( 0);
 
@@ -1789,7 +1947,7 @@ void image_del(image_t * d)
     //    cout << "nEy: " << nEy << endl;
 
     bool doubleFlag = false;
-    if ( par->Type() == DOUBLE) doubleFlag = true;
+    if ( par->Type() == GDL_DOUBLE) doubleFlag = true;
     if ( e->KeywordSet( "DOUBLE")) doubleFlag = true;
 
     DString retTypeString;
@@ -1921,7 +2079,8 @@ void image_del(image_t * d)
     // Execute command string
     EnvBaseT* caller;
     caller = e->Caller();
-    e->Interpreter()->CallStack().pop_back();
+// ms: commented out to comply with new stack handling
+//     e->Interpreter()->CallStack().pop_back();
 
     executeString( caller, &istr);
 
@@ -1973,26 +2132,3 @@ void image_del(image_t * d)
   }
 
 } // namespace
-
-/*
-
-	gsl_matrix *mat = gsl_matrix_alloc(4,4);
-	gsl_matrix *inverse = gsl_matrix_calloc(4, 4);
-	gsl_permutation *perm = gsl_permutation_alloc(4);
-
-	memcpy(mat->data, &(*p0D)[0], nEl*szdbl);
-
-	gsl_linalg_LU_decomp (mat, perm, &s);
-	det = gsl_linalg_LU_lndet(mat);
-	if (gsl_isinf(det) == 0) {
-	  gsl_linalg_LU_invert (mat, perm, inverse);
-	}
-	else singular = 1;
-
-	memcpy(&(*res)[0], inverse->data, nEl*szdbl);
-
-	gsl_permutation_free(perm);
-	gsl_matrix_free(mat);
-	gsl_matrix_free(inverse);
-
-*/
