@@ -17,6 +17,7 @@
 
 #include "includefirst.hpp"
 
+#include <limits>
 #include <ios>
 
 #include "str.hpp"
@@ -26,6 +27,8 @@
 
 #include "objects.hpp"
 #include "graphics.hpp"
+#include "preferences.hpp"
+#include "overload.hpp"
 
 //#include "dinterpreter.hpp"
 
@@ -65,6 +68,7 @@ int           debugMode;
 DLong CpuTPOOL_NTHREADS;
 DLong CpuTPOOL_MIN_ELTS;
 DLong CpuTPOOL_MAX_ELTS;
+const DLong CpuTPOOL_MAX_ELTS_max = numeric_limits<DLong>::max();
 
 // instantiate own AST factory
 //_DNodeFactory DNodeFactory;
@@ -77,8 +81,9 @@ void ResetObjects()
   Purge(sysVarList);
   Purge(funList);
   Purge(proList);
+  Purge(structList); // now deletes member subroutines (and they in turn common block references
+  // hence delete common blocks after structList
   Purge(commonList);
-  Purge(structList);
   // no purging of library
 
 #ifdef USE_PYTHON
@@ -93,11 +98,62 @@ void InitStructs()
   SpDLong   aLong;
   SpDString aString;
   SpDByte   aByte;
+  SpDULong  aULong;
   SpDLong64 aLong64;
   SpDFloat  aFloat;
   SpDDouble aDouble;
   SpDLong   aLongArr8( dimension(8));
+  SpDPtr    aPtrRef;
+  SpDObj    aObjRef;
 
+  // OBJECTS 
+
+  DStructDesc* gdl_object = new DStructDesc( GDL_OBJECT_NAME);
+  gdl_object->AddTag("GDL_OBJ_TOP", &aLong64);
+  gdl_object->AddTag("__OBJ__", &aObjRef);
+  gdl_object->AddTag("GDL_OBJ_BOTTOM", &aLong64);
+  // special for GDL_OBJECT ony
+  gdl_object->InitOperatorList();
+  // insert into structList
+  structList.push_back(gdl_object);
+  
+//   DStructDesc* gdlList = new DStructDesc( "LIST");
+//   gdlList->AddTag("GDL_CONTAINER_TOP", &aLong64);
+//   gdlList->AddTag("GDLCONTAINERVERSION", &aInt);
+//   gdlList->AddTag("PHEAD", &aPtrRef);
+//   gdlList->AddTag("PTAIL", &aPtrRef);
+//   gdlList->AddTag("NLIST", &aLong);
+//   gdlList->AddTag("GDL_CONTAINER_BOTTOM", &aLong64);
+//   // insert into structList
+//   structList.push_back(gdlList);
+// 
+//   DStructDesc* gdlContainerNode = new DStructDesc( "GDL_CONTAINER_NODE");
+//   gdlContainerNode->AddTag("PNEXT", &aPtrRef);
+//   gdlContainerNode->AddTag("OOBJ", &aObjRef);
+//   gdlContainerNode->AddTag("FLAGS", &aLong);
+//   // insert into structList
+//   structList.push_back(gdlContainerNode);
+// 
+//   DStructDesc* gdlHash = new DStructDesc( "HASH");
+//   gdlHash->AddTag("TABLE_BITS", &aULong);
+//   gdlHash->AddTag("TABLE_SIZE", &aULong);
+//   gdlHash->AddTag("TABLE_COUNT", &aULong);
+//   gdlHash->AddTag("TABLE_REMOVE", &aULong);
+//   gdlHash->AddTag("TABLE_FOREACH", &aULong);
+//   gdlHash->AddTag("TABLE_DATA", &aPtrRef);
+//   // insert into structList
+//   structList.push_back(gdlHash);
+// 
+//   DStructDesc* gdlHashTE = new DStructDesc( "GDL_HASHTABLEENTRY");
+//   gdlHashTE->AddTag("PKEY", &aPtrRef);
+//   gdlHashTE->AddTag("PVALUE", &aPtrRef);
+//   // insert into structList
+//   structList.push_back(gdlHashTE);
+//   
+//   // OBJECTS END
+
+  
+  
   DStructDesc* gdl_size = new DStructDesc( "IDL_SIZE");
   gdl_size->AddTag("TYPE_NAME", &aString);
   gdl_size->AddTag("STRUCTURE_NAME", &aString);
@@ -110,6 +166,7 @@ void InitStructs()
   // insert into structList
   structList.push_back(gdl_size);
 
+
   for (int big = 1; big >= 0; --big) 
   {
     DStructDesc* fstat = new DStructDesc( big ? "FSTAT64" : "FSTAT");
@@ -235,6 +292,23 @@ void InitStructs()
   // insert into structList
   structList.push_back( widgdlist);
 
+  DStructDesc* widglist = new DStructDesc( "WIDGET_LIST");
+  widglist->AddTag("ID", &aLong);
+  widglist->AddTag("TOP", &aLong);
+  widglist->AddTag("HANDLER", &aLong);
+  widglist->AddTag("SELECT", &aLong);
+  // insert into structList
+  structList.push_back( widglist);
+
+  DStructDesc* widgbgroup =  new DStructDesc( "WIDGET_BGROUP");
+  widgbgroup->AddTag("ID", &aLong);
+  widgbgroup->AddTag("TOP", &aLong);
+  widgbgroup->AddTag("HANDLER", &aLong);
+  widgbgroup->AddTag("SELECT", &aLong);
+  widgbgroup->AddTag("VALUE", &aLong);
+  // insert into structList
+  structList.push_back(widgbgroup);
+
   DStructDesc* widgtxt = new DStructDesc( "WIDGET_TEXT");
   widgtxt->AddTag("ID", &aLong);
   widgtxt->AddTag("TOP", &aLong);
@@ -253,7 +327,6 @@ void InitStructs()
 
 void InitObjects()
 {
-  
 
   sigControlC = false;
   debugMode   = 0;
@@ -265,11 +338,16 @@ void InitObjects()
 
   // initialize struct descriptors which are not system variables
   InitStructs();
-
+  // add internal memeber subroutines
+  SetupOverloadSubroutines();
+  
   // graphic devices must be initialized after system variables
   // !D must already exist
   Graphics::Init();
 
+  // preferences
+  //  Preferences::Init();
+
 #ifdef HAVE_LIBWXWIDGETS
   // initialize widget system
   GDLWidget::Init();
@@ -280,8 +358,11 @@ void InitObjects()
 DLong GetLUN()
 {
   for( DLong lun=maxUserLun+1; lun <= fileUnits.size(); lun++)
-    if( !fileUnits[ lun-1].InUse())
+    if( !fileUnits[ lun-1].InUse() && !fileUnits[ lun-1].GetGetLunLock())
+    {
+      fileUnits[ lun-1].SetGetLunLock( true);
       return lun;
+    }
   
   return 0;
 }
