@@ -19,6 +19,10 @@
 #include "plotting.hpp"
 #include "math_utl.hpp"
 
+#ifdef _MSC_VER
+#define isinf !_finite
+#endif
+
 namespace lib {
 
   using namespace std;
@@ -92,7 +96,7 @@ namespace lib {
 	BaseGDL* p0 = e->GetNumericArrayParDefined( 0)->Transpose( NULL);
 
 	zVal = static_cast<DDoubleGDL*>
-	  (p0->Convert2( DOUBLE, BaseGDL::COPY));
+	  (p0->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 	p0_guard.reset( p0); // delete upon exit
 
 	xEl = zVal->Dim(1);
@@ -115,7 +119,7 @@ namespace lib {
       {
 	BaseGDL* p0 = e->GetNumericArrayParDefined( 0)->Transpose( NULL);
 	zVal = static_cast<DDoubleGDL*>
-	  (p0->Convert2( DOUBLE, BaseGDL::COPY));
+	  (p0->Convert2( GDL_DOUBLE, BaseGDL::COPY));
 	p0_guard.reset( p0); // delete upon exit
 
 	if(zVal->Dim(0) == 1)
@@ -226,7 +230,7 @@ namespace lib {
 		    " must have from 1 to 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* xMarginFl = static_cast<DFloatGDL*>
-	  ( xMargin->Convert2( FLOAT, BaseGDL::COPY));
+	  ( xMargin->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( xMarginFl);
 	xMarginL = (*xMarginFl)[0];
 	if( xMarginFl->N_Elements() > 1)
@@ -239,7 +243,7 @@ namespace lib {
 		    " must have from 1 to 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* yMarginFl = static_cast<DFloatGDL*>
-	  ( yMargin->Convert2( FLOAT, BaseGDL::COPY));
+	  ( yMargin->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( yMarginFl);
 	yMarginB = (*yMarginFl)[0];
 	if( yMarginFl->N_Elements() > 1)
@@ -252,7 +256,7 @@ namespace lib {
 		    " must have from 1 to 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* zMarginFl = static_cast<DFloatGDL*>
-	  ( zMargin->Convert2( FLOAT, BaseGDL::COPY));
+	  ( zMargin->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( zMarginFl);
 	zMarginB = (*zMarginFl)[0];
 	if( zMarginFl->N_Elements() > 1)
@@ -295,7 +299,7 @@ namespace lib {
 		   " must have 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* xRangeF = static_cast<DFloatGDL*>
-	  ( xRange->Convert2( FLOAT, BaseGDL::COPY));
+	  ( xRange->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( xRangeF);
 	xStart = (*xRangeF)[0];
 	xEnd = (*xRangeF)[1];
@@ -308,7 +312,7 @@ namespace lib {
 		   " must have 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* yRangeF = static_cast<DFloatGDL*>
-	  ( yRange->Convert2( FLOAT, BaseGDL::COPY));
+	  ( yRange->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( yRangeF);
 	yStart = (*yRangeF)[0];
 	yEnd = (*yRangeF)[1];
@@ -320,7 +324,7 @@ namespace lib {
 		   " must have 2 elements.");
 	auto_ptr<DFloatGDL> guard;
 	DFloatGDL* zRangeF = static_cast<DFloatGDL*>
-	  ( zRange->Convert2( FLOAT, BaseGDL::COPY));
+	  ( zRange->Convert2( GDL_FLOAT, BaseGDL::COPY));
 	guard.reset( zRangeF);
 	zStart = (*zRangeF)[0];
 	zEnd = (*zRangeF)[1];
@@ -343,6 +347,10 @@ namespace lib {
     //cout << "MIN_VALUE" << minVal << endl;
     //cout << "MAX_VALUE" << maxVal << endl;
 
+    //ISOTROPIC
+    DLong iso=0;
+    e->AssureLongScalarKWIfPresent( "ISOTROPIC", iso);
+
     DLong xTicks=0, yTicks=0, zTicks=0;
     e->AssureLongScalarKWIfPresent( "XTICKS", xTicks);
     e->AssureLongScalarKWIfPresent( "YTICKS", yTicks);
@@ -493,7 +501,7 @@ namespace lib {
       bool okVPWC = SetVP_WC( e, actStream, overplot?NULL:pos, clippingD,
                             xLog, yLog,
                             xMarginL, xMarginR, yMarginB, yMarginF,
-                            xStart, xEnd, yStart, yEnd);
+                            xStart, xEnd, yStart, yEnd, iso);
       if( !okVPWC) return;
     } else {
       // not using SetVP_WC as it seem to always select full window for plotting (FIXME)
@@ -849,15 +857,15 @@ clevel[nlevel-1]=zEnd; //make this explicit
 
     private: virtual void post_call(EnvT*, GDLGStream* actStream) // {{{
     {
-      UpdateSWPlotStructs(actStream, xStart, xEnd, yStart, yEnd);
+      UpdateSWPlotStructs(actStream, xStart, xEnd, yStart, yEnd, xLog, yLog);
 
       actStream->lsty(1);//reset linestyle
 
       if (!overplot)
       {
         // set ![XY].CRANGE
-        set_axis_crange("X", xStart, xEnd);
-        set_axis_crange("Y", yStart, yEnd);
+        set_axis_crange("X", xStart, xEnd, xLog);
+        set_axis_crange("Y", yStart, yEnd, yLog);
 
         //set ![x|y].type
         set_axis_type("X",xLog);
