@@ -83,7 +83,7 @@ namespace lib {
 	    for( SizeT i=start,ii=0; i<ctSize; ++i,++ii)
 	      actCT->Get( i, (*rgb)[ ii], (*rgb)[ ii+nCol], (*rgb)[ ii+2*nCol]);
 
-	    delete p0;
+	    GDLDelete(p0);
 	    p0 = rgb;
 	  }
 	else
@@ -152,9 +152,9 @@ namespace lib {
 	    for( SizeT i=start,ii=0; i<ctSize; ++i,++ii)
 	      actCT->Get( i, (*r)[ ii], (*g)[ ii], (*b)[ ii]);
 
-	    delete p0; p0 = r;
-	    delete p1; p1 = g;
-	    delete p2; p2 = b;
+	    GDLDelete(p0); p0 = r;
+	    GDLDelete(p1); p1 = g;
+	    GDLDelete(p2); p2 = b;
 	  }
 	else
 	  {
@@ -200,6 +200,16 @@ namespace lib {
 	      }
 	  }
       }
-  }
 
+    // AC, 07/02/2012, please report any unexpected side effect (see test_tvlct.pro)
+    if (~get) {
+      GDLGStream* actStream = Graphics::GetDevice()->GetStream(false);
+      if (actStream != NULL) {
+	PLINT red[ctSize], green[ctSize], blue[ctSize];
+	actCT->Get( red, green, blue);
+	actStream->scmap1( red, green, blue, ctSize); 
+      }
+    }
+  }
+  
 } // namespace
