@@ -19,6 +19,10 @@
 #include "plotting.hpp"
 #include "math_utl.hpp"
 
+#ifdef _MSC_VER
+#define isnan _isnan
+#endif
+
 namespace lib {
 
   using namespace std;
@@ -58,12 +62,12 @@ namespace lib {
       xVal = !mapSet
         ? e->GetParAs< DDoubleGDL>(0)
         : static_cast<DDoubleGDL*>(
-          e->GetNumericArrayParDefined(0)->Convert2(DOUBLE, BaseGDL::COPY)
+          e->GetNumericArrayParDefined(0)->Convert2(GDL_DOUBLE, BaseGDL::COPY)
         );
       yVal = !mapSet
         ? e->GetParAs< DDoubleGDL>(1)
         : static_cast<DDoubleGDL*>(
-          e->GetNumericArrayParDefined(1)->Convert2(DOUBLE, BaseGDL::COPY)
+          e->GetNumericArrayParDefined(1)->Convert2(GDL_DOUBLE, BaseGDL::COPY)
         );
 
       xEl = xVal->N_Elements();
@@ -134,14 +138,14 @@ namespace lib {
     DDouble minVal = yStart, maxVal = yEnd;
 
     //CLIPPING
-    DLong noclip = 1;
-    e->AssureLongScalarKWIfPresent( "NOCLIP", noclip);
-    if (noclip == 0)
-    {
-      static int clippingix = e->KeywordIx( "CLIP"); 
-      DDoubleGDL* clippingD = e->IfDefGetKWAs<DDoubleGDL>( clippingix);
-      if (clippingD != NULL) Clipping( clippingD, xStart, xEnd, minVal, maxVal);
-    }
+//    DLong noclip = 1;
+//    e->AssureLongScalarKWIfPresent( "NOCLIP", noclip);
+//    if (noclip == 0)
+//    {
+//      static int clippingix = e->KeywordIx( "CLIP");
+//      DDoubleGDL* clippingD = e->IfDefGetKWAs<DDoubleGDL>( clippingix);
+//      if (clippingD != NULL) Clipping( clippingD, xStart, xEnd, minVal, maxVal);
+//    }
 
     // SA: following a patch from Joanna (3029409) TODO: this is repeated in PLOTS POLYFILL and XYOUTS
     if ( xEnd - xStart == 0 || yEnd - yStart == 0 || isnan(xStart) || isnan(yStart) ) {
