@@ -75,7 +75,7 @@ namespace lib {
     
     DDouble xStart, xEnd, yStart, yEnd;
     bool xLog, yLog;
-    DDouble minVal, maxVal;
+//    DDouble minVal, maxVal;
 
     get_axis_crange("X", xStart, xEnd);
     get_axis_crange("Y", yStart, yEnd);
@@ -90,7 +90,7 @@ namespace lib {
     if(color_arr != NULL)
       {
 	l_color_arr=static_cast<DLongGDL*>
-	  (color_arr->Convert2(LONG, BaseGDL::COPY));
+	  (color_arr->Convert2(GDL_LONG, BaseGDL::COPY));
 	if(color_arr->N_Elements() < minEl && color_arr->N_Elements() > 1)
 	  e->Throw( "Array "+e->GetParString(cix)+
 		    " does not have enough elements for COLOR keyword.");
@@ -106,7 +106,8 @@ namespace lib {
 actStream->wid( 0);
 
 
-    //start drawing
+    //start drawing. KEYWORDS are: CLIP(NO), COLOR(PARTIALLY), DATA(YES) , DEVICE(YES) ,
+    // NORMAL(YES) , FONT(NO), ORIENTATION(YES), /NOCLIP(YES), T3D(NO), Z(NO)
     //    actStream->Background( background);
     //    actStream->Color( color);
     //    gkw_background(e, actStream);
@@ -141,7 +142,7 @@ actStream->wid( 0);
     GetSFromPlotStructs(&sx, &sy);
     GetWFromPlotStructs(&wx, &wy);
 
-    int toto=0;
+    int isdatabydefault=0;
 
     if(e->KeywordSet("DEVICE")) {
       PLFLT xpix, ypix;
@@ -161,7 +162,7 @@ actStream->wid( 0);
       actStream->vpor(0, 1, 0, 1);
       xLog = false; yLog = false;
     } else {
-      toto=1;
+      isdatabydefault=1;
       actStream->NoSub();
       if (xLog || yLog) actStream->vpor(wx[0], wx[1], wy[0], wy[1]);
       else actStream->vpor(0, 1, 0, 1); // (to be merged with the condition on DataCoordLimits...)
@@ -172,32 +173,26 @@ actStream->wid( 0);
     // variables because map routines change these directly.
 
     //    if (e->KeywordSet("NORMAL") || e->KeywordSet("DATA")) {
-    if (e->KeywordSet("DATA") || (toto == 1)) {
+    if (e->KeywordSet("DATA") || (isdatabydefault == 1)) {
       DataCoordLimits(sx, sy, wx, wy, &xStart, &xEnd, &yStart, &yEnd, (xLog || yLog));
     }
 
-    minVal=yStart; maxVal=yEnd;
+//    minVal=yStart; maxVal=yEnd;
 
     //CLIPPING
-    DLong noclip=1;
-    e->AssureLongScalarKWIfPresent( "NOCLIP", noclip);
-    if(noclip == 0)
-      {
-	static int clippingix = e->KeywordIx( "CLIP"); 
-	DDoubleGDL* clippingD = e->IfDefGetKWAs<DDoubleGDL>( clippingix);
-	if( clippingD != NULL)
-	    Clipping( clippingD, xStart, xEnd, minVal, maxVal);
-      }
+//    DLong noclip=1;
+//    e->AssureLongScalarKWIfPresent( "NOCLIP", noclip);
+//    if(noclip == 0)
+//      {
+//	static int clippingix = e->KeywordIx( "CLIP");
+//	DDoubleGDL* clippingD = e->IfDefGetKWAs<DDoubleGDL>( clippingix);
+//	if( clippingD != NULL)
+//	    Clipping( clippingD, xStart, xEnd, minVal, maxVal);
+//      }
 
     // for orient
     PLFLT xScale = abs(xEnd - xStart), yScale = abs(yEnd - yStart);
 
-    // ??
-    if( yLog)
-      {
-	if( yStart <= 0.0) yStart = 0.0; else yStart = log10( yStart);
-	if( yEnd   <= 0.0) return; else yEnd = log10( yEnd);
-      }
 
     // SA: following a patch from Joanna (3029409) TODO: this is repeated in PLOTS POLYFILL and XYOUTS
     if ( xEnd - xStart == 0 || yEnd - yStart == 0 || isnan(xStart) || isnan(yStart) ) {
@@ -221,7 +216,7 @@ actStream->wid( 0);
     if(orient != NULL)
       {
 	d_orient=static_cast<DDoubleGDL*>
-	  (orient->Convert2(DOUBLE, BaseGDL::COPY));
+	  (orient->Convert2(GDL_DOUBLE, BaseGDL::COPY));
 	if(orient->N_Elements() < minEl && orient->N_Elements() > 1)
 	  e->Throw( "Array "+e->GetParString(oix)+
 		    " does not have enough elements for ORIENTATION keyword.");
@@ -237,13 +232,14 @@ actStream->wid( 0);
     DDouble alignment = 0.0;
     e->AssureDoubleScalarKWIfPresent( "ALIGNMENT", alignment);
 
-    //CHARSIZE
-    DFloat charsize;
-    gkw_charsize(e, actStream, charsize);
-
     // !P.MULTI vs. POSITION
     handle_pmulti_position(e, actStream);
 
+    //CHARSIZE Note that SIZE is apparently used in some old implementations and
+    //seems to be supported silently with *DL. So we support it also:
+    DFloat charsize;
+    gkw_charsize_xyouts(e, actStream, charsize);
+
     // WIDTH keyword
     static int widthIx = e->KeywordIx( "WIDTH");
     bool kwWidth = e->KeywordPresent(widthIx);
@@ -254,6 +250,8 @@ actStream->wid( 0);
     //     getting character height so it can be later used to move the 
     //     "reference point" half character height lower (tracker item no. 2982623)
     PLFLT charheight;
+
+#ifdef HAVE_PLPLOT_BEFORE_5994
     {
       PLFLT nullf, htmm;
       plgchr(&nullf, &htmm); // height of a letter in millimetres
@@ -265,6 +263,12 @@ actStream->wid( 0);
       plcalc_world(0., 0., &nullf, &wy0, &nulli); // wy = height of a letter in world coordinates
       charheight = wy - wy0;
     }
+#else // HAVE_PLPLOT_BEFORE_5994
+    cout << "Warning : charheight not changeable" << endl;
+    cout << "Warning : no more useful symbols in plplot 5.9.9-4" << endl;
+    charheight=1.;
+#endif // HAVE_PLPLOT_BEFORE_5994
+
 
     if(minEl == 1)
       {
@@ -289,7 +293,9 @@ actStream->wid( 0);
 
 	out=(*strVal)[0];
 	actStream->ptex(x,y,p_orient_x, p_orient_y,alignment,out.c_str());
+#ifdef HAVE_PLPLOT_BEFORE_5994
         if (kwWidth) width = plstrl(out.c_str());
+#endif
       }
     else
       {
@@ -329,7 +335,10 @@ actStream->wid( 0);
 	    out=(*strVal)[i];
             y += .5 * charheight;
 	    actStream->ptex(x,y,p_orient_x, p_orient_y,alignment,out.c_str());
+
+#ifdef HAVE_PLPLOT_BEFORE_5994
             if (kwWidth) width = max(plstrl(out.c_str()), width);
+#endif
 	  }
       }
     
@@ -340,9 +349,12 @@ actStream->wid( 0);
       //     width contains output from plstrl() expressed in millimetres
       //     plP_mmpcx() converts it into physical coordinates
       //     plP_gphy() gives "physical device limits in physical coordinates"
+
+#ifdef HAVE_PLPLOT_BEFORE_5994
       PLINT p_ixmin, p_ixmax, p_iymin, p_iymax;
       plP_gphy(&p_ixmin, &p_ixmax, &p_iymin, &p_iymax);
       e->SetKW(widthIx, new DFloatGDL(plP_mmpcx(width)/double(p_ixmax - p_ixmin)));
+#endif
     }
   } // }}}
 
