@@ -24,6 +24,7 @@ email                : m_schellens@users.sf.net
 #include "dinterpreter.hpp"
 
 #include "objects.hpp"
+#include "nullgdl.hpp"
 
 using namespace std;
 
@@ -55,7 +56,8 @@ ProgNode::ProgNode( const RefDNode& refNode):
   down( NULL), 
   right( NULL),
   lineNumber( refNode->getLine()),
-  cData( refNode->StealCData()),
+  cData( refNode->StealCData()), // stealing is crucial here because references might exist
+  // as arrayindices
   libPro( refNode->libPro),
   libFun( refNode->libFun),
   var( refNode->var),
@@ -75,7 +77,7 @@ ProgNode::~ProgNode()
   // delete cData in case this node is a constant
   if( (getType() == GDLTokenTypes::CONSTANT))
      {
-      delete cData;
+      GDLDelete(cData);
      }
   if( (getType() == GDLTokenTypes::ARRAYIX))
     {
@@ -87,7 +89,6 @@ ProgNode::~ProgNode()
 }
 
 
-
 ProgNodeP ProgNode::GetNULLProgNodeP()
   {
 	return interpreter->GetNULLProgNodeP();
@@ -186,7 +187,7 @@ BaseGDL* ASSIGN_ARRAYEXPR_MFCALLNode::Eval()
 
 	  if( res != (*l))
 	    {
-	      delete *l;
+	      GDLDelete(*l);
 	      *l = res->Dup();
 
 	      if( r_guard.get() == res) // owner
@@ -242,7 +243,7 @@ BaseGDL* ASSIGN_REPLACENode::Eval()
 
     if( res != (*l))
     {
-      delete *l;
+      GDLDelete(*l);
       *l = res->Dup();
 
       if( r_guard.get() == res) // owner
@@ -259,7 +260,7 @@ BaseGDL* ASSIGN_REPLACENode::Eval()
 BaseGDL* ARRAYDEFNode::Eval()
 {
   // GDLInterpreter::
-  DType  cType=UNDEF; // conversion type
+  DType  cType=GDL_UNDEF; // conversion type
   SizeT maxRank=0;
   ExprListT            exprList;
   BaseGDL*           cTypeData;
@@ -275,15 +276,18 @@ BaseGDL* ARRAYDEFNode::Eval()
     //WRONG    _t = ProgNode::interpreter->_retTree;
 			
     // add first (this way it will get cleaned up anyway)
+    if( e == NullGDL::GetSingleInstance())
+      continue;
+      
     exprList.push_back(e);
 			
     DType ty=e->Type();
-    if( ty == UNDEF)
+    if( ty == GDL_UNDEF)
       {
 	throw GDLException( _t, "Variable is undefined: "+
 			    ProgNode::interpreter->Name(e),true,false);
       }
-    if( cType == UNDEF) 
+    if( cType == GDL_UNDEF) 
       {
 	cType=ty;
 	cTypeData=e;
@@ -306,7 +310,7 @@ BaseGDL* ARRAYDEFNode::Eval()
 		cTypeData=e;
 	      }
 	  }
-	if( ty == STRUCT)
+	if( ty == GDL_STRUCT)
 	  {
 	    // check for struct compatibility
 	    DStructDesc* newS=
@@ -336,6 +340,9 @@ BaseGDL* ARRAYDEFNode::Eval()
   }
   _t = this->getNextSibling();
 	
+  if( exprList.empty())
+    return NullGDL::GetSingleInstance();
+  
   BaseGDL* res=cTypeData->CatArray(exprList,this->arrayDepth,maxRank);
 	
 //   ProgNode::interpreter->_retTree = _t;
@@ -417,7 +424,7 @@ BaseGDL* NSTRUCNode::Eval()
     } 
   else
     {   // NTags() == 0
-	// not completely defined (only name in list)
+	// not completely defined yet (only name in list)
       nStructDesc= oStructDesc;
     }
 	
@@ -458,7 +465,17 @@ BaseGDL* NSTRUCNode::Eval()
 	    DStructDesc* inherit=ProgNode::interpreter->GetStruct( ii->getText(), _t);
 			
 	    //   nStructDesc->AddParent( inherit);
-	    instance->AddParent( inherit);
+	    instance->AddParent( inherit); // adds operatorList if GDL_OBJECT
+	    
+	    // set up operators form member functions
+	    // they were set when the struct was not derived from GDL_OBJECT and hence ignored
+	    if( nStructDesc == oStructDesc && inherit->Name() == GDL_OBJECT_NAME)
+	    {
+	      // operatorList should have been set in instance->AddParent( inherit); 
+	      assert( nStructDesc->GetOperatorList() != NULL);
+	      // find operators in member subroutines
+	      nStructDesc->SetupOperators();
+	    }
 			
 	    break;
 	  }
@@ -510,7 +527,7 @@ BaseGDL* NSTRUCNode::Eval()
 	{
 	  oStructDesc->AssureIdentical(nStructDesc);
 	  instance->DStructGDL::SetDesc(oStructDesc);
-	  //delete nStructDesc; // auto_ptr
+	  //GDLDelete(nStructDesc); // auto_ptr
 	}
     }
   else
@@ -552,13 +569,15 @@ BaseGDL* NSTRUC_REFNode::Eval()
 }
 
 
-
+// parameter nodes ////////////////////////////////////////////////
+// 1. keywords ////
 void KEYDEF_REFNode::Parameter( EnvBaseT* actEnv)
 {
   ProgNodeP _t = this->getFirstChild();
 //   ProgNodeP knameR = _t;
   // 			match(antlr::RefAST(_t),IDENTIFIER);
 //   _t = _t->getNextSibling();
+  
   BaseGDL** kvalRef=_t->getNextSibling()->LEval();
   //ProgNode::interpreter->ref_parameter(_t->getNextSibling(), actEnv);
 
@@ -567,10 +586,6 @@ void KEYDEF_REFNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
-// void KEYDEF_REFNode::ParameterVarNum( EnvBaseT* actEnv)
-// {
-//   return Parameter( actEnv);
-// }
 
 void KEYDEF_REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 {
@@ -579,7 +594,7 @@ void KEYDEF_REF_EXPRNode::Parameter( EnvBaseT* actEnv)
   // 			match(antlr::RefAST(_t),IDENTIFIER);
 //   _t = _t->getNextSibling();
   BaseGDL* kval= _t->getNextSibling()->Eval();//expr(_t);
-  delete kval;
+  GDLDelete(kval);
 //   _t = ProgNode::interpreter->_retTree;
   BaseGDL** kvalRef=_t->getNextSibling()->getNextSibling()->LEval();
 //   BaseGDL** kvalRef=ProgNode::interpreter->_retTree->LEval();
@@ -591,10 +606,6 @@ void KEYDEF_REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
-// void KEYDEF_REF_EXPRNode::ParameterVarNum( EnvBaseT* actEnv)
-// {
-//   return Parameter( actEnv);
-// }
 
 void KEYDEFNode::Parameter( EnvBaseT* actEnv)
 {
@@ -608,11 +619,63 @@ void KEYDEFNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
-// void KEYDEFNode::ParameterVarNum( EnvBaseT* actEnv)
-// {
-//   return Parameter( actEnv);
-// }
 
+void KEYDEF_REF_CHECKNode::Parameter( EnvBaseT* actEnv)
+{
+//   ProgNodeP _t = this->getFirstChild();
+//   ProgNodeP knameCk = _t;
+  // 			match(antlr::RefAST(_t),IDENTIFIER);
+//   _t = _t->getNextSibling();
+  ProgNodeP p = this->getFirstChild()->getNextSibling();
+
+  if( p->getType() == GDLTokenTypes::QUESTION)
+  {
+    QUESTIONNode* q = static_cast<QUESTIONNode*>( p);
+    ProgNodeP branch = q->AsParameter();
+    
+    while( branch->getType() == GDLTokenTypes::QUESTION)
+    {
+      QUESTIONNode* qRecursive = static_cast<QUESTIONNode*>( branch);
+      branch = qRecursive->AsParameter();
+    }
+    
+    BaseGDL* rVal;
+    BaseGDL** lVal = branch->EvalRefCheck( rVal);
+    if( lVal != NULL)
+    {   // pass reference
+      actEnv->SetKeyword(this->getFirstChild()->getText(), lVal); 
+    }
+    else
+    {   // pass value
+      actEnv->SetKeyword(this->getFirstChild()->getText(), rVal); 
+    }
+  }
+  else
+  {
+  BaseGDL* kval=ProgNode::interpreter->
+    lib_function_call(this->getFirstChild()->getNextSibling());
+			
+  BaseGDL** kvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( kval);
+  if( kvalRef != NULL)
+    {   // pass reference
+      actEnv->SetKeyword(this->getFirstChild()->getText(), kvalRef); 
+    }
+  else 
+    {   // pass value
+      actEnv->SetKeyword(this->getFirstChild()->getText(), kval); 
+    }
+  }
+  ProgNode::interpreter->_retTree = this->getNextSibling();
+}
+
+
+// 2. Parameters ////
+bool REFNode::ParameterDirect( BaseGDL*& ref)
+{
+  BaseGDL** pvalRef=this->getFirstChild()->LEval();
+  ref = *pvalRef;
+  return true;
+}
 void REFNode::Parameter( EnvBaseT* actEnv)
 {
 //   ProgNodeP _t = this->getFirstChild();
@@ -631,12 +694,21 @@ void REFVNNode::Parameter( EnvBaseT* actEnv)
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 
+
+bool REF_EXPRNode::ParameterDirect( BaseGDL*& ref)
+{
+  BaseGDL* pval= this->getFirstChild()->Eval();//expr(_t);
+  GDLDelete(pval);
+  BaseGDL** pvalRef=this->getFirstChild()->getNextSibling()->LEval();
+  ref = *pvalRef;
+  return true;
+}
 void REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 {
   // 			match(antlr::RefAST(_t),REF_EXPR);
 //   ProgNodeP _t = this->getFirstChild();
   BaseGDL* pval= this->getFirstChild()->Eval();//expr(_t);
-  delete pval;
+  GDLDelete(pval);
 //   _t = ProgNode::interpreter->_retTree;
   BaseGDL** pvalRef=this->getFirstChild()->getNextSibling()->LEval();
 //   ProgNode::interpreter->
@@ -650,69 +722,126 @@ void REF_EXPRNode::Parameter( EnvBaseT* actEnv)
 void REF_EXPRVNNode::Parameter( EnvBaseT* actEnv)
 {
   BaseGDL* pval= this->getFirstChild()->Eval();//expr(_t);
-  delete pval;
+  GDLDelete(pval);
   BaseGDL** pvalRef=this->getFirstChild()->getNextSibling()->LEval();
   actEnv->SetNextParUncheckedVarNum(pvalRef); 
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 
-void KEYDEF_REF_CHECKNode::Parameter( EnvBaseT* actEnv)
+// returns true if reference, false else
+bool REF_CHECKNode::ParameterDirect( BaseGDL*& pval)
 {
-//   ProgNodeP _t = this->getFirstChild();
-//   ProgNodeP knameCk = _t;
-  // 			match(antlr::RefAST(_t),IDENTIFIER);
-//   _t = _t->getNextSibling();
-  BaseGDL* kval=ProgNode::interpreter->
-    lib_function_call(this->getFirstChild()->getNextSibling());
-			
-  BaseGDL** kvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( kval);
-  if( kvalRef != NULL)
-    {   // pass reference
-      actEnv->SetKeyword(this->getFirstChild()->getText(), kvalRef); 
-    }
-  else 
-    {   // pass value
-      actEnv->SetKeyword(this->getFirstChild()->getText(), kval); 
-    }
-			
-  ProgNode::interpreter->_retTree = this->getNextSibling();
+  ProgNodeP p = this->getFirstChild();
+  if( p->getType() == GDLTokenTypes::QUESTION)
+  {
+    // for the trinary operator we just use pass by value which should be ok, 
+    // as direct functions cannot modify their (single) parameter anyway.
+    // LEval might save a copy operation here, but considering that direct functions are
+    // probably very seldom called with the trinary operator we leave it like this for now. 
+    pval = p->Eval();
+    return false; // pass value
+  }
+  pval=ProgNode::interpreter->lib_function_call(p);
+  BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
+  return (pvalRef != NULL);
+//   if( pvalRef != NULL)
+//     {   // pass reference
+//       return true;
+//     }
+//   else 
+//     {   // pass value
+//       return false;
+//     }
 }
-// void KEYDEF_REF_CHECKNode::ParameterVarNum( EnvBaseT* actEnv)
-// {
-//   return Parameter( actEnv);
-// }
-
 void REF_CHECKNode::Parameter( EnvBaseT* actEnv)
 {
-  BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
-			
-  BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
-  if( pvalRef != NULL)
+  ProgNodeP p = this->getFirstChild();
+
+  if( p->getType() == GDLTokenTypes::QUESTION)
+  {
+    QUESTIONNode* q = static_cast<QUESTIONNode*>( p);
+    ProgNodeP branch = q->AsParameter();
+    
+    while( branch->getType() == GDLTokenTypes::QUESTION)
+    {
+      QUESTIONNode* qRecursive = static_cast<QUESTIONNode*>( branch);
+      branch = qRecursive->AsParameter();
+    }
+    
+    BaseGDL* rVal;
+    BaseGDL** lVal = branch->EvalRefCheck( rVal);
+    if( lVal != NULL)
     {   // pass reference
-      actEnv->SetNextParUnchecked( pvalRef); 
+      actEnv->SetNextParUnchecked( lVal); 
     }
-  else 
+    else
     {   // pass value
-      actEnv->SetNextParUnchecked( pval); 
+      actEnv->SetNextParUnchecked( rVal); 
     }
-			
+  }
+  else
+  {  
+    BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
+			  
+    BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
+    if( pvalRef != NULL)
+      {   // pass reference
+	actEnv->SetNextParUnchecked( pvalRef); 
+      }
+    else 
+      {   // pass value
+	actEnv->SetNextParUnchecked( pval); 
+      }
+  }
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 void REF_CHECKVNNode::Parameter( EnvBaseT* actEnv)
 {
-  BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
-  BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
-  if( pvalRef != NULL)
+  ProgNodeP p = this->getFirstChild();
+
+  if( p->getType() == GDLTokenTypes::QUESTION)
+  {
+    QUESTIONNode* q = static_cast<QUESTIONNode*>( p);
+    ProgNodeP branch = q->AsParameter();
+    
+    while( branch->getType() == GDLTokenTypes::QUESTION)
+    {
+      QUESTIONNode* qRecursive = static_cast<QUESTIONNode*>( branch);
+      branch = qRecursive->AsParameter();
+    }
+    
+    BaseGDL* rVal;
+    BaseGDL** lVal = branch->EvalRefCheck( rVal);
+    if( lVal != NULL)
     {   // pass reference
-      actEnv->SetNextParUncheckedVarNum( pvalRef); 
+      actEnv->SetNextParUncheckedVarNum( lVal); 
     }
-  else 
+    else
     {   // pass value
-      actEnv->SetNextParUncheckedVarNum( pval); 
+      actEnv->SetNextParUncheckedVarNum( rVal); 
     }
+  }
+  else
+  {
+    BaseGDL* pval=ProgNode::interpreter->lib_function_call(this->getFirstChild());
+    BaseGDL** pvalRef = ProgNode::interpreter->callStack.back()->GetPtrTo( pval);
+    if( pvalRef != NULL)
+      {   // pass reference
+	actEnv->SetNextParUncheckedVarNum( pvalRef); 
+      }
+    else 
+      {   // pass value
+	actEnv->SetNextParUncheckedVarNum( pval); 
+      }
+  }
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 
+bool ParameterNode::ParameterDirect( BaseGDL*& pval)
+{
+  pval = this->getFirstChild()->Eval(); 
+  return false;			
+}
 void ParameterNode::Parameter( EnvBaseT* actEnv)
 {
 //   BaseGDL* pval=this->Eval();//expr(this);
@@ -722,14 +851,29 @@ void ParameterNode::Parameter( EnvBaseT* actEnv)
 			
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
-
 void ParameterVNNode::Parameter( EnvBaseT* actEnv)
 {
   actEnv->SetNextParUncheckedVarNum(this->getFirstChild()->Eval()); 
   ProgNode::interpreter->_retTree = this->getNextSibling();
 }
 
-
+RetCode  WRAPPED_FUNNode::Run()
+{
+  EnvUDT* env = static_cast<EnvUDT*>( ProgNode::interpreter->callStack.back());
+  BaseGDL* res = (*this->fun)( env);
+  interpreter->SetRetTree( this->getNextSibling()); // ???
+  assert( ProgNode::interpreter->returnValue == NULL);
+//   GDLDelete(ProgNode::interpreter->returnValue);
+  ProgNode::interpreter->returnValue = res;
+  return RC_RETURN;
+}
+RetCode  WRAPPED_PRONode::Run()
+{
+  EnvUDT* env = static_cast<EnvUDT*>( ProgNode::interpreter->callStack.back());
+  (*this->pro)( env);
+  interpreter->SetRetTree( this->getNextSibling()); // ???
+  return RC_RETURN;
+}
 
 RetCode  ASSIGNNode::Run()
 {
@@ -868,7 +1012,7 @@ RetCode  ASSIGN_ARRAYEXPR_MFCALLNode::Run()
   
       if( r != (*l))
 	{
-	  delete *l;
+	  GDLDelete(*l);
 
 	  if( r_guard.get() == r)
 	    *l = r_guard.release();
@@ -946,8 +1090,8 @@ RetCode  ASSIGN_REPLACENode::Run()
   
   BaseGDL** l=_t->LEval();
 
-  if( r != (*l))
-    delete *l;
+  if( r != (*l)) // && (*l) != NullGDL::GetSingleInstance())
+    GDLDelete(*l);
   
   *l = r_guard.release();
 
@@ -959,8 +1103,8 @@ RetCode  ASSIGN_REPLACENode::Run()
 
 RetCode  PCALL_LIBNode::Run()
 {
-  // better than auto_ptr: auto_ptr wouldn't remove newEnv from the stack
-  StackGuard<EnvStackT> guard( ProgNode::interpreter->CallStack());
+//   // better than auto_ptr: auto_ptr wouldn't remove newEnv from the stack
+//   StackGuard<EnvStackT> guard( ProgNode::interpreter->CallStack());
   BaseGDL *self;
 	
   // 		match(antlr::RefAST(_t),PCALL_LIB);
@@ -972,11 +1116,13 @@ RetCode  PCALL_LIBNode::Run()
   EnvT* newEnv=new EnvT( pl, pl->libPro);//libProList[pl->proIx]);
 		
   ProgNode::interpreter->parameter_def_nocheck(_t, newEnv);
+  auto_ptr<EnvT> guardEnv( newEnv);
+
   //   _t = _retTree;
   //if( this->getLine() != 0) ProgNode::interpreter->callStack.back()->SetLineNumber( this->getLine());
 		
-  // push environment onto call stack
-  ProgNode::interpreter->callStack.push_back(newEnv);
+//   // push environment onto call stack
+//   ProgNode::interpreter->callStack.push_back(newEnv);
 		
   // make the call
   static_cast<DLibPro*>(newEnv->GetPro())->Pro()(newEnv);
@@ -1158,14 +1304,20 @@ RetCode   FORNode::Run()//for_statement(ProgNodeP _t) {
 //   BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
   auto_ptr<BaseGDL> s_guard(s);
   
-  delete loopInfo.endLoopVar;
+  GDLDelete(loopInfo.endLoopVar);
   loopInfo.endLoopVar=this->GetFirstChild()->GetNextSibling()->Eval();
 //   loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
   
   s->ForCheck( &loopInfo.endLoopVar);
   
+  if( loopInfo.endLoopVar->Type() != s->Type()) // promote s
+    {
+      BaseGDL* sPromote = s->Convert2(loopInfo.endLoopVar->Type(), BaseGDL::COPY);
+      s_guard.reset( sPromote);
+    }
+  
   // ASSIGNMENT used here also
-  delete (*v);
+  GDLDelete((*v));
   (*v)= s_guard.release(); // s held in *v after this
   
   if( (*v)->ForCondUp( loopInfo.endLoopVar))
@@ -1211,7 +1363,7 @@ RetCode   FOR_LOOPNode::Run()
 	}
 	else
 	{
-		delete loopInfo.endLoopVar;
+		GDLDelete(loopInfo.endLoopVar);
 		loopInfo.endLoopVar = NULL;
 		ProgNode::interpreter->_retTree = this->GetNextSibling();
 	}
@@ -1233,18 +1385,25 @@ RetCode   FOR_STEPNode::Run()//for_statement(ProgNodeP _t) {
 //  BaseGDL* s=ProgNode::interpreter->expr( this->GetFirstChild());
   auto_ptr<BaseGDL> s_guard(s);
 
-  delete loopInfo.endLoopVar;
+  GDLDelete(loopInfo.endLoopVar);
   loopInfo.endLoopVar=this->GetFirstChild()->GetNextSibling()->Eval();
 //  loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling());
 
-  delete loopInfo.loopStepVar;
+  GDLDelete(loopInfo.loopStepVar);
   loopInfo.loopStepVar=this->GetFirstChild()->GetNextSibling()->GetNextSibling()->Eval();
 //   loopInfo.loopStepVar=ProgNode::interpreter->expr(this->GetFirstChild()->GetNextSibling()->GetNextSibling());
 
   s->ForCheck( &loopInfo.endLoopVar, &loopInfo.loopStepVar);
 
+  if( loopInfo.endLoopVar->Type() != s->Type()) // promote s
+    {
+      BaseGDL* sPromote = s->Convert2(loopInfo.endLoopVar->Type(), BaseGDL::COPY);
+      s_guard.reset( sPromote);
+      assert( loopInfo.loopStepVar->Type() == s_guard.get()->Type());
+    }
+
   // ASSIGNMENT used here also
-  delete (*v);
+  GDLDelete((*v));
   (*v)= s_guard.release(); // s held in *v after this
 
   if( loopInfo.loopStepVar->Sgn() == -1)
@@ -1307,9 +1466,9 @@ RetCode   FOR_STEP_LOOPNode::Run()
     }
   }
   
-  delete loopInfo.endLoopVar;
+  GDLDelete(loopInfo.endLoopVar);
   loopInfo.endLoopVar = NULL;
-  delete loopInfo.loopStepVar;
+  GDLDelete(loopInfo.loopStepVar);
   loopInfo.loopStepVar = NULL;
   ProgNode::interpreter->_retTree = this->GetNextSibling();
   return RC_OK;
@@ -1324,7 +1483,7 @@ RetCode   FOREACHNode::Run()
 
 	BaseGDL** v=vP->LEval(); // ProgNode::interpreter->l_simple_var(vP);
 
-	delete loopInfo.endLoopVar;
+	GDLDelete(loopInfo.endLoopVar);
 	loopInfo.endLoopVar=this->GetFirstChild()->Eval();
 //	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
 
@@ -1334,7 +1493,7 @@ RetCode   FOREACHNode::Run()
 	//SizeT nEl = loopInfo.endLoopVar->N_Elements();
 
 	// ASSIGNMENT used here also
-	delete (*v);
+	GDLDelete((*v));
 	(*v) = loopInfo.endLoopVar->NewIx( 0);
 
 	ProgNode::interpreter->_retTree = vP->GetNextSibling();
@@ -1362,14 +1521,14 @@ RetCode   FOREACH_LOOPNode::Run()
 	if( loopInfo.foreachIx < nEl)
 	{
 		// ASSIGNMENT used here also
-		delete (*v);
+		GDLDelete((*v));
 		(*v) = loopInfo.endLoopVar->NewIx( loopInfo.foreachIx);
 
 		ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling();
 		return RC_OK;
 	}
 
-	delete loopInfo.endLoopVar;
+	GDLDelete(loopInfo.endLoopVar);
 	loopInfo.endLoopVar = NULL;
 	// 	loopInfo.foreachIx = -1;
 	ProgNode::interpreter->SetRetTree( this->GetNextSibling());
@@ -1389,7 +1548,7 @@ RetCode   FOREACH_INDEXNode::Run()
 	BaseGDL** v=vP->LEval(); //ProgNode::interpreter->l_simple_var(vP);
 	BaseGDL** index=indexP->LEval(); //ProgNode::interpreter->l_simple_var(indexP);
 
-	delete loopInfo.endLoopVar;
+	GDLDelete(loopInfo.endLoopVar);
 	loopInfo.endLoopVar=this->GetFirstChild()->Eval(); 
 // 	loopInfo.endLoopVar=ProgNode::interpreter->expr(this->GetFirstChild());
 
@@ -1399,11 +1558,11 @@ RetCode   FOREACH_INDEXNode::Run()
 	//SizeT nEl = loopInfo.endLoopVar->N_Elements();
 
 	// ASSIGNMENT used here also
-	delete (*v);
+	GDLDelete((*v));
 	(*v) = loopInfo.endLoopVar->NewIx( 0);
 	
 	// ASSIGNMENT used here also
-	delete (*index);
+	GDLDelete((*index));
 	(*index) = new DLongGDL( 0);
 
 	ProgNode::interpreter->_retTree = indexP->GetNextSibling();
@@ -1432,18 +1591,18 @@ RetCode   FOREACH_INDEX_LOOPNode::Run()
 	if( loopInfo.foreachIx < nEl)
 	{
 		// ASSIGNMENT used here also
-		delete (*v);
+		GDLDelete((*v));
 		(*v) = loopInfo.endLoopVar->NewIx( loopInfo.foreachIx);
 
 		// ASSIGNMENT used here also
-		delete (*index);
+		GDLDelete((*index));
 		(*index) = new DLongGDL( loopInfo.foreachIx);
 		
 		ProgNode::interpreter->_retTree = this->GetFirstChild()->GetNextSibling()->GetNextSibling();
 		return RC_OK;
 	}
 
-	delete loopInfo.endLoopVar;
+	GDLDelete(loopInfo.endLoopVar);
 	loopInfo.endLoopVar = NULL;
 	// 	loopInfo.foreachIx = -1;
 	ProgNode::interpreter->SetRetTree( this->GetNextSibling());
@@ -1748,13 +1907,13 @@ RetCode      CONTINUENode::Run()
 }
 RetCode      BREAKNode::Run()
 {
-  if( this->breakTarget == NULL)
+  if( !this->breakTargetSet)
 	{
 	assert( interpreter != NULL);
 	EnvBaseT* e = interpreter->CallStack().back();
-	throw GDLException( this, "BREAK must be enclosed within a FOR, WHILE, or REPEAT loop.", true, false);
+	throw GDLException( this, "BREAK must be enclosed within a loop (FOR, WHILE, REPEAT), CASE or SWITCH statement.", true, false);
 	}
-ProgNode::interpreter->SetRetTree( this->breakTarget);
+  ProgNode::interpreter->SetRetTree( this->breakTarget);
   return RC_OK;
 }
 RetCode      LABELNode::Run()
@@ -1784,7 +1943,8 @@ RetCode   RETFNode::Run()
 		{
 			BaseGDL* e=_t->Eval(); //ProgNode::interpreter->expr(_t);
 			interpreter->SetRetTree( _t->getNextSibling()); // ???
-			delete ProgNode::interpreter->returnValue;
+			assert(ProgNode::interpreter->returnValue == NULL);
+// 			GDLDelete(ProgNode::interpreter->returnValue);
 			ProgNode::interpreter->returnValue=e;
 
 			GDLInterpreter::CallStack().back()->RemoveLoc( e); // steal e from local list
@@ -1793,6 +1953,7 @@ RetCode   RETFNode::Run()
 		{
 			BaseGDL** eL=ProgNode::interpreter->l_ret_expr(_t);
 			// returnValueL is otherwise owned
+			assert(ProgNode::interpreter->returnValueL == NULL);
 			ProgNode::interpreter->returnValueL=eL;
 		}
 	//if( !(interruptEnable && sigControlC) && ( debugMode == DEBUG_CLEAR))
