@@ -60,6 +60,11 @@ public:
 
   void AdjustTypesNC( std::auto_ptr<BaseGDL>& g1, BaseGDL*& e1, 
 		      std::auto_ptr<BaseGDL>& g2, BaseGDL*& e2);
+  // for overloaded operators 
+  void SetupGuards( Guard<BaseGDL>& g1, BaseGDL*& e1,
+		    Guard<BaseGDL>& g2, BaseGDL*& e2);
+  void AdjustTypesNCNull( Guard<BaseGDL>& g1, BaseGDL*& e1, 
+		      Guard<BaseGDL>& g2, BaseGDL*& e2);
 
   //  int getType() { return GDLTokenTypes::EXPR;}
 };
@@ -103,6 +108,24 @@ public:
   BaseGDL* Eval();
 };
 
+class FCALL_LIB_DIRECTNode: public LeafNode
+{
+public:
+  FCALL_LIB_DIRECTNode( const RefDNode& refNode): LeafNode( refNode)
+  {}
+  BaseGDL** LEval();
+  BaseGDL* Eval();
+};
+
+class FCALL_LIB_N_ELEMENTSNode: public LeafNode
+{
+public:
+  FCALL_LIB_N_ELEMENTSNode( const RefDNode& refNode): LeafNode( refNode)
+  {}
+  BaseGDL** LEval();
+  BaseGDL* Eval();
+};
+
 
 
 class FCALL_LIBNode: public LeafNode
@@ -110,6 +133,7 @@ class FCALL_LIBNode: public LeafNode
 public:
   FCALL_LIBNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* Eval();
   BaseGDL* EvalFCALL_LIB(); // might return a non new value
@@ -124,6 +148,7 @@ class MFCALLNode: public LeafNode
 public:
   MFCALLNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* Eval();
   BaseGDL** LExpr( BaseGDL* right);
@@ -137,6 +162,7 @@ class MFCALL_PARENTNode: public LeafNode
 public:
   MFCALL_PARENTNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* Eval();
   BaseGDL** LExpr( BaseGDL* right);
@@ -150,12 +176,37 @@ class FCALLNode: public LeafNode
 public:
   FCALLNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LExpr( BaseGDL* right);
   BaseGDL** LEval();
   BaseGDL* Eval();
 //   BaseGDL** LExprGrab( BaseGDL* right);  
 };
 
+class ARRAYEXPR_FCALLNode: public LeafNode
+{
+private:
+  int fcallNodeFunIx;
+  FCALLNode* fcallNode;
+  ARRAYEXPRNode* arrayExprNode;
+  
+  ARRAYEXPR_FCALLNode(){} // disable
+
+public:
+  ARRAYEXPR_FCALLNode( const RefDNode& refNode): LeafNode( refNode)
+  {
+    assert( dynamic_cast<FCALLNode*>(this->getFirstChild()->getNextSibling()));
+    assert( dynamic_cast<ARRAYEXPRNode*>(this->getFirstChild()));
+    arrayExprNode = static_cast<ARRAYEXPRNode*>(this->getFirstChild());
+    fcallNode = dynamic_cast<FCALLNode*>(arrayExprNode->getNextSibling());
+    fcallNodeFunIx = fcallNode->funIx;
+  }
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
+  BaseGDL** LExpr( BaseGDL* right);
+  BaseGDL** LEval();
+  BaseGDL* Eval();
+};
+
 
 
 class ARRAYEXPR_MFCALLNode: public LeafNode
@@ -163,6 +214,7 @@ class ARRAYEXPR_MFCALLNode: public LeafNode
 public:
   ARRAYEXPR_MFCALLNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval(); //{ ThrowGDLException( "Internal errorr: ARRAYEXPR_MFCALLNode called.");}
   BaseGDL* Eval();
   BaseGDL** LExpr( BaseGDL* right);
@@ -176,8 +228,10 @@ class VARNode: public LeafNode
 public:
   VARNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* EvalNCNull();
   BaseGDL* Eval();
   BaseGDL** LExpr( BaseGDL* right);  
 //   BaseGDL** LExprGrab( BaseGDL* right);  
@@ -187,8 +241,10 @@ class VARPTRNode: public LeafNode
 public:
   VARPTRNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* EvalNC();
+  BaseGDL* EvalNCNull();
   BaseGDL* Eval();
   BaseGDL** LExpr( BaseGDL* right);
 //   BaseGDL** LExprGrab( BaseGDL* right);  
@@ -198,6 +254,7 @@ class SYSVARNode: public LeafNode
 public:
   SYSVARNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* EvalNC();
   BaseGDL* Eval();
@@ -209,6 +266,7 @@ class DEREFNode: public LeafNode
 public:
   DEREFNode( const RefDNode& refNode): LeafNode( refNode)
   {}
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* EvalNC();
   BaseGDL* Eval();
@@ -220,18 +278,25 @@ class CONSTANTNode: public LeafNode
 public:
   CONSTANTNode( ProgNodeP r, BaseGDL* c): LeafNode()
   { 
+if( c == NULL)
+    assert( c != NULL);
     setType( GDLTokenTypes::CONSTANT);
     cData = c;
     right = r;
   }
   CONSTANTNode( const RefDNode& refNode): LeafNode( refNode)
-  {}
+  {
+if( cData == NULL)
+    assert( cData != NULL);
+  }
   BaseGDL* EvalNC();
   BaseGDL* Eval();
 };
 class ARRAYDEFNode: public DefaultNode
 { public:
-  ARRAYDEFNode( const RefDNode& refNode): DefaultNode( refNode){}
+  ARRAYDEFNode( const RefDNode& refNode): DefaultNode( refNode)
+  {}
+  
   BaseGDL* Eval();
   bool ConstantArray()
   {
@@ -271,6 +336,7 @@ class QUESTIONNode: public TrinaryExpr
 { public:
   QUESTIONNode( const RefDNode& refNode): TrinaryExpr( refNode){}
   BaseGDL* Eval();
+  ProgNodeP AsParameter(); // as parameter or keyword
   BaseGDL** LExpr(BaseGDL* right);
 //   BaseGDL** LExprGrab(BaseGDL* right);
 };
@@ -499,11 +565,21 @@ class PLUSNCNode: public BinaryExprNC
   PLUSNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
   BaseGDL* Eval();
 };
+class PLUSNC12Node: public BinaryExprNC // both are NC
+{ public:
+  PLUSNC12Node( const RefDNode& refNode): BinaryExprNC( refNode){}
+  BaseGDL* Eval();
+};
 class MINUSNCNode: public BinaryExprNC
 { public:
   MINUSNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
   BaseGDL* Eval();
 };
+class MINUSNC12Node: public BinaryExprNC
+{ public:
+  MINUSNC12Node( const RefDNode& refNode): BinaryExprNC( refNode){}
+  BaseGDL* Eval();
+};
 class LTMARKNCNode: public BinaryExprNC
 { public:
   LTMARKNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
@@ -514,6 +590,11 @@ class GTMARKNCNode: public BinaryExprNC
   GTMARKNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
   BaseGDL* Eval();
 };
+class ASTERIXNC12Node: public BinaryExprNC // both are NC
+{ public:
+  ASTERIXNC12Node( const RefDNode& refNode): BinaryExprNC( refNode){}
+  BaseGDL* Eval();
+};
 class ASTERIXNCNode: public BinaryExprNC
 { public:
   ASTERIXNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
@@ -534,6 +615,11 @@ class SLASHNCNode: public BinaryExprNC
   SLASHNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
   BaseGDL* Eval();
 };
+class SLASHNC12Node: public BinaryExprNC
+{ public:
+  SLASHNC12Node( const RefDNode& refNode): BinaryExprNC( refNode){}
+  BaseGDL* Eval();
+};
 class MOD_OPNCNode: public BinaryExprNC
 { public:
   MOD_OPNCNode( const RefDNode& refNode): BinaryExprNC( refNode){}
