@@ -27,9 +27,11 @@ extern "C" {
 // quoting http://permalink.gmane.org/gmane.os.openbsd.tech/19860 :
 // 'wordexp() will never be in OpenBSD's libc' :)
 // (TODO: perhaps better to implement it using HAVE_WORDEXP_H? + once more below in WordExp())
+#ifndef _MSC_VER
 #if !defined(__OpenBSD__)
 #  include <wordexp.h>
 #endif
+#endif
 
 #ifdef __CYGWIN__
 }
@@ -179,6 +181,17 @@ string StrUpCase(const string& s)
     r[i]=toupper(sCStr[i]);
   return string(r);
 }
+void StrUpCaseInplace( string& s)
+{
+  unsigned len=s.length();
+//   char const *sCStr=s.c_str();
+//   char* r = new char[len+1];
+//   ArrayGuard<char> guard( r);
+//   r[len]=0;
+  for(unsigned i=0;i<len;i++)
+    s[i]=toupper(s[i]);
+//   return string(r);
+}
 
 string StrLowCase(const string& s)
 {
@@ -191,6 +204,14 @@ string StrLowCase(const string& s)
     r[i]=tolower(sCStr[i]);
   return string(r);
 }
+void StrLowCaseInplace(string& s)
+{
+  unsigned len=s.length();
+//   char const *sCStr=s.c_str();
+  for(unsigned i=0;i<len;i++)
+    s[i]=tolower(s[i]);
+//     s[i]=tolower(sCStr[i]);
+}
 
 double Str2D( const char* cStart)
 {
@@ -246,18 +267,32 @@ void WordExp( string& s)
 {
 //cout << "WordExp  in: " << s << endl;
 #if !defined(__OpenBSD__)
+#ifndef _MSC_VER
 // esc whitespace
-       string sEsc;
-       for( int i=0; i<s.length(); ++i)
-       {
-               if( s[i] == ' ')
-                       sEsc += "\\ ";
-               else
-                       sEsc += s[i];
-       }
+// which is not already escaped
+//   string sEsc;
+//   for( int i=0; i<s.length(); ++i)
+//   {
+//     if( s[i] == ' ')
+// 	sEsc += "\\ ";
+//     else if( s[i] == '\\')
+//       {
+// 	if( (i+1)<s.length())
+// 	{
+// 	  if( s[i+1] == ' ')
+// 	  {
+// 	    sEsc += "\\ ";
+// 	    ++i;
+// 	  }
+// 	}	  
+//       }    
+//     else
+// 	sEsc += s[i];
+//   }
 //cout << "WordExp esc: " << sEsc << endl;
  wordexp_t p;
- int ok0 = wordexp( sEsc.c_str(), &p, 0);
+ int ok0 = wordexp( s.c_str(), &p, 0);
+//  int ok0 = wordexp( sEsc.c_str(), &p, 0);
  if( ok0 == 0)
    {
      if( p.we_wordc > 0)
@@ -268,6 +303,7 @@ void WordExp( string& s)
      wordfree( &p);
    }
 #endif
+#endif
 //cout << "WordExp out: " << s << endl;
 }
 
