@@ -56,7 +56,7 @@
 #include <vector>
 #include <valarray>
 #include <cassert>
-
+// #include <cstdio>
 
 // undef for releases (should not give diagnostics)
 // define for the CVS (where the default sizes can easily be adjusted)
@@ -155,26 +155,80 @@ extern DLong CpuTPOOL_MAX_ELTS;
 
 // convert something to string
 template <typename T>
-inline std::string i2s( T i, SizeT w = 0)      
+inline std::string i2s( T i, SizeT w)// = 0)      
 {
   std::ostringstream os;
   os.width(w);
   os << i;
   return os.str();
 }
+template <typename T>
+inline std::string i2s( T i)      
+{
+  std::ostringstream os;
+  assert( os.width() == 0);
+  os << i;
+  return os.str();
+}
+
+// debug 
+//#include <iostream>
 
 // searches IDList idL for std::string s, returns its position, -1 if not found
 inline int FindInIDList(IDList& idL,const std::string& s)
 {
-  int ix=0;
-  for(IDList::iterator i=idL.begin();
-      i != idL.end(); ++i, ++ix) if( *i==s) {
-	return ix;
+//   int ix=0;
+  for(IDList::iterator i=idL.begin(); i != idL.end(); ++i)//, ++ix) 
+    if( *i==s) 
+      {
+	return i - idL.begin();
       }
 
   return -1;
 }
 
+// as auto_ptr is obsoleted Guard offers an alternative
+template <class T>
+class Guard
+{
+private:
+  T*      guarded;
+  
+public:
+  Guard(): guarded( NULL)
+  {}
+  Guard( T* c): guarded( c)
+  {}
+  
+  void Init( T* iniGuarded) // saves a call to delete
+  {
+    assert( guarded == NULL);
+    guarded = iniGuarded;
+  }  
+  void Reset( T* newGuarded)
+  {
+    delete guarded;
+    guarded = newGuarded;
+  }  
+  void Release()
+  {
+    guarded = NULL;
+  }  
+  T* Get()
+  {
+    return guarded;
+  }  
+  bool IsNull()
+  {
+    return guarded == NULL;
+  }
+  
+  ~Guard()
+  {
+    delete guarded;
+  }
+};
+
 // like auto_ptr but for arrays (delete[] is used upon destruction)
 template <class T>
 class ArrayGuard
@@ -193,6 +247,10 @@ public:
     delete guarded;
     guarded = newGuarded;
   }  
+  void Release()
+  {
+    guarded = NULL;
+  }  
 
   ~ArrayGuard()
   {
@@ -205,8 +263,8 @@ template <class T>
 class StackGuard
 {
 private:
-  T&     container;
-  SizeT  cSize;
+  T& container;
+  typename T::size_type cSize;
   
 public:
   StackGuard( T& c): container( c)
@@ -216,7 +274,7 @@ public:
   
   ~StackGuard()
   {
-    for( SizeT s=container.size(); s > cSize; s--)
+    for( typename T::size_type s=container.size(); s > cSize; s--)
       {
 	delete container.back();
 	container.pop_back();
@@ -225,6 +283,7 @@ public:
 };
 
 // needed for exceptions
+// does not delete elements 
 template <class T>
 class StackSizeGuard
 {
@@ -406,13 +465,14 @@ public:
 
   T& operator[]( SizeT ix) throw()
   {
-   if( ix >= sz)
+    // if( ix >= sz) 
     assert( ix < sz);
     return buf[ ix];
   }
   const T& operator[]( SizeT ix) const throw()
   {
-	assert( ix < sz);
+//     if( ix >= sz) // debug 
+      assert( ix < sz);
     return buf[ ix];
   }
 
@@ -422,7 +482,7 @@ void InitFrom( const GDLArray& right )
 {
 // 	// assert( sz == right.size());
 // 	if ( sz != right.size() )
-// 		ThrowGDLException ( "GDLArray::operator= operands have not same size (this: " + i2s ( sz ) +", right: " + i2s ( right.size() ) + ")");;
+// 		ThrowGDLException ( "GDLArray::operator= operands have not same size (this: " + i2s ( sz ) +", right: " + i2s ( right.size() ) + ")");
 	assert( &right != this);
  	assert ( sz == right.size() );
     std::memcpy(buf,right.buf,sz*sizeof(T));
@@ -431,7 +491,7 @@ void InitFrom( const GDLArray& right )
 GDLArray& operator= ( const GDLArray& right )
 {
 // 	if ( sz != right.size() )
-// 		ThrowGDLException ( "GDLArray::operator= operands have not same size (this: " + i2s ( sz ) +", right: " + i2s ( right.size() ) + ")");;
+// 		ThrowGDLException ( "GDLArray::operator= operands have not same size (this: " + i2s ( sz ) +", right: " + i2s ( right.size() ) + ")");
 
     assert( this != &right);
 	assert( sz == right.size());
@@ -743,5 +803,52 @@ const int ExprListDefaultLength = 64;
 typedef PreAllocPListT<BaseGDL*, ExprListDefaultLength> ExprListT;
 typedef ExprListT::iterator ExprListIterT;
 
+// exception save usage of GSL types
+// you need to pass the gsl-object to guard and the gsl-clenaup (free) function
+// example usage (for gsl_matrix):
+//
+// gsl_matrix *matrix = gsl_matrix_alloc(p0->Dim(0), p0->Dim(0));
+//
+// GDLGuard< gsl_matrix> gsl_matrix_guard( matrix, gsl_matrix_free);
+// (of course no explicit call to the gsl-cleanup function must be done anymore)
+template< typename GSLType, typename cleanupReturnType=void>
+class GDLGuard
+{
+  GSLType* gslObject;
+  
+  cleanupReturnType (*gslDestructor)(GSLType*);
+  
+  GDLGuard() {}
+  
+public:
+  GDLGuard( void (*d)(GSLType*)): gslObject( NULL), gslDestructor(d) {}
+  GDLGuard( GSLType* o, cleanupReturnType (*d)(GSLType*)): gslObject( o), gslDestructor(d) {}
+  ~GDLGuard()
+  {
+    (*gslDestructor)( gslObject);
+  }
+  void Set( GSLType* o)
+  {
+    assert( gslObject == NULL);
+    gslObject = o;
+  }
+};
+
+// int fclose(...);
+typedef GDLGuard< FILE, int> FILEGuard;
+
+// class FILEGuard
+// {
+//   FILE* fp;
+//   
+//   FILEGuard() {}
+//   
+// public:
+//   FILEGuard( FILE* f): fp( f) {}
+//   ~FILEGuard()
+//   {
+//     fclose( fp);
+//   }
+// };
 
 #endif
