@@ -355,6 +355,125 @@ namespace lib {
 #endif
   }
 
+	// WIDGET CW_BGROUP
+	BaseGDL* widget_bgroup( EnvT* e)
+	{
+#ifndef HAVE_LIBWXWIDGETS
+    e->Throw("GDL was compiled without support for wxWidgets");
+#else
+		//SizeT nParam = e->NParam();
+		DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
+    WidgetIDT parentID = (*p0L)[0];
+
+		DStringGDL* names = e->GetParAs<DStringGDL>(1);
+
+    GDLWidget *widget = GDLWidget::GetWidget( parentID);
+
+		DLong xsize = -1;
+		static int xsizeIx = e->KeywordIx( "XSIZE");
+    e->AssureLongScalarKWIfPresent( xsizeIx, xsize);
+
+		DLong ysize = -1;
+		static int ysizeIx = e->KeywordIx( "YSIZE");
+    e->AssureLongScalarKWIfPresent( ysizeIx, ysize);
+
+    static int buttonuvalueIx = e->KeywordIx( "BUTTON_UVALUE");
+		DString buttonuvalue = "";
+    e->AssureStringScalarKWIfPresent(buttonuvalueIx, buttonuvalue);
+
+    static int uvalueIx = e->KeywordIx( "UVALUE");
+    BaseGDL* uvalue = e->GetKW( uvalueIx);
+    if( uvalue != NULL)
+      uvalue = uvalue->Dup();
+
+    static int labelIx = e->KeywordIx( "LABEL_TOP");
+    DString labeltop = "";
+    e->AssureStringScalarKWIfPresent( labelIx, labeltop);
+
+		GDLWidgetBGroup::BGroupMode mode = GDLWidgetBGroup::NORMAL;
+		static int modeIx = e->KeywordIx( "EXCLUSIVE");
+		if(e->KeywordSet( modeIx)){
+			mode = GDLWidgetBGroup::EXCLUSIVE;
+		}else{
+			modeIx = e->KeywordIx( "NONEXCLUSIVE");
+			if(e->KeywordSet(modeIx)){
+				mode = GDLWidgetBGroup::NONEXCLUSIVE;
+			}
+		}
+
+		GDLWidgetBGroup::BGroupReturn ret = GDLWidgetBGroup::RETURN_ID;
+		static int retIx = e->KeywordIx( "RETURN_INDEX");
+		if(e->KeywordSet( retIx)){
+			ret = GDLWidgetBGroup::RETURN_INDEX;
+		}else{
+			retIx = e->KeywordIx( "RETURN_NAME");
+			if(e->KeywordSet(retIx)){
+				ret = GDLWidgetBGroup::RETURN_NAME;
+			}
+		}
+
+		DLong rows = -1;
+		static int rowIx = e->KeywordIx("ROW");
+		e->AssureLongScalarKWIfPresent( rowIx, rows);
+
+		DLong cols = -1;
+		static int colIx = e->KeywordIx("COLUMN");
+		e->AssureLongScalarKWIfPresent( colIx, cols);
+
+    GDLWidgetBGroup* group = new GDLWidgetBGroup( parentID, names,
+																									uvalue, buttonuvalue,
+																									xsize, ysize, labeltop,
+																									rows, cols, mode, ret);
+    group->SetWidgetType("GROUP");
+
+    return new DLongGDL( group->WidgetID());
+
+#endif
+	}
+
+
+
+
+	// WIDGET_LIST
+	BaseGDL* widget_list( EnvT* e)
+	{
+#ifndef HAVE_LIBWXWIDGETS
+    e->Throw("GDL was compiled without support for wxWidgets");
+#else
+    DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
+    WidgetIDT parentID = (*p0L)[0];
+    GDLWidget *widget = GDLWidget::GetWidget( parentID);
+
+		DLong xsize = -1;
+		static int xsizeIx = e->KeywordIx( "XSIZE");
+    e->AssureLongScalarKWIfPresent( xsizeIx, xsize);
+
+		DLong ysize = -1;
+		static int ysizeIx = e->KeywordIx( "YSIZE");
+    e->AssureLongScalarKWIfPresent( ysizeIx, ysize);
+
+		static int valueIx = e->KeywordIx( "VALUE");
+    BaseGDL* value = e->GetKW( valueIx);
+    if( value != NULL)
+      value = value->Dup();
+
+    static int uvalueIx = e->KeywordIx( "UVALUE");
+    BaseGDL* uvalue = e->GetKW( uvalueIx);
+    if( uvalue != NULL)
+      uvalue = uvalue->Dup();
+
+		static int multipleIx = e->KeywordIx( "MULTIPLE");
+		bool multiple = e->KeywordSet( multipleIx);
+
+    DLong style = multiple ? wxLB_MULTIPLE : wxLB_SINGLE;
+    GDLWidgetList* list = new GDLWidgetList( parentID, uvalue, value,
+																						 xsize, ysize,
+																						 style);
+    list->SetWidgetType( "LIST");
+
+    return new DLongGDL( list->WidgetID());
+#endif
+	}
 
   // WIDGET_DROPLIST
   BaseGDL* widget_droplist( EnvT* e)
@@ -421,8 +540,14 @@ namespace lib {
     if( uvalue != NULL)
       uvalue = uvalue->Dup();
 
-    GDLWidgetText* text = new GDLWidgetText( parentID, uvalue, value, xsize);
- 
+		DLong edit = 0;
+		static int editableIx = e->KeywordIx("EDITABLE");
+		e->AssureLongScalarKWIfPresent( editableIx, edit);
+		bool editable = edit == 1;
+
+    GDLWidgetText* text = new GDLWidgetText( parentID, uvalue, value,
+    xsize, editable);
+
     text->SetWidgetType( "TEXT");
 
     return new DLongGDL( text->WidgetID());
@@ -675,9 +800,10 @@ namespace lib {
       // Call widget event handler routine
       if ( lasttop != top) {
 	caller = e->Caller();
-	e->Interpreter()->CallStack().pop_back();
+// ms: commented out to comply with new stack handling
+	// 	e->Interpreter()->CallStack().pop_back();
 	//	std::cout << "before delete e" << std::endl;
-	delete e;
+// 	delete e;
       }
 
       executeString2( caller, &istr);
@@ -822,7 +948,7 @@ namespace lib {
 
     if ( getuvalue) {
       BaseGDL** uvalueKW = &e->GetKW( getuvalueIx);
-      delete (*uvalueKW);
+      GDLDelete((*uvalueKW));
 
       BaseGDL *widval = widget->GetUvalue();
       //      *uvalueKW = widget->GetUvalue();
@@ -830,11 +956,11 @@ namespace lib {
 
       /*
       if ( *uvalueKW != NULL) {
-	if( (*uvalueKW)->Type() == STRING)
+	if( (*uvalueKW)->Type() == GDL_STRING)
 	  *uvalueKW = new DStringGDL( (*( DStringGDL*) (*uvalueKW))[0]);
-	if( (*uvalueKW)->Type() == LONG)
+	if( (*uvalueKW)->Type() == GDL_LONG)
 	  *uvalueKW = new DLongGDL( (*( DLongGDL*) (*uvalueKW))[0]);
-	if( (*uvalueKW)->Type() == STRUCT) {
+	if( (*uvalueKW)->Type() == GDL_STRUCT) {
 	  DStructGDL* s = static_cast<DStructGDL*>( *uvalueKW);
 	  //	  DStructGDL* parStruct = dynamic_cast<DStructGDL*>( *uvalueKW);
 	  cout << s->Desc()->Name() << endl;
@@ -846,7 +972,7 @@ namespace lib {
 
     if ( setuvalue) {
       BaseGDL* uvalue = e->GetKW( setuvalueIx);
-      if( uvalue->Type() == STRUCT) {
+      if( uvalue->Type() == GDL_STRUCT) {
 	//	cout << "Structure uvalue" << endl;
 	//DStructGDL* s1 = static_cast<DStructGDL*>( uvalue);
 	//cout << s1->Desc()->Name() << endl;
@@ -902,8 +1028,9 @@ namespace lib {
 	  istringstream istr(line+"\n");
 
 	  // Call SETV procedure
+// ms: commented out to comply with new stack handling
 	  EnvBaseT* caller = e->Caller();
-	  e->Interpreter()->CallStack().pop_back();
+// 	  e->Interpreter()->CallStack().pop_back();
 	  executeString2( caller, &istr);
 	}
 
@@ -930,7 +1057,7 @@ namespace lib {
 
     if ( getvalue) {
       BaseGDL** valueKW = &e->GetKW( getvalueIx);
-      delete (*valueKW);
+      GDLDelete((*valueKW));
 
       DString getFuncName = widget->GetFuncValue();
       if ( getFuncName != "") {
@@ -942,7 +1069,7 @@ namespace lib {
 	callF = StrUpCase( callF);
 
 	SizeT funIx = GDLInterpreter::GetFunIx( callF);
-	EnvUDT* newEnv= new EnvUDT( e, funList[ funIx], NULL);
+	EnvUDT* newEnv= new EnvUDT( e->CallingNode(), funList[ funIx], (BaseGDL**)NULL);
 
 	// add parameter
 	newEnv->SetNextPar( new DLongGDL(widgetID)); // pass as local
@@ -957,13 +1084,13 @@ namespace lib {
       } else {
 	// "Regular" getvalue
 	BaseGDL** valueKW = &e->GetKW( getvalueIx);
-	delete (*valueKW);
+	GDLDelete((*valueKW));
 
 	*valueKW = widget->GetVvalue();
 	if ( *valueKW != NULL) {
-	  if( (*valueKW)->Type() == STRING)
+	  if( (*valueKW)->Type() == GDL_STRING)
 	    *valueKW = new DStringGDL( (*( DStringGDL*) (*valueKW))[0]);
-	  if( (*valueKW)->Type() == LONG)
+	  if( (*valueKW)->Type() == GDL_LONG)
 	    *valueKW = new DLongGDL( (*( DLongGDL*) (*valueKW))[0]);
 	} else {
 	  DLongGDL* res = new DLongGDL( 0);
